/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.OSMObject;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.define.IOrmAction;
import com.nonesole.persistence2.sql.define.ISingleValueAction;
import com.nonesole.persistence2.sql.define.IVoidAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.basic.ICommon;
import com.nonesole.web.platform.basic.IPojo;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.pages.Page2Parameter;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;
import com.nonesole.web.platform.sys.rightset.RightSetEnum;
import com.nonesole.web.platform.sys.user.User;
import com.nonesole.web.platform.sys.user.UserEnum;

/**
 * This is the toolkit for business actions.
 * @author jack lee
 * @version 1.0
 * @date 2016-10-06
 */
public class ToolsAction {
		
	protected Logger logger;
	
	public ToolsAction(){
		logger = Logger.getInstance();
	}

	/**
	 * Check whether the relation is exist.
	 * @param response
	 * @param actionName
	 * @param objectsId
	 * @return
	 * @throws ExecuteSQLException
	 */
	protected boolean isExistRelation(HttpServletResponse response, String actionName, String objectsId) 
			throws ExecuteSQLException{
		ISingleValueAction sAction = PersistenceFactory.getInstance().getSingleValueAction();
		Map<String,Object> c = new HashMap<String,Object>();

		if ( objectsId == null ) {
			objectsId = "";
		}
		String[] idArray = objectsId.split(",");
		String[] idKeys = new String[idArray.length];
		StringBuilder condtionId = new StringBuilder();
		for (int i = 0; i < idArray.length; i++) {
			idKeys[i] = "#" + idArray[i] + "#";
			condtionId.append(idKeys[i]);
			if (i < idArray.length - 1) {
				condtionId.append(",");
			}
			c.put(idKeys[i], idArray[i]);
		}
		c.put(RightSetEnum.PARAM_CONDITION_ID.toString(), condtionId.toString());
		
		Integer row = sAction.intAction(actionName, c);
		if ( row == null || row < 1 ) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * Batch delete.
	 * @param conn
	 * @param actionName
	 * @param objects
	 * @param objectsId
	 * @throws ExecuteSQLException 
	 */
	protected void batchDelete(Connection conn, List<?> objects) 
			throws ExecuteSQLException{
		IOrmAction action = PersistenceFactory.getInstance().getOrmAction();
		DefaultBaseAction.commitConnection(conn, false);
		action.batchDelete(objects);
	}
	
	/**
	 * 带级联的批量删除
	 * Batch delete relations.
	 * @param conn
	 * @param actionName
	 * @param objects
	 * @param objectsId
	 * @throws ExecuteSQLException 
	 */
	protected void batchDeleteRelation(Connection conn, String actionName, String objectsId) 
			throws ExecuteSQLException{		
		IVoidAction iAction = PersistenceFactory.getInstance().getVoidAction();
		DefaultBaseAction.commitConnection(conn, false);
		Map<String,Object> c = new HashMap<String,Object>();
		c.put(RightSetEnum.PARAM_ID_ARRAY.toString(), objectsId);
		iAction.voidAction(actionName, c);
		DefaultBaseAction.commitConnection(conn, true);
	}
	
	/**
	 * 返回数据库链接
	 * Get default connection
	 * @param request
	 * @param response
	 * @param url 出错后跳转的页面URL
	 * @return
	 */
	protected Connection getDefaultConnection(HttpServletRequest request, HttpServletResponse response, String url){
		Connection conn = null;
		try {
			conn = PersistenceFactory.getInstance().getDefaultConnection();
			//conn = MultiConnectionPool.getInstance().getConnection(MultiConnectionPool.getInstance().getDefaultDataSourceName());
		} catch (OperationsException e) {
			Logger.getInstance().error("Connection is error.", e);
			this.errorRedirect(request, response, url, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.CommonAction_info_1"));
		} return conn;
	}
	
	/**
	 * Redirect the wrong url when the error happens.
	 * @param response
	 * @param url
	 * @param message
	 */
	protected void errorRedirect(HttpServletRequest request, HttpServletResponse response, String url, String message) {
		PlatformTools.setCookieMessage(response, message);
		try {
			response.sendRedirect(url);
		} catch (IOException e) {
			this.errorRedirect(request, response, ParameterEnum.URL_WELCOME.toString(), 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.CommonAction_info_2"));
		}
	}
	
	/**
	 * Redirect the url at the end.
	 * @param response
	 * @param url
	 * @param message
	 */
	protected void redirect(HttpServletRequest request, HttpServletResponse response, String url) {
		try {
			response.sendRedirect(url);
		} catch (IOException e) {
			this.errorRedirect(request, response, ParameterEnum.URL_WELCOME.toString(), 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.CommonAction_info_2"));
		}
	}
	
	/**
	 * Transform array to String.
	 * @param array
	 * @return
	 */
	protected String array2string(String[] array){
		if ( array != null ) {
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < array.length; i++ ) {
				if (PlatformTools.isNumber(array[i])) {
					// 过滤非数字选项
					sb.append(array[i]);
					if (i < (array.length - 1)) {
						sb.append(",");
					}
				}
			}
			return sb.toString();
		}
		return null;
	}
	
	/**
	 * Transform array to String.
	 * @param array
	 * @return
	 */
	protected String array2str(String[] array) {
		StringBuilder sb = new StringBuilder();
		if ( array != null ) {
			for ( String id : array ) {
				sb.append(id).append(",");
			}
			String str = sb.toString();
			str = str.substring(0, str.length()-1);
			return str;
		}
		return "";
	}
	
	/**
	 * Get array from request.
	 * @param parameter
	 * @param request
	 * @param response
	 * @return
	 * @throws IOException
	 */
	protected String[] getArrayParameter(String parameter, 
			HttpServletRequest request) 
			throws IOException{
		if ( parameter != null && request != null ) {
			String[] array = request.getParameterValues(parameter);
			if ( array == null || array.length < 1 ) {
				return null;
			}
			return array;
		}
		return null;
	}
	
	/**
	 * Get array from request and transform the array to map.
	 * @param actionName
	 * @param request
	 * @return Map
	 */
	protected Map<String,Object> parameter2map(String actionName, HttpServletRequest request) {
		if ( request != null ) {
			Action action = ConfigLoader.getInstance().getAction(actionName);
			if ( action != null ) {
				Map<String,Object> map = new HashMap<String,Object>();
				List<String> params = action.getParameters();
				if ( params != null ) {
					String value = null;
					Class<?> clazz = null;
					for ( String param : params ) {
						value = request.getParameter(param);
						clazz = action.getParameterClass(param);
						if ( clazz != null ) {
							if ( action.isNumberType(clazz) ) {
								if ( value == null || "".equals(value) || !PlatformTools.isNumber(value) ) {
									value = "0";
								} 
								if ( Double.class == clazz || double.class == clazz ) {
									map.put(param, Double.valueOf(value));
								} else if ( Integer.class == clazz || int.class == clazz ) {
									map.put(param, Integer.valueOf(value));
								} else if ( Short.class == clazz || short.class == clazz ) {
									map.put(param, Short.valueOf(value));
								} else if ( Float.class == clazz || float.class == clazz ) {
									map.put(param, Float.valueOf(value));
								} else if ( Long.class == clazz || long.class == clazz ) {
									map.put(param, Long.valueOf(value));
								} 	
							} else {
								map.put(param, value);
							}
						} 						
					}
					return map;
				}
			}
		}
		return null;
	}
	
	/**
	 * Get values from request and transform the values to Object.
	 * @param actionName
	 * @param request
	 * @return Map
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 * @throws OperationsException 
	 * @throws ClassNotFoundException 
	 */
	protected Object parameter2orm(Object object, HttpServletRequest request) 
	throws ClassNotFoundException, OperationsException {
		if ( request != null && object != null) {
			OSMObject osm = ConfigLoader.getInstance().getObjectSqlMapping(object.getClass().getName());
			if ( osm != null ) {
				Class<?> objectClazz = Class.forName(osm.getClassName());
				Field[] fields = objectClazz.getDeclaredFields();
				if ( fields != null ) {
					String value = null;
					for ( Field field : fields ) {
						int modifier = field.getModifiers();
						if ( Modifier.isPrivate(modifier) && 
								!Modifier.isStatic(modifier) && 
								!Modifier.isFinal(modifier) ) {
							value = request.getParameter(field.getName());
							PlatformTools.setFieldValue(field.getName(), field.getType(), object, value);
						} 
					}
				}
				return object;
			}
		}
		return null;
	}
	
	/**
	 * Get values from request and transform the values to Object by using the configuration of some Action object.
	 * @param actionName
	 * @param request
	 * @return Map
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 * @throws OperationsException 
	 */
	protected Object parameter2object(String actionName, Object object, HttpServletRequest request) 
	throws InstantiationException, IllegalAccessException, OperationsException {
		if ( request != null && object != null && actionName != null && !"".equals(actionName) ) {
			Action action = ConfigLoader.getInstance().getAction(actionName);
			if ( action != null ) {
				List<String> params = action.getParameters();
				if ( params != null ) {
					String value = null;
					Class<?> clazz = null;
					for ( String param : params ) {
						value = request.getParameter(param);
						clazz = action.getParameterClass(param);
						if ( clazz != null ) {
							PlatformTools.setFieldValue(param, clazz, object, value);
						} 						
					}
				}
				return object;
			}
		}
		return null;
	}
		
	/**
	 * Get condition like 'id in (#1#,#2#,#3#)' from request
	 * @param parameter 用户提交的参数名
	 * @param fieldName 数据库中字段名
	 * @param values 用于拼预编译SQL的参数
	 * @param request Request请求对象
	 * @return 用于预编译的SQL片段
	 * @throws IOException
	 */
	protected String ids2string(
			String parameter, String fieldName, 
			Map<String,Object> values, HttpServletRequest request) throws IOException{
		String[] idArray = this.getArrayParameter(parameter, request);
		if ( idArray == null ) {
			return "";
		}
		StringBuilder sb = new StringBuilder();
		sb.append(" ").append(fieldName).append(" in (");
		String tempKey;
		for (int i = 0; i < idArray.length; i++ ) {
			if (PlatformTools.isNumber(idArray[i])) {
				tempKey = parameter+idArray[i];
				sb.append("#").append(tempKey).append("#");
				values.put("#"+tempKey+"#", idArray[i]);
				if (i < (idArray.length - 1)) {
					sb.append(",");
				}
			}
		}
		sb.append(") ");
		return sb.toString();
	}
	
	/**
	 * Get the primary key array from request and build the object with these keys.
	 * @param parameter 用户提交的参数名
	 * @param setIdMethodName id的Set方法名
	 * @param clazz 对象类型
	 * @param request
	 * @return 对象集合
	 * @throws Exception
	 */
	protected List<Object> ids2objects(String parameter, String setIdMethodName, 
			Class<?> clazz, HttpServletRequest request) throws Exception{
		String[] idArray = request.getParameterValues(parameter);
		List<Object> objects = null;
		if ( idArray != null ) {
			objects = new ArrayList<Object>();
			Object obj = null;
			Method m = null;
			for (String id : idArray) {
				if (PlatformTools.isNumber(id)) {
					obj = clazz.newInstance();
					m = clazz.getMethod(setIdMethodName, new Class[]{int.class});
					m.invoke(obj, new Object[]{Integer.valueOf(id)});
					objects.add(obj);
				}
			}
		}
		return objects;
	}
	
	/**
	 * Get values from request and transform the values to IPOJO object.
	 * @param action
	 * @param clazz
	 * @param request
	 * @return
	 * @throws CoreException
	 */
	protected IPojo getObject(int action, Class<?> clazz, HttpServletRequest request) 
			throws Exception {
		Object obj = clazz.newInstance();
		if ( !(obj instanceof IPojo) ) {
			throw new Exception("The input object is wrong.");
		}
		IPojo pojo = (IPojo)obj;
		this.parameter2orm(obj, request);
		if ( !pojo.validate(action) ) {
			throw new Exception(clazz.getSimpleName() + " object has wrong fields.Please check it.");
		}
		return pojo;
	}

	/**
	 * Get IOrmAction object
	 * @return IOrmAction
	 */
	protected IOrmAction getAction(){
		return PersistenceFactory.getInstance().getOrmAction();
	}
	
	/**
	 * Get the login user
	 * @param request
	 * @return 
	 */
	protected User getLoginUser(HttpServletRequest request){
		HttpSession session = request.getSession();
		Object obj = session.getAttribute(UserEnum.ATTR_SESSION_USER.toString());
		if ( obj != null && (obj instanceof User) ) {
			return (User)obj;
		}
		return null;
	}
		
	/**
	 * Build the Page2Parameter object.
	 * @param request
	 * @param column
	 * @return
	 */
	protected Page2Parameter createKeyWordParam(HttpServletRequest request, String column){
		return new Page2Parameter()
				.setRequest(request)
				.setKey(ICommon.PARAM_QUERY_KEY_WORD)
				.setClazz(String.class)
				.setSign(Page2Parameter.LIKE_ALL)
				.setColumn(column)
				.initValue();
	}
	
	/**
	 * Build the Page2Parameter object.
	 * @param request
	 * @param column
	 * @return
	 */
	protected Page2Parameter createIsPrivateParam(HttpServletRequest request){
		return new Page2Parameter()
				.setRequest(request)
				.setKey(ICommon.PARAM_QUERY_IS_PRIVATE)
				.setClazz(Integer.class)
				.setSign(Page2Parameter.EQUAL)
				.setColumn(ICommon.DAO_PARAM_IS_PRIVATE)
				.initValue();
	}

	/**
	 * Build the Page2Parameter object.
	 * @param request
	 * @param column
	 * @return
	 */
	protected Page2Parameter createStatusParam(HttpServletRequest request){
		return new Page2Parameter()
				.setRequest(request)
				.setKey(ICommon.DAO_PARAM_STATUS)
				.setClazz(Integer.class)
				.setSign(Page2Parameter.EQUAL)
				.setColumn(ICommon.DAO_PARAM_STATUS)
				.initValue();
	}
	
	/**
	 * Check whether the relation is exist.
	 * @param actionName 
	 * @param parameter 
	 * @param sqlPart 
	 * @param url 
	 * @param request 
	 * @param response
	 * @throws ExecuteSQLException
	 */
	protected boolean checkRelation(String actionName, String parameter, 
			String sqlPart, String url,
			HttpServletRequest request, HttpServletResponse response) 
			throws ExecuteSQLException{
		String[] idArray = request.getParameterValues(parameter);
		if ( idArray != null ) {
			Map<String,Object> values = new HashMap<String,Object>();
			String[] idKeys = new String[idArray.length];
			StringBuilder condtionId = new StringBuilder();
			for (int i = 0; i < idArray.length; i++) {
				if ( PlatformTools.isNumber(idArray[i]) ) {
					idKeys[i] = "#" + idArray[i] + "#";
					condtionId.append(idKeys[i]);
					if (i < idArray.length - 1) {
						condtionId.append(",");
					}
					values.put(idKeys[i], idArray[i]);
				}
				
			}
			values.put(sqlPart, condtionId.toString());
			Integer row = PersistenceFactory.getInstance()
			                  				.getSingleValueAction()
			                  				.intAction(actionName, values);
			if ( row != null && row > 0 ) {
				this.errorRedirect(request, response, url, "Can not delete because there are relations exist.");
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Check whether the child node is exist.
	 * @param actionName 
	 * @param parameter 
	 * @param sqlPart 
	 * @param url 
	 * @param request 
	 * @param response
	 * @throws ExecuteSQLException
	 */
	protected boolean checkChildNode(String actionName, String parameter, 
			String sqlPart, String url,
			HttpServletRequest request, HttpServletResponse response) 
			throws ExecuteSQLException{
		String[] idArray = request.getParameterValues(parameter);
		if ( idArray != null ) {
			StringBuilder sbc = new StringBuilder();
			sbc.append(" where pid in(")
			   .append(this.array2str(idArray))
			   .append(")");
			Map<String,Object> values = new HashMap<String,Object>();
			values.put(sqlPart, sbc.toString());
			Integer row = PersistenceFactory.getInstance()
      				.getSingleValueAction()
      				.intAction(actionName, values);
			if ( row != null && row > 0 ) {
				this.errorRedirect(request, response, url, "There are child nodes exist, it can not be delete.");
				return false;
			}
			return true;
		}
		return false;
	}
	

	/**
	 * Put user id into the map.
	 * @param values Map<String, Object>
	 * @param request
	 */
	protected void setUserId(Map<String, Object> values, HttpServletRequest request){
		if ( values == null ) {
			values = new HashMap<String, Object>();
		}
		values.put(ICommon.DAO_PARAM_USER_ID, this.getLoginUser(request).getId());
	}
	
	/**
	 * Put object id into the map.
	 * @param values Map<String, Object>
	 * @param request
	 */
	protected void setObjectId(Map<String, Object> values, HttpServletRequest request){
		if ( values == null ) {
			values = new HashMap<String, Object>();
		}
		Integer id = PlatformTools.getIntParameter(request, ICommon.DAO_PARAM_ID);
		values.put(ICommon.DAO_PARAM_ID, id==null?0:id);
	}
}
