/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.nonesole.log.Logger;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * 多杈树
 * TreeNode
 * @author jack lee
 * @version 1.0
 * @date 2016-10-11
 */
public class TreeNode<T> implements Serializable, Comparable<TreeNode<T>> {

	private static final long serialVersionUID = 1L;
	protected int id; 
	protected int parentId;  
    protected String nodeName;  
    protected T obj;  
    protected TreeNode<T> parentNode;  
    protected List<TreeNode<T>> childList;  
    protected int level = -1;
    protected List<Integer> parentsId;
    protected List<Integer> childrenId;
    protected boolean selected = false;
    protected String url;
    protected String ico;

	/**
     * Constructor
     */
    public TreeNode() {  
        initChildList();  
    }  
  
    /**
     * Constructor
     * @param parentNode
     */
    public TreeNode(TreeNode<T> parentNode) {  
        this.getParentNode();  
        initChildList();  
    }  
  
    /**
     * Check whether current node is leaf.
     * @return
     */
    public boolean isLeaf() {  
        if (childList == null) {  
            return true;  
        } else {  
            if (childList.isEmpty()) {  
                return true;  
            } else {  
                return false;  
            }  
        }  
    }  
    
    /**
     * Check whether child nodes are leaves.
     * @return
     */
    public boolean isChildrenLeaf() {  
        if (childList == null) {  
            return true;  
        } else {
        	boolean r = true;
        	for ( TreeNode<T> node : this.childList ) {
        		r = r && node.isLeaf();
        	}
        	return r;
        }  
    }
  
    /**
     * Add child node
     * @param treeNode
     */
    public void addChildNode(TreeNode<T> treeNode) {  
        initChildList();  
        childList.add(treeNode);  
    }  
  
    /**
     * Initialize child
     */
    public void initChildList() {  
        if (childList == null)  
            childList = new ArrayList<TreeNode<T>>();  
    }  
  
    /**
     * Check whether current node is valid.
     * @return
     */
    public boolean isValidTree() {  
        return true;  
    }  
    
    /**
     * Get ancestors node
     * @return
     */
    public List<TreeNode<T>> getAncestors() {  
        List<TreeNode<T>> ancestors = new ArrayList<TreeNode<T>>();
        if ( this.getParentNode() != null ) {
        	ancestors.add(this.getParentNode());
        	ancestors.addAll(this.getParentNode().getAncestors());
        }
        Collections.sort(ancestors);//从小到大排序
        this.level = ancestors.size() + 1;
        return ancestors;
    }  
    
    /**
	 * @return the level
	 */
	public int getLevel() {
		if ( level == -1 ) {
			this.getAncestors();
		}
		return level;
	}

	/**
     * Get all child nodes.
     * @return
     */
    public List<TreeNode<T>> getAllChildren() {  
        List<TreeNode<T>> juniorList = new ArrayList<TreeNode<T>>();  
        List<TreeNode<T>> childList = this.getChildList();  
        if (childList == null) {  
            return juniorList;  
        } else {  
        	Collections.sort(childList);//从小到大排序
            int childNumber = childList.size();  
            for (int i = 0; i < childNumber; i++) {  
                TreeNode<T> junior = childList.get(i);  
                juniorList.add(junior);  
                juniorList.addAll(junior.getAllChildren());  
            }  
            return juniorList;  
        }  
    }  
  
    /**
     * Get child nodes
     * @return
     */
    public List<TreeNode<T>> getChildList() {  
        return childList;  
    }  
  
    /**
     * Delete node
     */
    public void deleteNode() {  
        TreeNode<T> parentNode = this.getParentNode();  
        int id = this.getId();  
  
        if (parentNode != null) {  
            parentNode.deleteChildNode(id);  
        }  
    }  
  
    /**
     * Delete child node of current node by id.
     * @param chidlId
     */
    public void deleteChildNode(int childId) {  
        List<TreeNode<T>> childList = this.getChildList();  
        int childNumber = childList.size();  
        for (int i = 0; i < childNumber; i++) {  
            TreeNode<T> child = childList.get(i);  
            if (child.getId() == childId) {  
                childList.remove(i);  
                return;  
            }  
        }  
    }  
  
    /**
     * Insert a node into the tree.
     * @param treeNode
     * @return
     */
    public boolean insertJuniorNode(TreeNode<T> treeNode) {  
        int juniorParentId = treeNode.getParentId();  
        if (this.parentId == juniorParentId) {  
            addChildNode(treeNode);  
            return true;  
        } else {  
            List<TreeNode<T>> childList = this.getChildList();  
            int childNumber = childList.size();  
            boolean insertFlag;  
  
            for (int i = 0; i < childNumber; i++) {  
                TreeNode<T> childNode = childList.get(i);  
                insertFlag = childNode.insertJuniorNode(treeNode);  
                if (insertFlag == true)  
                    return true;  
            }  
            return false;  
        }  
    }  
    
    /**
     * Find node by id.
     * @param id
     * @return
     */
    public TreeNode<T> findTreeNodeById(int id) {  
        if (this.id == id)  
            return this;  
        if (childList.isEmpty() || childList == null) {  
            return null;  
        } else {  
            int childNumber = childList.size();  
            for (int i = 0; i < childNumber; i++) {  
                TreeNode<T> child = childList.get(i);  
                TreeNode<T> resultNode = child.findTreeNodeById(id);  
                if (resultNode != null) {  
                    return resultNode;  
                }  
            }  
            return null;  
        }  
    }  
  
    /**
     * Traverse a tree
     */
    public void traverse() {  
        if (id < 0)  
            return;  
        print(this.id);  
        if (childList == null || childList.isEmpty())  
            return;  
        int childNumber = childList.size();  
        for (int i = 0; i < childNumber; i++) {  
            TreeNode<T> child = childList.get(i);  
            child.traverse();  
        }  
    }  
  
    public void print(String content) {  
        System.out.println(content);  
    }  
  
    public void print(int content) {  
        System.out.println(String.valueOf(content));  
    }  
  
    /**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the parentId
	 */
	public int getParentId() {
		return parentId;
	}

	/**
	 * @param parentId the parentId to set
	 */
	public void setParentId(int parentId) {
		this.parentId = parentId;
	}

	/**
	 * @return the nodeName
	 */
	public String getNodeName() {
		return nodeName;
	}

	/**
	 * @param nodeName the nodeName to set
	 */
	public void setNodeName(String nodeName) {
		this.nodeName = nodeName;
	}

	/**
	 * @return the parentNode
	 */
	public TreeNode<T> getParentNode() {
		return parentNode;
	}

	/**
	 * @param parentNode the parentNode to set
	 */
	public void setParentNode(TreeNode<T> parentNode) {
		this.parentNode = parentNode;
	}

	/**
	 * @return
	 */
	public T getObj() {  
        return obj;  
    }  
  
	/**
	 * @param obj
	 */
    public void setObj(T obj) {  
        this.obj = obj;  
    }

    /**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * @param url the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}
	
	@Override
	public int compareTo(TreeNode<T> node) {
		if ( node == null ) {
			return 1;
		}
		return node==null?1:this.id>node.getId()?1:(this.id==node.getId())?0:-1;
	}  
	
	/**
	 * @return the selected
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * @param selected the selected to set
	 * @deprecated
	 */
	protected void setRelationSelected(boolean selected) {
		this.selected = selected;
	}
	
	/**
	 * @param selected the selected to set
	 */
	public void setSelected() {
		Logger.getInstance().debug("TreeNode set selected " + this.nodeName, null);
		this.selected = true;
	}
	
	/**
	 * Check whether the select status is validate.
	 * @param selected the selected to set
	 */
	public void checkSelectedValidate() {
		if ( this.isSelected() ) {
			this.setParentSelected();
			if ( !this.isLeaf() ) {
				this.setChildrenSelected();
			}
		}
	}
	
	/**
	 * Update parent
	 * @param selected the selected to set
	 */
	public void updateParent() {
		if ( this.isSelected() ) {
			this.setParentSelected();
		}
	}
	
	/**
	 * Update children
	 * @param selected the selected to set
	 */
	public void updateChildren() {
		if ( this.isSelected() ) {
			if ( !this.isLeaf() ) {
				this.setChildrenSelected();
			}
		}
	}
	
	/**
	 * Set parent nodes selected
	 * @param selected
	 */
	private void setParentSelected() {
		if( this.parentNode != null && !this.parentNode.isSelected() ) {
			this.parentNode.setSelected();
		}
	}
	
	/**
	 * Set child nodes selected
	 * @param selected
	 */
	private void setChildrenSelected() {
		if ( !this.isLeafSelected() ) {
			if ( childList != null && !childList.isEmpty() ) {
				for ( TreeNode<T> node : childList ) {
					node.setSelected();
				}
			} 
		}
	}
	
	/**
	 * Check whether the leaves of current node are selected.
	 * @param selected
	 */
	private boolean isLeafSelected() {
		boolean isChildrenSelected = false;
		if ( childList != null && !childList.isEmpty() ) {
			for ( TreeNode<T> node : childList ) {
				if ( !node.isLeaf() ) {
					isChildrenSelected = isChildrenSelected || node.isLeafSelected();
				} else {
					isChildrenSelected = isChildrenSelected || node.isSelected();
				}
				Logger.getInstance().debug("isLeafSelected : " + node.getNodeName() + " : " + node.isSelected(), null);
			}
		} 
		return isChildrenSelected;
	}
	
	/**
	 * 返回被选中节点的内置对象
	 * Get the object of selected nodes.
	 * @return
	 */
	public List<T> getSelectedObjects(){
		List<T> objects = new ArrayList<T>();
		if( this.selected ) {
			objects.add(this.getObj());
			List<TreeNode<T>> children = this.getAllChildren();
			if ( children != null ) {
				for ( TreeNode<T> node : children ) {
					if ( node.isSelected() ) {
						objects.add(node.getObj());
					}
				}
			}
			List<TreeNode<T>> parent = this.getAncestors();
			if ( parent != null ) {
				for ( TreeNode<T> node : parent ) {
					if ( node.isSelected() ) {
						objects.add(node.getObj());
					}
				}
			}
		}
		return objects;
	}
	
	/**
	 * 返回被选中节点的内置对象
	 * Get the object of selected leaf-nodes.
	 * @return
	 */
	public List<T> getSelectedLeafObjects(){
		List<T> objects = new ArrayList<T>();
		if( this.selected ) {
			objects.add(this.getObj());
			List<TreeNode<T>> children = this.getAllChildren();
			if ( children != null ) {
				for ( TreeNode<T> node : children ) {
					if ( node.isSelected() && node.isLeaf() ) {
						objects.add(node.getObj());
					}
				}
			}
		}
		return objects;
	}
	
	/**
	 * 返回置顶菜单
	 * Get html of top menu
	 * @param node
	 * @return
	 */
	public String getTopMenuHtml(){
		StringBuilder sb = new StringBuilder();
		if ( this.isLeaf() ) {
			return "";
		} else {
			if ( this.isChildrenLeaf() ) {
				sb.append("<li>")
				  .append("<a href=\"#\" onclick=\"toMenu(").append(this.id).append(")\">").append("\r\n");
				sb.append(this.getIcoHtml(this));
				sb.append(this.nodeName).append("</a></li>").append("\r\n");
			} else {
				sb.append("<li role=\"presentation\" class=\"dropdown\">").append("\r\n")
				  .append("<a class=\"dropdown-toggle ft2\" data-toggle=\"dropdown\" href=\"#\" role=\"button\" aria-expanded=\"false\" onclick=\"toMenu(").append(this.id).append(")\">").append("\r\n");
				sb.append(this.getIcoHtml(this));
				sb.append(this.nodeName).append("\r\n")
				  .append("<span class=\"caret\"></span>").append("\r\n")
				  .append("</a>").append("\r\n")
				  .append("<ul class=\"dropdown-menu\" role=\"menu\">").append("\r\n");
				Collections.sort(this.childList);//从小到大排序
				for ( TreeNode<T> child : this.childList ) {
					sb.append(child.getTopMenuHtml()).append("\r\n").append("\r\n");
				}
				sb.append("</ul>").append("\r\n")
				  .append("</li>").append("\r\n");
			}
			
		}
		return sb.toString();
	}
	/*
	<li role="presentation" class="dropdown">
		<a class="dropdown-toggle ft2" data-toggle="dropdown" href="#" role="button" aria-expanded="false" onclick="toMenu(1)" >
			系统设置
			<span class="caret"></span>
		</a>
		<ul class="dropdown-menu" role="menu">
			<li><a href="#">插件管理</a></li>
		</ul>
	</li>
	 */
	
	/**
	 * 返回左侧1菜单
	 * Get html of left menu
	 * @param node
	 * @return
	 */
	public String getLeftMenuHtml(){
		StringBuilder sb = new StringBuilder();
		if ( !this.isLeaf() ) {
			boolean hasValidateChild = false;
			sb.append("<ul class=\"nav navbar-nav navbar-left\" id=\"menu").append(this.id).append("\" name=\"leftmenu\">").append("\r\n");
			Collections.sort(this.childList);//从小到大排序
			for ( TreeNode<T> child : this.childList ) {
				if ( child.isLeaf() ) {
					sb.append("<li class=\"leaf-menu-item\">").append("\r\n")
					  .append("<a onclick=\"toUrl('").append(PlatformTools.getUrlForRoot(child.getUrl())).append("')\" href=\"#\">").append("\r\n");
					sb.append(this.getIcoHtml(child));
					sb.append("&nbsp;").append(child.getNodeName()).append("\r\n")
					  .append("</a></li>").append("\r\n");
					hasValidateChild = true;
				}
			}
			sb.append("</ul>");
			if ( !hasValidateChild ) {
				return "";
			}
		}
		return sb.toString();
	}
	
	
	/*
	<ul class="nav navbar-nav navbar-right" id="menu-1" name="leftmenu">
		<li class="leaf-menu-item">
			<a onclick="toUrl('sys/users.jsp')" href="#">
              <span class="glyphicon glyphicon-user" aria-hidden="true"></span>
              &nbsp;用户管理
              </a>
		</li>
    </ul>
	 */
	

	/**
	 * @return the ico
	 */
	public String getIco() {
		return ico;
	}

	/**
	 * @param ico the ico to set
	 */
	public void setIco(String ico) {
		this.ico = ico;
	}
	
	/**
	 * 返回ICO的HTML代码
	 * Get icon html
	 * @return
	 */
	protected String getIcoHtml(TreeNode<T> node){
		if ( node != null && node.getIco() != null && !"".equals(node.getIco()) ) {
			StringBuilder sb = new StringBuilder();
			sb.append("<img src=\"").append(PlatformTools.getUrlForRoot(node.getIco())).append("\" width=\"16\" height=\"16\" />").append("\r\n");
			return sb.toString();
		}
		return "";
	}
	
}
