/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.access;

import javax.servlet.http.HttpServletRequest;

import com.nonesole.web.platform.basic.IPojo;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * POJO of Access
 * @author jack lee
 * @version 1.0
 */
public class AccessInfo implements IPojo {

	private Integer id;
	private String ip;
	private Long vtime;
	private String source_url;
	private String target_url;
	private String country;
	private String source_domain;
	private String target_domain;
	private String browser;
	private String language;
	private String os;
	private String session_id;
	private Integer user_id;
	private String user_agent;
	private Long ltime;
	private Integer new_visitor;
	private Integer mobile;
	private Integer source_type;
	private String search_engine;
	
	/**
	 * @return the source_type
	 */
	public Integer getSource_type() {
		return source_type;
	}

	/**
	 * @return the source_name
	 */
	public String getSourceName(HttpServletRequest request) {
		if ( this.source_type == 0 ) {
			return LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessInfo_info_1");
		} else if ( this.source_type == 1 ) {
			return LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessInfo_info_2");
		} else if ( this.source_type == 2 ) {
			return LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessInfo_info_3") + "("+this.search_engine+")";
		}
		return LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessInfo_info_4");
	}
	
	/**
	 * @param source_type the source_type to set
	 */
	public void setSource_type(Integer source_type) {
		this.source_type = source_type;
	}

	/**
	 * @return the search_engine
	 */
	public String getSearch_engine() {
		if ( search_engine != null && search_engine.length() > 20 ) {
			return user_agent.substring(0, 19);
		}
		return search_engine;
	}

	/**
	 * @param search_engine the search_engine to set
	 */
	public void setSearch_engine(String search_engine) {
		this.search_engine = search_engine;
	}

	/**
	 * @return the new_visitor
	 */
	public Integer getNew_visitor() {
		return new_visitor;
	}

	/**
	 * @param new_visitor the new_visitor to set
	 */
	public void setNew_visitor(Integer new_visitor) {
		this.new_visitor = new_visitor;
	}

	/**
	 * @return the mobile
	 */
	public Integer getMobile() {
		return mobile;
	}

	/**
	 * @param mobile the mobile to set
	 */
	public void setMobile(Integer mobile) {
		this.mobile = mobile;
	}

	/**
	 * @return the ltime
	 */
	public Long getLtime() {
		return ltime;
	}

	/**
	 * @param ltime the ltime to set
	 */
	public void setLtime(Long ltime) {
		this.ltime = ltime;
	}

	/**
	 * @return the userAgent
	 */
	public String getUser_agent() {
		if ( user_agent != null && user_agent.length() > 200 ) {
			return user_agent.substring(0, 199);
		}
		return user_agent;
	}

	/**
	 * @param userAgent the userAgent to set
	 */
	public void setUser_agent(String user_agent) {
		this.user_agent = user_agent;
	}

	/**
	 * @return the session_id
	 */
	public String getSession_id() {
		return session_id;
	}

	/**
	 * @param session_id the session_id to set
	 */
	public void setSession_id(String session_id) {
		this.session_id = session_id;
	}

	/**
	 * @return the user_id
	 */
	public Integer getUser_id() {
		return user_id;
	}

	/**
	 * @param user_id the user_id to set
	 */
	public void setUser_id(Integer user_id) {
		this.user_id = user_id;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(Integer id) {
		this.id = id;
	}

	/**
	 * @param vtime the vtime to set
	 */
	public void setVtime(Long vtime) {
		this.vtime = vtime;
	}

	public String toString(){
		return PlatformTools.getNow() + " " + PlatformTools.isNull(ip) != null?"NoIP":ip;
	}
	
	/**
	 * @param action
	 * @return
	 */
	public boolean validate(int action) {
		if ( action == IAccess.INSERT ) {
			if ( PlatformTools.isNull(ip) || vtime <= 0 ) {
				return false;
			} else {
				return true;
			}
		}
		return false;
	}
		
	/**
	 * @return the id
	 */
	public Integer getId() {
		return id;
	}
	/**
	 * @return the ip
	 */
	public String getIp() {
		return ip;
	}
	/**
	 * @param ip the ip to set
	 */
	public void setIp(String ip) {
		this.ip = ip;
	}
	/**
	 * @return the vtime
	 */
	public Long getVtime() {
		return vtime;
	}
	/**
	 * @return the source_url
	 */
	public String getSource_url() {
		if ( source_url != null && source_url.length() > 2000 ) {
			return source_url.substring(0, 1999);
		}
		return source_url;
	}
	/**
	 * @param source_url the source_url to set
	 */
	public void setSource_url(String source_url) {
		this.source_url = source_url;
	}
	/**
	 * @return the target_url
	 */
	public String getTarget_url() {
		if ( target_url != null && target_url.length() > 2000 ) {
			return target_url.substring(0, 1999);
		}
		return target_url;
	}
	/**
	 * @param target_url the target_url to set
	 */
	public void setTarget_url(String target_url) {
		this.target_url = target_url;
	}
	/**
	 * @return the country
	 */
	public String getCountry() {
		return country;
	}
	/**
	 * @param country the country to set
	 */
	public void setCountry(String country) {
		this.country = country;
	}
	/**
	 * @return the source_domain
	 */
	public String getSource_domain() {
		if ( source_domain != null && source_domain.length() > 100 ) {
			return source_domain.substring(0, 99);
		}
		return source_domain;
	}
	/**
	 * @param source_domain the source_domain to set
	 */
	public void setSource_domain(String source_domain) {
		this.source_domain = source_domain;
	}
	/**
	 * @return the target_domain
	 */
	public String getTarget_domain() {
		if ( target_domain != null && target_domain.length() > 100 ) {
			return target_domain.substring(0, 99);
		}
		return target_domain;
	}
	/**
	 * @param target_domain the target_domain to set
	 */
	public void setTarget_domain(String target_domain) {
		this.target_domain = target_domain;
	}
	/**
	 * @return the browser
	 */
	public String getBrowser() {
		if ( browser != null && browser.length() > 20 ) {
			return browser.substring(0, 19);
		}
		return browser;
	}
	/**
	 * @param browser the browser to set
	 */
	public void setBrowser(String browser) {
		this.browser = browser;
	}
	/**
	 * @return the language
	 */
	public String getLanguage() {
		return language;
	}
	/**
	 * @param language the language to set
	 */
	public void setLanguage(String language) {
		this.language = language;
	}
	/**
	 * @return the os
	 */
	public String getOs() {
		if ( os != null && os.length() > 20 ) {
			return os.substring(0, 19);
		}
		return os;
	}
	/**
	 * @param os the os to set
	 */
	public void setOs(String os) {
		this.os = os;
	}

	@Override
	public int getObjectId() {
		return id;
	}
	
	
}
