/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.access;

import java.io.BufferedReader;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.sql.tools.FileHandler;
import com.nonesole.web.platform.ActionControler;
import com.nonesole.web.platform.ParameterEnum;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;
import com.nonesole.web.platform.sys.user.User;

/**
 * This class is used to statistics access data.
 * @author jack lee
 * @version 1.0
 */
public class AccessStatistics implements Filter {

	public static boolean reloadSet = false;
	public static String OPEN = "open";
	public static String CLOSE = "close";
	public static String switchValue = null;
	public static boolean open = true;
	public static List<String> resources = null;
	public static List<String> osKeys = null;
	public static Map<String,String> osMap = null;
	public static List<String> browserKeys = null;
	public static Map<String,String> browserMap = null;
	public static List<String> mobileKeys = null;
	public static Map<String,String> mobileMap = null;
	public static List<String> searchEngineKeys = null;
	public static Map<String,String> searchEngineMap = null;
	public static List<String> clientKeys = null;
	public static Map<String,String> clientMap = null;
		
	private Logger logger = null;
	private ActionControler controler = null;
	
	private void initIPSearch(){
		String url = IP.class.getClassLoader().getResource("17monipdb.dat").getPath();
        IP.load(url);
	}
	
	/**
	 * @return URI
	 */
	public static URI getSetURI(){
		URI uri = null;
		try {
			uri = AccessStatistics.class.getClassLoader().getResource("access.set").toURI();
			Logger.getInstance().debug("access.set url :" + uri.toString(), null);
		} catch (URISyntaxException e) {
			Logger.getInstance().error("Can not get resource access.set", e);
		}
		return uri;
	}
	
	/**
	 * @param uri
	 * @return
	 */
	public static BufferedReader getSetFileStream(URI uri){
		String charset = "utf-8";
		BufferedReader br = null;
		try {
			br = FileHandler.getInstance().loadFileToBufferedReader(uri,charset);
		} catch (IOException e) {
			Logger.getInstance().error("Can not load file access.set", e);
		}
		return br;
	}
	
	/**
	 * @return
	 */
	public static String getTag(String line){
		line = line.trim();
		if ( IAccess.SET_RESOURCE.equals(line) ) {
			return IAccess.SET_RESOURCE;
		} else if ( IAccess.SET_OS.equals(line) ) {
			return IAccess.SET_OS;
		} else if ( IAccess.SET_BROWSER.equals(line) ) {
			return IAccess.SET_BROWSER;
		} else if ( IAccess.SET_MOBILE.equals(line) ) {
			return IAccess.SET_MOBILE;
		} else if ( IAccess.SET_SEARCH_ENGINE.equals(line) ) {
			return IAccess.SET_SEARCH_ENGINE;
		} else if ( IAccess.SET_SWITCH.equals(line) ) {
			return IAccess.SET_SWITCH;
		} else {
			return null;
		}
	}
	
	/**
	 * @param tag 
	 * @param line 
	 */
	public static void resource(String tag, String line){
		if ( IAccess.SET_RESOURCE.equals(tag) ) {
			resources.add(line.trim());
		}
	}
	
	/**
	 * @param tag
	 * @param line
	 */
	public static void switchValue(String tag, String line){
		if ( IAccess.SET_SWITCH.equals(tag) ) {
			switchValue = null;
			if ( OPEN.equals(line) ) {
				switchValue = OPEN;
				open = true;
			} else if ( CLOSE.equals(line) ) {
				switchValue = CLOSE;
				open = false;
			} 
		}
	}
	
	/**
	 * @param tag 
	 * @param line 
	 */
	public static void os(String tag, String line){
		if ( IAccess.SET_OS.equals(tag) ) {
			String[] keyValue = line.trim().split("=");
			if ( keyValue.length == 2 ) {
				osKeys.add(keyValue[0]);
				osMap.put(keyValue[0], keyValue[1]);
			}
		}
	}
		
	/**
	 * @param tag 
	 * @param line 
	 */
	public static void browser(String tag, String line){
		if ( IAccess.SET_BROWSER.equals(tag) ) {
			String[] keyValue = line.trim().split("=");
			if ( keyValue.length == 2 ) {
				browserKeys.add(keyValue[0]);
				browserMap.put(keyValue[0], keyValue[1]);
			}
		}
	}
	
	/**
	 * @param tag 
	 * @param line 
	 */
	public static void mobile(String tag, String line){
		if ( IAccess.SET_MOBILE.equals(tag) ) {
			String[] keyValue = line.trim().split("=");
			if ( keyValue.length == 2 ) {
				mobileKeys.add(keyValue[0]);
				mobileMap.put(keyValue[0], keyValue[1]);
			}
		}
	}
	
	/**
	 * @param tag 
	 * @param line 
	 */
	public static void searchEngine(String tag, String line){
		if ( IAccess.SET_SEARCH_ENGINE.equals(tag) ) {
			String[] keyValue = line.trim().split("=");
			if ( keyValue.length == 2 ) {
				searchEngineKeys.add(keyValue[0]);
				searchEngineMap.put(keyValue[0], keyValue[1]);
			}
		}
	}
	
	/**
	 * @return
	 */
	public static String linkSearchEngine(){
		if ( searchEngineKeys == null || searchEngineKeys.isEmpty() ) {
			return "";
		} else {
			StringBuilder sb = new StringBuilder();
			for ( String s : searchEngineKeys ) {
				sb.append(s).append("=").append(searchEngineMap.get(s)).append("\r\n");
			}
			return sb.toString();
		}
	}
	
	/**
	 * @return
	 */
	public static String linkBrowser(){
		if ( browserKeys == null || browserKeys.isEmpty() ) {
			return "";
		} else {
			StringBuilder sb = new StringBuilder();
			for ( String s : browserKeys ) {
				sb.append(s).append("=").append(browserMap.get(s)).append("\r\n");
			}
			return sb.toString();
		}
	}
	
	/**
	 * @return
	 */
	public static String linkMobile(){
		if ( mobileKeys == null || mobileKeys.isEmpty() ) {
			return "";
		} else {
			StringBuilder sb = new StringBuilder();
			for ( String s : mobileKeys ) {
				sb.append(s).append("=").append(mobileMap.get(s)).append("\r\n");
			}
			return sb.toString();
		}
	}
	
	/**
	 * @return
	 */
	public static String linkOS(){
		if ( osKeys == null || osKeys.isEmpty() ) {
			return "";
		} else {
			StringBuilder sb = new StringBuilder();
			for ( String s : osKeys ) {
				sb.append(s).append("=").append(osMap.get(s)).append("\r\n");
			}
			return sb.toString();
		}
	}
	
	/**
	 * @return
	 */
	public static String linkResource(){
		if ( resources == null || resources.isEmpty() ) {
			return "";
		} else {
			StringBuilder sb = new StringBuilder();
			for ( String s : resources ) {
				sb.append(s).append("\r\n");
			}
			return sb.toString();
		}
	}
	
	public static void initSet(){
		if ( !reloadSet ) {
			return;
		}
		Logger logger = Logger.getInstance();
		reloadSet = false;
		
		resources = new ArrayList<String>();
		osKeys = new ArrayList<String>();
		osMap = new HashMap<String,String>();
		browserKeys = new ArrayList<String>();
		browserMap = new HashMap<String,String>();
		mobileKeys = new ArrayList<String>();
		mobileMap = new HashMap<String,String>();
		searchEngineKeys = new ArrayList<String>();
		searchEngineMap = new HashMap<String,String>();
		
		clientKeys = new ArrayList<String>();
		clientMap = new HashMap<String,String>();
		
		clientKeys.add("0");
		clientKeys.add("1");
		
		clientMap.put("0", "PC");
		clientMap.put("1", "Mobile");
		
		URI uri = AccessStatistics.getSetURI();
		BufferedReader br = AccessStatistics.getSetFileStream(uri);
		
    	String line = null;
    	String tag = null;
    	String tmp = null;
    	
    	try {
			while( null != (line = br.readLine()) ) {
				if ( !PlatformTools.isNull(line.trim()) ) {
					tmp = AccessStatistics.getTag(line);
					if ( tmp == null ) {
						AccessStatistics.resource(tag, line);
						AccessStatistics.os(tag, line);
						AccessStatistics.browser(tag, line);
						AccessStatistics.mobile(tag, line);
						AccessStatistics.searchEngine(tag, line);
						AccessStatistics.switchValue(tag, line);
					} else {
						tag = tmp;
					}
				}
			}
		} catch (IOException e) {
			logger.error("Can not read line from access.set", e);
			return;
		}
    	try {
			br.close();
		} catch (IOException e) {
			logger.error("Can not close BufferedReader.", e);
		}
    	br = null;
	}
	
	@Override
	public void destroy() {
		logger.debug("Destroy access statistics.", null);
	}

	@Override
	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {
		if ( !open ) {
			chain.doFilter(request, response);
			return;
		}
		
		initSet();
		
		String encoding = "utf-8";
		request.setCharacterEncoding(encoding); 
		response.setContentType("text/html;charset="+encoding); 
		
		HttpServletRequest httpRequest = (HttpServletRequest) request;
		HttpServletResponse httpResponse = (HttpServletResponse) response;
		String path = httpRequest.getServletPath();
		
		if ( this.isResource(path) ) {	
			AccessInfo info = new AccessInfo();
			String userAgent = PlatformTools.htmlTransform(httpRequest.getHeader("User-agent"));
			
			info.setVtime(System.currentTimeMillis());
			info.setIp(this.getIpAddr(httpRequest));
			info.setCountry(this.getSimpleCountry(Arrays.toString(IP.find(info.getIp()))));
			info.setLanguage(httpRequest.getLocale().getLanguage());
			info.setUser_agent(userAgent);
			info.setBrowser(this.getBrowser(userAgent));
			info.setOs(this.getOS(userAgent));
			info.setSource_url(this.getReferer(httpRequest));
			info.setSource_domain(this.getSourceDomain(info.getSource_url()));
			info.setTarget_url(httpRequest.getServletPath());
			info.setTarget_domain(this.getTargetDomain(httpRequest));
			this.setSessionId(httpRequest, httpResponse, info);
			info.setUser_id(this.getUserId(httpRequest));
			info.setLtime(0l);
			info.setMobile(this.getMobile(userAgent));
			this.getSource(userAgent, info);
			
			logger.debug("accessStat: doAction", null);
			request.setAttribute(IAccess.BEAN_NAME, info);
			request.setAttribute(ParameterEnum.ACTION.toString(), IAccess.FUNCTION_ADD);
			try {
				controler.doAction(httpRequest, httpResponse);
			} catch (CoreException e) {
				logger.error("AccessStatistics", e);
				chain.doFilter(request, response);
				return;
			}
			
			//This is the first solution but not the best
			//this.writeJavascript(httpResponse);
			
			//This is the second solution which is better
			int stime = 60 * 60;
			String accesspath = httpRequest.getContextPath() + info.getTarget_url();
			PlatformTools.setCookie(httpResponse, "accessid", String.valueOf(info.getId()), stime, accesspath);
			PlatformTools.setCookie(httpResponse, "vtime", String.valueOf(info.getVtime()), stime, accesspath);
			PlatformTools.setCookie(httpResponse, "accesspath", info.getTarget_url(), stime, accesspath);
		}
		chain.doFilter(request, response);
	}

	@Override
	public void init(FilterConfig arg0) throws ServletException {
		controler = ActionControler.getInstance();
		logger = Logger.getInstance();
		
		this.initIPSearch();
		
		reloadSet = true;
		initSet();
	}
	
	/**
	 * Insert ajax code into the head of JSP or html for updating leave time
	 * @param httpRequest
	 * @param response
	 * @param info 
	 * @throws IOException 
	 */
	protected void writeJavascript(HttpServletRequest httpRequest, 
			ServletResponse response, AccessInfo info) throws IOException{
		//下面这段代码需注入到客户端方可将离开页面的时间采集并发送到服务器端，如果用户直接关闭界面，则无法完成时间采集
//		<script>
//		var vtime = 1486099876870;
//		var sessionId = "1F03E06683910276C5CB8BC338034D23";
//		var targetUrl = "/login.jsp";
//
//		var vd = new Date();
//		var clientVtime = vd.getTime();
//
//			window.onbeforeunload = function (e) {
//		        e = e || window.event;
//
//		        var ld = new Date();
//		        var clientLtime = ld.getTime();
//		        var ltime = vtime + (clientLtime - clientVtime);
//		        
//		        var xmlhttp;
//		        if (window.XMLHttpRequest){
//		        	// code for IE7+, Firefox, Chrome, Opera, Safari
//		        	xmlhttp=new XMLHttpRequest();
//		        } else {
//		        	// code for IE6, IE5
//					xmlhttp=new ActiveXObject("Microsoft.XMLHTTP");
//		        }
//		        var url = "<%=request.getContextPath()%>"+"/LeaveServlet";
//		        xmlhttp.open("POST",url,true);
//		        xmlhttp.setRequestHeader("Content-type","application/x-www-form-urlencoded");
//		        xmlhttp.send("vtime="+vtime+"&ltime="+ltime+"&session_id="+sessionId+"&target_url="+targetUrl);
//		        
//		        console.log("vtime="+vtime+"&ltime="+ltime+"&session_id="+sessionId+"&target_url="+targetUrl);
//		        
//		        //if (e) {
//		        //  e.returnValue = window.location.hostname;
//		        //}
//
//		        // Chrome, Safari, Firefox 4+, Opera 12+ , IE 9+
//		        //return window.location.hostname;
//		        return;
//		      };
//
//		</script>
//		
		
			StringBuilder sb = new StringBuilder();
			sb.append("<script>").append("\r\n")
			  .append("var id = ").append(info.getId()).append(";").append("\r\n")
			  .append("var vtime = ").append(info.getVtime()).append(";").append("\r\n")
			  .append("var vd = new Date();").append("\r\n")
			  .append("var clientVtime = vd.getTime();").append("\r\n")
			  .append("window.onbeforeunload = function (e) {").append("\r\n")
			  .append("var ld = new Date();").append("\r\n")
			  .append("var clientLtime = ld.getTime();").append("\r\n")
			  .append("var ltime = vtime + (clientLtime - clientVtime);").append("\r\n")
			  .append("var xmlhttp;").append("\r\n")
			  .append("if (window.XMLHttpRequest){").append("\r\n")
			  .append("xmlhttp=new XMLHttpRequest();").append("\r\n")
			  .append("} else {").append("\r\n")
			  .append("xmlhttp=new ActiveXObject('Microsoft.XMLHTTP');").append("\r\n")
			  .append("}").append("\r\n")
			  .append("var url = '").append(httpRequest.getContextPath()).append("'+'/LeaveServlet';").append("\r\n")
			  .append("xmlhttp.open('POST',url,true);").append("\r\n")
			  .append("xmlhttp.setRequestHeader('Content-type','application/x-www-form-urlencoded');").append("\r\n")
			  .append("xmlhttp.send('id='+id+'&ltime='+ltime);").append("\r\n")
			  .append("return;").append("\r\n")
			  .append("}").append("\r\n")
			  .append("</script>");
		
		response.getWriter().write(sb.toString());
	}
	
	/**
	 * @param request
	 * @return
	 */
	private boolean isResource(String path){
		if ( resources.isEmpty() || PlatformTools.isNull(path) ) {
			return false;
		}
		for ( String res : resources ) {
			if ( path.endsWith(res) ) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @param request
	 * @return
	 */
	private String getIpAddr(HttpServletRequest request) {
		String ip = request.getHeader("x-forwarded-for");  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getHeader("Proxy-Client-IP");  
        }  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getHeader("WL-Proxy-Client-IP");  
        }  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getHeader("HTTP_CLIENT_IP");  
        }  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");  
        }  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getRemoteAddr();  
        }
		if ( ip.equals("0:0:0:0:0:0:0:1") ) {
			ip = "127.0.0.1";
		}
		return ip;
	}
	
	/**
	 * @param locale
	 * @return
	 */
	private String getSimpleCountry(String locale) {
		if (locale == null) {
			return "";
		}
		String[] array = locale.substring(1, locale.length() - 1).split(", ");
		return array[1];
	}
	
	/**
	 * @param httpRequest
	 * @return
	 */
	private String getReferer(HttpServletRequest httpRequest){
		Object referer = null;
		if ( httpRequest.getHeader("Referer") == null || "null".equals(httpRequest.getHeader("Referer")) ) {
			referer = httpRequest.getSession().getAttribute("from_index_referer");
		} else {
			referer = httpRequest.getHeader("Referer");
		}
		return referer==null?"":(String)referer;
	}
	
	/**
	 * @param httpRequest
	 * @return
	 */
	private String getSourceDomain(String url){
		if ( PlatformTools.isNull(url) ) {
			return "";
		}
		if ( url.startsWith("http://") ) {
			url = url.substring(7);
		}
		return url.substring(0, url.indexOf("/"));
	}
	
	/**
	 * @param httpRequest
	 * @return
	 */
	private String getTargetDomain(HttpServletRequest httpRequest){
		StringBuffer url = httpRequest.getRequestURL();
		return url.delete(url.length() - httpRequest.getRequestURI().length(), url.length()).append("/").toString();
	}
	
	/**
	 * @param referer
	 * @return
	 */
	public String getOS(String referer) {
		String unknown = "Unknow";
		if ( referer == null || "".equals(referer) || osKeys.isEmpty() ) {
			return unknown;
		}
		String key = null;
		for ( int i = 0 ; i < osKeys.size() ; i++ ) {
			key = osKeys.get(i);
			if ( referer.toLowerCase().contains(key.toLowerCase()) ) {
				return osMap.get(key);
			}
		}
		return unknown;
	}
	
	/**
	 * @param referer
	 * @return
	 */
	public String getBrowser(String referer) {
		String unknown = "Unknow";
		if ( referer == null || "".equals(referer) || browserKeys.isEmpty() ) {
			return unknown;
		}
		String key = null;
		for ( int i = 0 ; i < browserKeys.size() ; i++ ) {
			key = browserKeys.get(i);
			if ( referer.toLowerCase().contains(key.toLowerCase()) ) {
				return browserMap.get(key);
			}
		}
		return unknown;
	}
	
	/**
	 * @param httpRequest
	 * @return
	 */
	private void setSessionId(HttpServletRequest httpRequest, HttpServletResponse response, AccessInfo info){
		String sid = PlatformTools.getCookie(httpRequest, "sid");
		if ( PlatformTools.isNull(sid) ) {
			info.setNew_visitor(1);
			sid = httpRequest.getSession().getId();
			PlatformTools.setCookie(response, "sid", sid, 30*24*60*60, true);
		}
		info.setNew_visitor(0);
		info.setSession_id(sid);
	}
	
	/**
	 * @param httpRequest
	 * @return
	 */
	private Integer getUserId(HttpServletRequest httpRequest){
		User user = PlatformTools.getUser(httpRequest);
		if ( user == null ) {
			return 0;
		} else {
			return user.getId();
		}
	}
	
	/**
	 * @param referer
	 * @return
	 */
	private Integer getMobile(String referer){		
		if ( referer == null || "".equals(referer) || mobileKeys.isEmpty() ) {
			return 0;
		}
		String key = null;
		for ( int i = 0 ; i < mobileKeys.size() ; i++ ) {
			key = mobileKeys.get(i);
			if ( referer.toLowerCase().contains(key.toLowerCase()) ) {
				return 1;
			}
		}
		return 0;
	}
	
	/**
	 * @param referer
	 * @return
	 */
	public void getSource(String referer, AccessInfo info) {
		info.setSearch_engine("");
		if ( PlatformTools.isNull(info.getSource_url()) ) {
			info.setSource_type(0);
			return;
		} else {
			if ( referer == null || "".equals(referer) || searchEngineKeys.isEmpty() ) {
				info.setSource_type(1);
				return;
			}
			String key = null;
			for ( int i = 0 ; i < searchEngineKeys.size() ; i++ ) {
				key = searchEngineKeys.get(i);
				if ( referer.toLowerCase().contains(key.toLowerCase()) ) {
					info.setSearch_engine(searchEngineMap.get(key));
					info.setSource_type(2);
					return;
				}
			}
			info.setSource_type(1);
		}
		
	}
}
