/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.access;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.web.platform.sys.CoreException;

/**
 * This is the interface of access statistics
 * @author jack lee
 * @version 1.0 - build on 2017-01-30
 */
public interface IAccess {

	/**
	 * Add
	 * @param request
	 * @param response	
	 * @throws CoreException
	 */
	public void add(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Set leave time
	 * @param request
	 * @param response	
	 * @throws CoreException
	 */
	public void setLeaveTime(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Source url analysis
	 * @param request
	 * @param response	
	 * @throws CoreException
	 */
	public void sourceAnalysis(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Target url analysis
	 * @param request
	 * @param response	
	 * @throws CoreException
	 */
	public void targetAnalysis(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Visitor analysis
	 * @param request
	 * @param response	
	 * @throws CoreException
	 */
	public void visitorAnalysis(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Save the configuration
	 * @param request
	 * @param response	
	 * @throws CoreException
	 */
	public void saveSetting(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Get access details
	 * @param request
	 * @param response	
	 * @throws CoreException
	 */
	public void getAccessInfoDetail(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException;

	public static final int INSERT = 1;
	public static final String ACCESS_ACTION = "access_action";
	public static final String FUNCTION_ADD = "access01";
	public static final String FUNCTION_SET_LEAVE_TIME  = "access02";
	public static final String FUNCTION_GET_SOURCE_DATA  = "access03";
	public static final String FUNCTION_GET_TARGET_DATA  = "access04";
	public static final String FUNCTION_VISITOR_DATA  = "access05";
	public static final String FUNCTION_SAVE_SETTING  = "access06";
	public static final String FUNCTION_GET_DETAIL  = "access07";
	public static final String DAO_FIND_ACCESS_INFO_DETAIL = "findAccessInfoDetail";
	public static final String DAO_SET_LEAVE_TIME = "setLeaveTime";
	public static final String DAO_FIND_SOURCE_BY_HOURS = "findAccessSourceByHours";
	public static final String DAO_FIND_SOURCE_BY_DAYS = "findAccessSourceByDays";
	public static final String DAO_FIND_SOURCE_DOMAIN = "findAccessSourceDomain";
	public static final String DAO_FIND_SOURCE_DIRECT = "findAccessSourceDirect";
	public static final String DAO_FIND_URL_TARGET = "findAccessUrlTarget";
	public static final String DAO_FIND_TOTAL_URL_TARGET = "findTotalAccessUrlTarget";
	public static final String DAO_FIND_SOURCE_SEARCH_ENGINE = "findAccessSourceSearchEngine";
	public static final String DAO_FIND_VISITOR_PROPERTY = "findVisitorInfo";
	public static final String DAO_FIND_SOURCE_TOTAL_UV = "findAccessSourceUv";
	public static final String DAO_ALL_ROWS = "getAllRowsOfAccessInfo";
	public static final String CONDITION = "#condition";
	public static final String SOURCE_DOMAIN = "source_domain";
	public static final String STAY_TIME = "stay_time";
	public static final String PV = "pv";
	public static final String TARGET_URL = "target_url";
	public static final String PV_PERCENT = "pv-percent";
	public static final String ONLY_ONE_PV = "opv";
	public static final String KEY_SEARCH_ENGINE_NAME = "search_engine_name";
	public static final String KEY_SOURCE_DOMAIN_NAME = "source_domain";
	public static final String ONLY_ONE_PV_PERCENT = "one-pv-percent";
	public static final String UV = "uv";
	public static final String FIRST_PV = "fpv";
	public static final String IP = "ips";
	public static final String PARAM_IP = "ip";
	public static final String ID = "id";
	public static final String TIME_FORMAT = "yyyy-MM-dd HH:mm:ss:SSS";
	public static final String BROWSER = "browser";
	public static final String BEAN_NAME = "AccessInfo";
	public static final String SOURCE_TYPE = "source_type";
	public static final String TIME = "time";
	public static final String NUMBER = "number";
	public static final String SET_SWITCH = "[switch]";
	public static final String SET_RESOURCE = "[resource]";
	public static final String SET_OS = "[os]";
	public static final String SET_BROWSER = "[browser]";
	public static final String SET_MOBILE = "[mobile]";
	public static final String SET_SEARCH_ENGINE = "[search_engine]";
	public static final String PARAM_SOURCE_DATE_TYPE = "dt";
	public static final String PARAM_RESOURCE_NAME = "resname";
	public static final int TODAY = 0;
	public static final int YESTODAY = -1;
	public static final int LAST_7_DAYS = 7;
	public static final int LAST_30_DAYS = 30;
	public static final String SOURCE_START_TIME = "stime";
	public static final String SOURCE_END_TIME = "etime";
	public static final String SWITCH_VALUE = "swithValue";
	public static final String RESOURCE = "resource";
	public static final String RESOURCE_SIZE = "resource_size";
	public static final String SEARCH_ENGINE = "search_engine";
	public static final String USER_AGENT = "user_agent";
	public static final String START_TIME = "stime";
	public static final String END_TIME = "etime";
	public static final String VIEW_TIME_FORMAT = "yyyy-MM-dd HH:mm";
	public static final String OS = "os";
	public static final String MOBILE = "mobile";
	public static final String COUNTRY = "country";
	public static final String LANGUAGE = "language";
	public static final String DS = "ds";
	public static final String PROP = "prop";
	public static final String NUM = "num";
	public static final String URL_DETAIL = "detail.jsp";
	public static final String URL_DETAIL_2 = "./access/detail.jsp";
	public static final String ATTR_ECHART_BEN = "attr_echart_bean";
	public static final String ATTR_TOTAL_DATA = "attr_total_data";
	public static final String ATTR_LINKS_DATA = "attr_links_data";
	public static final String ATTR_ENGINES_DATA = "attr_engines_data";
	public static final String ATTR_DIRECTS_DATA = "attr_directs_data";
	public static final String ATTR_TARGET_TOTAL_DATA = "attr_target_total_data";
	public static final String ATTR_LINKS_DETAIL = "attr_links_detail";
	public static final String ATTR_TARGET_DETAIL = "attr_links_detail";
	public static final String ATTR_ENGINES_DETAIL = "attr_engines_detail";
	public static final String ATTR_DIRECTS_DETAIL = "attr_directs_detail";
	public static final String ATTR_ACCESS_DETAIL = "attr_access_detail";
	public static final String VISIT_TIME = "vtime";
	public static final String ATTR_ACCESS_INFO = "attr_info_object";

}
