/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.basic;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IVoidAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.ToolsAction;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * Implementation class of IBasic
 * @author jack lee
 * @version 1.0 - build in 2017-04-10
 */
public class BasicAction extends ToolsAction implements IBasic{

	protected Logger logger;
	
	public BasicAction(){
		logger = Logger.getInstance();
	}
	
	@Override
	public Object save(Class<?> clazz, 
			HttpServletRequest request, HttpServletResponse response) throws CoreException {
		Object obj = null;
		try {
			obj = this.getObject(IBasic.INSERT, clazz, request);
			this.getAction().insert(obj);
		} catch (Exception e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
		PlatformTools.setCookieMessage(response, "Success!");
		return obj;
	}

	@Override
	public Object update(Class<?> clazz, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Object obj = null;
		try {
			obj = this.getObject(IBasic.UPDATE, clazz, request);
			this.getAction().update(obj);
		} catch (Exception e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
		PlatformTools.setCookieMessage(response, "Success!");
		return obj;
	}

	@Override
	public void delete(Class<?> clazz, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		try {
			Object obj = this.getObject(IBasic.DELETE, clazz, request);
			this.getAction().delete(obj);
		} catch (Exception e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
		PlatformTools.setCookieMessage(response, "Success!");
	}
		
	@Override
	public List<?> query(String actionName, Map<String, Object> values) throws CoreException {
		try {
			return PersistenceFactory.getInstance().getPojoAction().pojoListAction(actionName, values);
		} catch (Exception e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
	}
	
	@Override
	public Object findObjectById(String actionName, Integer id) throws CoreException {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("id", id);
		try {
			return PersistenceFactory.getInstance().getPojoAction().pojoAction(actionName, map);
		} catch (ExecuteSQLException e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
	}
	
	@Override
	public void deleteByIdArray(String parameter, Class<?> clazz,
			HttpServletRequest request,HttpServletResponse response) throws CoreException {
		try {
			List<Object> objects = this.ids2objects(parameter, SET_ID_METHOD_NAME, clazz, request);
			this.getAction().batchDelete(objects);
		} catch (Exception e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
		PlatformTools.setCookieMessage(response, "Success!");
	}
	
	@Override
	public void deleteByIdArray(String parameter, String actionName, 
			String fieldName, String sqlPart,
			HttpServletRequest request,HttpServletResponse response) throws CoreException {
		try {
			Map<String, Object> values = new HashMap<String, Object>();
			String ids = this.ids2string(parameter, fieldName, values, request);
			values.put(sqlPart, ids);
			PersistenceFactory.getInstance().getVoidAction().voidAction(actionName, values);
		} catch (Exception e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
		PlatformTools.setCookieMessage(response, "Success!");
	}
	
	/**
	 * Set status，open(value=1) or close(value=0)
	 * @param parameter Key of array in request
	 * @param filedId 
	 * @param idSqlPart 
	 * @param statusSqlPart
	 * @param actionName 
	 * @param request
	 * @param response
	 * @param status
	 * @throws CoreException
	 */
	public void setStatus(String parameter, String filedId, 
			String idSqlPart, String statusSqlPart, String actionName, 
			HttpServletRequest request, HttpServletResponse response, int status) throws CoreException {
		try {			
			Map<String, Object> c = new HashMap<String, Object>();
			String idsql = this.ids2string(parameter, filedId, c, request);
			if ( PlatformTools.isNull(idsql) ) {
				return;
			}
			if ( idSqlPart != null ) {
				c.put(idSqlPart, "where " + idsql);
			}
			if ( statusSqlPart != null ) {
				c.put(statusSqlPart, status);
			}
			
			IVoidAction acts = PersistenceFactory.getInstance().getVoidAction();
			acts.voidAction(actionName, c);
						
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.safe.wordblacklist.WordBlackListAction_info_5"));
		} catch (ExecuteSQLException | IOException e) {
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.safe.wordblacklist.WordBlackListAction_info_6"));
		} 
	}

	/**
	 * Get values from request
	 * @param request
	 * @return
	 */
	@SuppressWarnings("unchecked")
	protected Map<String, Object> getSqlValues(HttpServletRequest request){
		Object map = request.getAttribute(ATTR_SQL_VALUES);
		if ( map == null ) {
			map = new HashMap<String, Object>();
			request.setAttribute(ATTR_SQL_VALUES, map);
		}
		return (Map<String, Object>)map;
	}
	
	@Override
	public Object findObjectById(String actionName, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Map<String, Object> values = this.getSqlValues(request);
		this.setObjectId(values, request);
		try {
			return PersistenceFactory.getInstance().getPojoAction().pojoAction(actionName, values);
		} catch (ExecuteSQLException e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
	}

	@Override
	public Object save(String actionName, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Map<String, Object> values = this.getSqlValues(request);
		try {
			return PersistenceFactory.getInstance().getPojoAction().pojoAction(actionName, values);
		} catch (ExecuteSQLException e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
	}

	@Override
	public Object update(String actionName, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Map<String, Object> values = this.getSqlValues(request);
		try {
			return PersistenceFactory.getInstance().getPojoAction().pojoAction(actionName, values);
		} catch (ExecuteSQLException e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
	}
	
	@Override
	public void logicDelete(String actionName, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Map<String, Object> values = this.getSqlValues(request);
		try {
			PersistenceFactory.getInstance().getVoidAction().voidAction(actionName, values);
		} catch (ExecuteSQLException e) {
			throw new CoreException(e.fillInStackTrace()); 
		}
	}
		
}
