/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.basic;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.nonesole.web.platform.sys.CoreException;


/**
 * Base Interface
 * @author jack lee
 * @version 1.0
 */
public interface IBasic {

	public static final int INSERT = 1;
	public static final int UPDATE = 2;
	public static final int DELETE = 3;
	public static final int QUERY = 4;
	public static final String SET_ID_METHOD_NAME = "setId";
	public final static String ATTR_SQL_VALUES = "sql_values";
	
	/**
	 * Save
	 * @param clazz Class<?>
	 * @param request Request
	 * @param response Response
	 * @throws CoreException 
	 */
	public Object save(Class<?> clazz, 
			HttpServletRequest request, HttpServletResponse response) throws CoreException;
	
	/**
	 * Update
	 * @param clazz Class<?>
	 * @param request Request
	 * @param response Response
	 * @throws CoreException 
	 */
	public Object update(Class<?> clazz, 
			HttpServletRequest request, HttpServletResponse response) throws CoreException;
	
	/**
	 * Delete
	 * @param clazz Class<?>
	 * @param request Request
	 * @param response Response
	 * @throws CoreException 
	 */
	public void delete(Class<?> clazz, 
			HttpServletRequest request, HttpServletResponse response) throws CoreException;
	
	/**
	 * Delete objects by array of id
	 * @param parameter Key of array in request
	 * @param clazz 
	 * @param request 
	 * @param response
	 * @throws CoreException
	 */
	public void deleteByIdArray(String parameter, Class<?> clazz, 
			HttpServletRequest request, HttpServletResponse response) throws CoreException;
	
	/**
	 * Delete objects by array of id
	 * @param parameter Key of array in request
	 * @param actionName 
	 * @param fieldName id name
	 * @param sqlPart 
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void deleteByIdArray(String parameter, String actionName, 
			String fieldName, String sqlPart,
			HttpServletRequest request,HttpServletResponse response) throws CoreException;
			
	/**
	 * Query
	 * @param actionName
	 * @param values
	 * @throws CoreException
	 */
	public List<?> query(String actionName, Map<String, Object> values) throws CoreException;
	
	/**
	 * Find object by id
	 * @param actionName 
	 * @param id 
	 * @throws CoreException 
	 */
	public Object findObjectById(String actionName, Integer id) throws CoreException;
	
	/**
	 * Find object by id
	 * @param actionName 
	 * @param id 
	 * @throws CoreException 
	 */
	public Object findObjectById(String actionName, HttpServletRequest request, HttpServletResponse response) throws CoreException;
	
	/**
	 * Save
	 * @param actionName 
	 * @param id 
	 * @throws CoreException 
	 */
	public Object save(String actionName, HttpServletRequest request, HttpServletResponse response) throws CoreException;
	
	/**
	 * Update
	 * @param actionName 
	 * @param id 
	 * @throws CoreException 
	 */
	public Object update(String actionName, HttpServletRequest request, HttpServletResponse response) throws CoreException;
	
	/**
	 * Delete
	 * @param actionName 
	 * @param id 
	 * @throws CoreException 
	 */
	public void logicDelete(String actionName, HttpServletRequest request, HttpServletResponse response) throws CoreException;
		
}
