/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.i18n;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.IOException;
import java.net.URI;
import java.util.Locale;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import com.nonesole.web.platform.sys.PropertiesTools;

/**
 * @author jack lee
 * @version 1.0
 */
public class LocaleLanguageEditor extends JFrame {

	private static final long serialVersionUID = 1L;

	protected ComboxPanel langSelectedPanel = null;
	
	protected JTextArea text = null;
	
	public LocaleLanguageEditor(){
		super();
		Locale l = Locale.getDefault();  
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (ClassNotFoundException | InstantiationException
				| IllegalAccessException | UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}//windows风格
		
		this.setTitle(LocaleToolkit.getLocaleString(l, "com.nonesole.web.platform.i18n.LocaleLanguageEditor_info_1"));
		this.setSize(640, 480);
		this.getContentPane().setLayout(new BorderLayout());
		LocaleLanguageEditor.setToScreenCenter(this);
		JPanel pb = new JPanel();
		JPanel pc = new JPanel();
		pc.setLayout(new GridLayout(0,2));
		
		langSelectedPanel = new ComboxPanel(LocaleToolkit.getLocaleString(l, "com.nonesole.web.platform.i18n.LocaleLanguageEditor_info_2"),"","");
		langSelectedPanel.addListener(new ItemListener(){
			@Override
			public void itemStateChanged(ItemEvent e) {
				if ( e.getStateChange() == ItemEvent.SELECTED ) {
					String value = langSelectedPanel.getValue();
					if ( value != null ) {
						load(value);
					}
				}
			}
		});
		pc.add(langSelectedPanel);
		
		this.add(pc,BorderLayout.NORTH);
		
		//编辑器
		text = new JTextArea(10,20);
		Font textFont = new Font("宋体",Font.PLAIN,16);
		text.setFont(textFont);
		JScrollPane textPanel = new JScrollPane(text);
		this.add(textPanel,BorderLayout.CENTER);
		
		FlowLayout fl = new FlowLayout(FlowLayout.RIGHT);
		pb.setLayout(fl);
			
		JButton btnSure = new JButton(LocaleToolkit.getLocaleString(l, "com.nonesole.web.platform.i18n.LocaleLanguageEditor_info_3"));
		JButton btnCancel = new JButton(LocaleToolkit.getLocaleString(l, "com.nonesole.web.platform.i18n.LocaleLanguageEditor_info_4"));
		
		Font btnFont = new Font("宋体",Font.PLAIN,14);
		btnSure.setFont(btnFont);
		btnCancel.setFont(btnFont);
		
		pb.add(btnSure);
		pb.add(btnCancel);
		
		this.add(pb,BorderLayout.SOUTH);
		
		btnSure.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				save();
			}			
		});
		
		btnCancel.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}			
		});
		
		this.init();
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	
	/**
	 * 加载配置文件
	 * @param fileName
	 */
	protected void load(String fileName){
		Properties prop = new Properties();
		try {
			System.out.println(LocaleLanguageEditor.class
					.getResource(fileName).getPath());
			prop.load(LocaleLanguageEditor.class
					.getResourceAsStream(fileName));
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		Set<Entry<Object,Object>> ens = prop.entrySet();
		StringBuilder sb = new StringBuilder();
		for ( Entry<Object,Object> en : ens ) {
			sb.append(en.getKey())
			  .append("=")
			  .append(en.getValue().toString())
			  //.append(LocaleToolkit.unicode2String(en.getValue().toString()))
			  .append("\r\n");
		}
		text.setText(sb.toString());
	}
	
	/**
	 * 初始化
	 */
	protected void init(){
		Properties prop = new Properties();
		try {
			prop.load(LocaleLanguageEditor.class
					.getResourceAsStream("language.properties"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		Set<Entry<Object,Object>> ens = prop.entrySet();
		for ( Entry<Object,Object> en : ens ) {
			langSelectedPanel.addItem(
					en.getKey().toString(), en.getValue().toString());
		}
		langSelectedPanel.setSelectedIndex(0);
		load(langSelectedPanel.getValue());
	}
	
	/**
	 * 保存数据
	 */
	protected void save(){
		String fileName = langSelectedPanel.getValue();
		String s1 = LocaleToolkit.getLocaleString(Locale.getDefault(), "com.nonesole.web.platform.i18n.LocaleLanguageEditor_info_5");
		String s2 = LocaleToolkit.getLocaleString(Locale.getDefault(), "com.nonesole.web.platform.i18n.LocaleLanguageEditor_info_6");
		String s3 = LocaleToolkit.getLocaleString(Locale.getDefault(), "com.nonesole.web.platform.i18n.LocaleLanguageEditor_info_7");
		
		try {
			URI uri = LocaleLanguageEditor.class.getResource(fileName).toURI();
			//FileHandler.getInstance().saveFileFromString(uri, text.getText());
			String txt = text.getText(); 
			String[] lines = txt.split("\n");
			Properties prop = new Properties();
			String[] kv = null;
			for ( int i = 0 ; i < lines.length ; i++ ) {
				if ( lines[i] != null ) {
					kv = lines[i].split("=");
					if ( kv.length == 2 ) {
						prop.put(kv[0], kv[1]);
					}
				}
			}
			boolean result = PropertiesTools.saveUnicodeProperties(prop, uri);
			
			if ( result ) {
				JOptionPane.showMessageDialog(this,s1,s3,JOptionPane.INFORMATION_MESSAGE);
				load(fileName);
			} else {
				JOptionPane.showMessageDialog(this,s2,s3,JOptionPane.WARNING_MESSAGE);
			}
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this,s2,s3,JOptionPane.WARNING_MESSAGE);
			e.printStackTrace();
		}
	}
	
	/**
	 * 居中显示
	 * @param window
	 */
	public static void setToScreenCenter(Window window) {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = window.getSize();
		if (windowSize.height > screenSize.height)
			windowSize.height = screenSize.height;
		if (windowSize.width > screenSize.width)
			windowSize.width = screenSize.width;
		window.setLocation((screenSize.width - windowSize.width) / 2,
				(screenSize.height - windowSize.height) / 2);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		LocaleLanguageEditor ui = new LocaleLanguageEditor();
		ui.setVisible(true);
	}

}
