/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.i18n;

import java.io.IOException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import com.nonesole.log.Logger;

/**
 * Locale toolkit of platform
 * @author jack lee
 * @version 1.0
 */
public class LocaleToolkit {
	
	private final static String LANGUAGE_RESOURCE = "language.properties";
	private final static String DEFAULT = "default";
	private final static String DEFAULT_LANGUAGE = "en";
	private final static String BASE_RESOURCE = "com.nonesole.web.platform.i18n.messages";
	private static Map<String,ResourceBundle> bundles;
	
	static {
		bundles = new HashMap<String,ResourceBundle>();
		Properties prop = new Properties();
		try {
			prop.load(LocaleToolkit.class.getResourceAsStream(LANGUAGE_RESOURCE));
		} catch (IOException e) {
			Logger.getInstance().error("There is an error on load config for language.", e);
			//e.printStackTrace();
		}
		Locale locale = null;
		ResourceBundle messages = null;
		Set<Object> set = prop.keySet();
		for ( Object obj : set ) {
			locale = new Locale(obj.toString());
			messages = ResourceBundle.getBundle(BASE_RESOURCE, locale);
			bundles.put(obj.toString(), messages);
		}
		locale = new Locale(DEFAULT_LANGUAGE);
		messages = ResourceBundle.getBundle(BASE_RESOURCE, locale);
		bundles.put(DEFAULT, messages);
	}
	
	/**
	 * Get local string
	 * @param locale HttpServletRequest
	 * @param input key of string
	 * @return 
	 */
	public static String getLocaleString(HttpServletRequest request,String input) {
		Locale locale = request.getLocale();
		ResourceBundle messages = getBundle(locale);
		if ( messages != null ) {
			return messages.getString(input);
		} 
		return null;
	}
	
	/**
	 * Get local string
	 * @param locale 
	 * @param input 
	 * @return 
	 */
	public static String getLocaleString(Locale locale,String input) {
		ResourceBundle messages = getBundle(locale);
		if ( messages != null ) {
			return messages.getString(input);
		} 
		return null;
	}

	/**
	 * Get language resource
	 * @param locale
	 * @return
	 */
	public static ResourceBundle getBundle(Locale locale) {
		if ( locale != null ) {
			ResourceBundle messages = bundles.get(locale.getLanguage());
			if ( messages == null ) {
				messages = bundles.get(DEFAULT);
			}
			return messages;
		}
		return null;
	}

	/**
	 * Get parameter value and transformation to local string
	 * @param messages
	 * @param request
	 * @param parameter
	 * @return
	 */
	public static String getString(ResourceBundle messages,HttpServletRequest request,String parameter) {
		if ( messages != null && request != null ) {
			return messages.getString((String)request.getParameter(parameter));
		}
		return null;
	}
	
}
