/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.pages;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import com.nonesole.web.platform.sys.PlatformTools;

/**
 * Condition of Page object
 * @author jack lee
 * @version 1.0
 */
public class Condition {

	private String paramKey;
	private String paramValue;
	private String condition;
	private boolean empty = true;
	
	/**
	 * @param request
	 * @return
	 */
	public Condition makeCommonCondition(HttpServletRequest request){
		this.empty = true;
		this.condition = null;
		if ( PlatformTools.isNull(paramKey) || request == null ) {
			return this;
		}
		paramValue = PlatformTools.getRequestParameter(request, paramKey);
		if ( PlatformTools.isNull(paramValue) ) {
			return this;
		}
		StringBuilder sb = new StringBuilder();
		sb.append(" ").append(paramKey).append("= #").append(paramKey).append("# ");
		condition = sb.toString();
		this.empty = false;
		return this;
	}
	
	/**
	 * @param request
	 * @param tag
	 * @return
	 */
	public Condition makeSelfDefinedCondition(HttpServletRequest request, String conditionKey, String tag){
		this.empty = true;
		this.condition = null;
		if ( PlatformTools.isNull(paramKey) || request == null ) {
			return this;
		}
		paramValue = request.getParameter(paramKey);
		if ( PlatformTools.isNull(paramValue) ) {
			return this;
		}
		StringBuilder sb = new StringBuilder();
		sb.append(" ").append(conditionKey).append(" ").append(tag).append(" #").append(paramKey).append("# ");
		condition = sb.toString();
		this.empty = false;
		return this;
	}
	
	/**
	 * @param request
	 * @return
	 */
	public Condition makeLikeCondition(HttpServletRequest request){
		this.empty = true;
		this.condition = null;
		if ( PlatformTools.isNull(paramKey) || request == null ) {
			return this;
		}
		paramValue = request.getParameter(paramKey);
		if ( PlatformTools.isNull(paramValue) ) {
			return this;
		}
		StringBuilder sb = new StringBuilder();
		sb.append(" ").append(paramKey).append(" like #").append(paramKey).append("# ");
		condition = sb.toString();
		this.empty = false;
		return this;
	}
	
	/**
	 * @param where 
	 * @param pagesParameter 
	 * @param preparedValues 
	 * @return
	 */
	public Condition makePreparedSQL(StringBuilder where, 
			Map<String, String> pagesParameter, Map<String, Object> preparedValues){
		if ( !this.isEmpty() ) {
			if ( !pagesParameter.isEmpty() ) {
				where.append(" and ");
			} 
			where.append(this.getCondition());
			pagesParameter.put(this.getParamKey(), this.getParamValue());
			if ( PlatformTools.isNumber(this.getParamValue()) ) {
				if ( PlatformTools.isLong(this.getParamValue()) ) {
					preparedValues.put("#"+ this.getParamKey() +"#", Long.valueOf(this.getParamValue()));
				} else {
					preparedValues.put("#"+ this.getParamKey() +"#", Double.valueOf(this.getParamValue()));
				}
			} else {
				preparedValues.put("#"+ this.getParamKey() +"#", this.getParamValue());
			}
			
		}
		return this;
	}
	
	/**
	 * @param where 
	 * @param pagesParameter
	 * @param preparedValues 
	 * @return
	 */
	public Condition makePreparedSQLWithLike(StringBuilder where, 
			Map<String, String> pagesParameter, Map<String, Object> preparedValues){
		if ( !this.isEmpty() ) {
			if ( !pagesParameter.isEmpty() ) {
				where.append(" and ");
			} 
			where.append(this.getCondition());
			pagesParameter.put(this.getParamKey(), this.getParamValue());
			if ( PlatformTools.isNumber(this.getParamValue()) ) {
				if ( PlatformTools.isLong(this.getParamValue()) ) {
					preparedValues.put("#"+ this.getParamKey() +"#", "%"+Long.valueOf(this.getParamValue())+"%");
				} else {
					preparedValues.put("#"+ this.getParamKey() +"#", "%"+Double.valueOf(this.getParamValue())+"%");
				}
			} else {
				preparedValues.put("#"+ this.getParamKey() +"#", "%"+this.getParamValue()+"%");
			}
			
		}
		return this;
	}
	
	/**
	 * @param where 
	 * @param pagesParameter 
	 * @param preparedValues
	 * @return
	 */
	public Condition makePreparedSQLWithLeftLike(StringBuilder where, 
			Map<String, String> pagesParameter, Map<String, Object> preparedValues){
		if ( !this.isEmpty() ) {
			if ( !pagesParameter.isEmpty() ) {
				where.append(" and ");
			} 
			where.append(this.getCondition());
			pagesParameter.put(this.getParamKey(), this.getParamValue());
			if ( PlatformTools.isNumber(this.getParamValue()) ) {
				if ( PlatformTools.isLong(this.getParamValue()) ) {
					preparedValues.put("#"+ this.getParamKey() +"#", "%"+Long.valueOf(this.getParamValue()));
				} else {
					preparedValues.put("#"+ this.getParamKey() +"#", "%"+Double.valueOf(this.getParamValue()));
				}
			} else {
				preparedValues.put("#"+ this.getParamKey() +"#", "%"+this.getParamValue());
			}
			
		}
		return this;
	}
	
	/**
	 * @return the paramKey
	 */
	public String getParamKey() {
		return paramKey;
	}
	/**
	 * @param paramKey the paramKey to set
	 */
	public Condition setParamKey(String paramKey) {
		this.paramKey = paramKey;
		return this;
	}
	/**
	 * @return the paramValue
	 */
	public String getParamValue() {
		return paramValue==null?"":paramValue;
	}
	/**
	 * @param paramValue the paramValue to set
	 */
	public Condition setParamValue(String paramValue) {
		this.paramValue = paramValue;
		return this;
	}
	/**
	 * @return the condition
	 */
	public String getCondition() {
		return condition==null?"":condition;
	}
	/**
	 * @param condition the condition to set
	 */
	public Condition setCondition(String condition) {
		this.condition = condition;
		return this;
	}
	/**
	 * @return the empty
	 */
	public boolean isEmpty() {
		return empty;
	}
	/**
	 * @param empty the empty to set
	 */
	public Condition setEmpty(boolean empty) {
		this.empty = empty;
		return this;
	}
	
	
}
