/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.pages;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.nonesole.web.platform.sys.PlatformTools;

/**
 * Condition for Pages2 object.
 * @author jack lee
 * @version 1.0 - build on 2017-04-11
 */
public abstract class Page2Condtion {

	protected boolean isPrepared;
	protected String daoConditionKey;
	protected String daoConditionValue;
	protected List<Page2Parameter> list;
	protected Map<String, Object> valuesMap;
	protected String urlCondition;
	
	/**
	 * Constructor
	 */
	public Page2Condtion(){
		list = new ArrayList<Page2Parameter>();
	}
	
	/**
	 * @param pp
	 * @return
	 */
	public Page2Condtion add(Page2Parameter pp) {
		if ( pp != null ) {
			list.add(pp);
		}
		return this;
	}
	
	/**
	 * @return
	 */
	public String getUrlCondition(){
		if ( urlCondition == null ) {
			StringBuilder sb = new StringBuilder();
			String s = null;
			int index = 1;
			for ( Page2Parameter pp : list ) {
				s = pp.getUrlCondition();
				if ( !PlatformTools.isNull(s) ) {
					sb.append(s);
					if ( index < list.size() ) {
						sb.append("&");
					}
				}
				index++;
			}
			this.urlCondition = sb.toString();
		}
		return this.urlCondition;
	}
	
	/**
	 * @return the preparedValues
	 */
	public Map<String, Object> getValuesMap() {
		valuesMap = new HashMap<String,Object>();
		if ( PlatformTools.isNull(this.getUrlCondition()) ) {
			return valuesMap;
		}
		for ( Page2Parameter pp : list ) {
			valuesMap.put(pp.getDaoKey(), pp.getValue());
		}
		valuesMap.put(this.daoConditionKey, this.daoConditionValue);
		return valuesMap;
	}
	
	/**
	 * @return the daoConditionKey
	 */
	public String getDaoConditionKey() {
		return daoConditionKey;
	}

	/**
	 * @param daoConditionKey the daoConditionKey to set
	 */
	public Page2Condtion setDaoConditionKey(String daoConditionKey) {
		this.daoConditionKey = daoConditionKey;
		return this;
	}

	/**
	 * @return the daoConditionValue
	 */
	public String getDaoConditionValue() {
		return daoConditionValue;
	}

	/**
	 * @param daoConditionValue the daoConditionValue to set
	 */
	public Page2Condtion setDaoConditionValue(String daoConditionValue) {
		this.daoConditionValue = daoConditionValue;
		return this;
	}

	/**
	 * @return the list
	 */
	public List<Page2Parameter> getList() {
		return list;
	}

	/**
	 * @param isPrepared the isPrepared to set
	 */
	public void setPrepared(boolean isPrepared) {
		this.isPrepared = isPrepared;
	}
	
	/**
	 * @return
	 */
	public Page2Condtion createPreparedSqlCondition() {
		if ( !PlatformTools.isNull(this.getUrlCondition()) && this.isPrepared ) {
			StringBuilder sb = new StringBuilder();
			sb.append(" where ");
			int index = 1;
			for ( Page2Parameter pp : this.list ) {
				sb.append(pp.getComplexPreparedSqlCondition());
				if ( index < this.list.size() ) {
					sb.append(" and ");
				}
				index++;
			}
			this.daoConditionValue = sb.toString();
		}
		return this;
	}

	/**
	 * @return
	 */
	public Page2Condtion createSqlCondition() {
		if ( !PlatformTools.isNull(this.getUrlCondition()) && !this.isPrepared  ) {
			StringBuilder sb = new StringBuilder();
			sb.append(" where ");
			int index = 1;
			for ( Page2Parameter pp : this.list ) {
				sb.append(pp.getSqlCondition());
				if ( index < this.list.size() ) {
					sb.append(" and ");
				}
				index++;
			}
			this.daoConditionValue = sb.toString();
		}
		return this;
	}
	
}
