/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.pages;

import javax.servlet.http.HttpServletRequest;

import com.nonesole.web.platform.sys.PlatformTools;

/**
 * Parameter for Pages2 object.
 * @author jack lee
 * @version 1.0 - build in 2017-04-11
 */
public class Page2Parameter {
	
	private String key;
	private Class<?> clazz;
	private String value;
	private String column;
	private String daoKey;
	private int sign = 0;
	private HttpServletRequest request;
	private String urlCondition;
	
	/**
	 * @param request
	 * @return
	 */
	public Page2Parameter initValue(){
		value = request.getParameter(key);
		request.setAttribute(key, value);
		return this;
	}
	
	/**
	 * @param request
	 * @return
	 */
	public String getUrlCondition(){
		if ( urlCondition == null ) {
			if ( PlatformTools.isNull(key) || PlatformTools.isNull(value) ) {
				urlCondition = "";
			} else {
				urlCondition = key+"="+value;
			}
		}
		return urlCondition;
	}
	
	/**
	 * @param type
	 * @return
	 */
	public String getComplexPreparedSqlCondition(){
		if ( sign < 1 ) {
			return "";
		} 
		value = request.getParameter(key);
		if ( PlatformTools.isNull(value) ) {
			return "";
		}
		StringBuilder sb = new StringBuilder();
		sb.append(" ").append(column).append(" ");
		sb.append(this.getCondtionSign());
		daoKey = "#" + column + "#";
		sb.append(" ").append(daoKey).append(" ");
		return sb.toString();
	}
	
	/**
	 * @param type
	 * @return
	 */
	public String getSqlCondition(){
		if ( sign < 1 ) {
			return "";
		} 
		value = request.getParameter(key);
		if ( PlatformTools.isNull(value) ) {
			return "";
		}
		StringBuilder sb = new StringBuilder();
		sb.append(" ").append(column).append(" ");
		sb.append(this.getCondtionSign());
		daoKey = column;
		if ( PlatformTools.isNumber(clazz) ) {
			sb.append(" ").append(value).append(" ");
		} else {
			sb.append(" '").append(value).append("' ");
		}
		return sb.toString();
	}
	
	/**
	 * @return
	 */
	public String getCondtionSign(){
		switch( sign ) {
		case Page2Parameter.EQUAL:
			return "=";
		case Page2Parameter.GREATER:
			return ">";
		case Page2Parameter.LESS:
			return "<";
		case Page2Parameter.LIKE_LEFT:
			value = "%" + value; 
			return " like ";
		case Page2Parameter.LIKE_RIGHT:
			value = value + "%"; 
			return " like ";
		case Page2Parameter.LIKE_ALL:
			value = "%" + value + "%"; 
			return " like ";
		case Page2Parameter.GREATER_EQUAL:
			return ">=";
		case Page2Parameter.LESS_EQUAL:
			return "<=";
		default:break;
		}
		return "=";
	}
	
	/**
	 * @return the key
	 */
	public String getKey() {
		return key;
	}
	/**
	 * @param key the key to set
	 */
	public Page2Parameter setKey(String key) {
		this.key = key;
		return this;
	}
	/**
	 * @return the clazz
	 */
	public Class<?> getClazz() {
		return clazz;
	}
	/**
	 * @param clazz the clazz to set
	 */
	public Page2Parameter setClazz(Class<?> clazz) {
		this.clazz = clazz;
		return this;
	}
	/**
	 * @return the value
	 */
	public String getValue() {
		return value;
	}
	/**
	 * @param value the value to set
	 */
	public Page2Parameter setValue(String value) {
		this.value = value;
		return this;
	}
	/**
	 * @return the column
	 */
	public String getColumn() {
		return column;
	}
	/**
	 * @param column the column to set
	 */
	public Page2Parameter setColumn(String column) {
		this.column = column;
		return this;
	}
	/**
	 * @return the sign
	 */
	public int getSign() {
		return sign;
	}
	/**
	 * @param sign the sign to set
	 */
	public Page2Parameter setSign(int sign) {
		this.sign = sign;
		return this;
	}
	
	/**
	 * @param request the request to set
	 */
	public Page2Parameter setRequest(HttpServletRequest request) {
		this.request = request;
		return this;
	}
	
	/**
	 * @return the daoKey
	 */
	public String getDaoKey() {
		return daoKey;
	}
	
	/**
	 * @return key
	 */
	public String toString(){
		return key;
	}
	
	public static final int EQUAL = 1;
	public static final int GREATER = 2;
	public static final int LESS = 3;
	public static final int LIKE_LEFT = 4;
	public static final int LIKE_RIGHT = 5;
	public static final int LIKE_ALL = 6;
	public static final int GREATER_EQUAL = 7;
	public static final int LESS_EQUAL = 8;
	
}
