/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.pages;

import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import com.nonesole.log.Logger;
import com.nonesole.web.platform.ParameterEnum;
import com.nonesole.web.platform.i18n.LocaleToolkit;

/**
 * The first Pages class
 * @author jack lee
 * @version 1.0
 * @date 2016-09-21
 */
public class Pages<T> {

	private Integer pageSize;
	private Integer currentPage;
	private Integer rows;
	private T object;	
	private String url;
	private Map<String,String> params;
	private String condition;
	private HttpServletRequest request;
	
	public Pages(HttpServletRequest request){
		this.request = request;
		rows = 0;
		currentPage = 1;
		pageSize = 10;
	}
	
	/**
	 * @param object
	 */
	public void setList(T object) {
		this.object = object;
	}
	
	/**
	 * @return object
	 */
	public T getList(){
		return this.object;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getRows() {
		return rows;
	}
	
	/**
	 * @param Integer
	 */
	public void setRows(Integer rows) {
		if ( rows == null )
			rows = 0;
		this.rows = rows;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getPages() {
		return rows%pageSize==0?rows/pageSize:rows/pageSize+1;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getPageSize() {
		return pageSize;
	}
	
	/**
	 * @param Integer
	 */
	public void setPageSize(Integer pageSize) {
		if ( pageSize == null || pageSize < 1 )
			pageSize = 10;
		this.pageSize = pageSize;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getCurrentPage() {
		return currentPage<=1?1:currentPage>=this.getLastPage()?this.getLastPage():currentPage;
	}
	
	/**
	 * @param Integer
	 */
	public void setCurrentPage(Integer currentPage) {
		this.currentPage = currentPage;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getFirstPage() {
		return 1;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getPreviousPage() {
		return currentPage<=1?1:currentPage-1;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getNextPage() {
		return currentPage>=this.getLastPage()?this.getLastPage():currentPage+1;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getLastPage() {
		int mod = rows%pageSize;
		int result = rows/pageSize;
		return mod==0?(result==0?1:result):(result+1);
	}
	
	/**
	 * @return Integer
	 */
	public int getStartNumber(){
		return (this.getCurrentPage()-1)*pageSize;
	}
	
	/**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * @param url the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}

	/**
	 * @return the params
	 */
	public Map<String, String> getParams() {
		return params;
	}

	/**
	 * @param params the params to set
	 * @throws UnsupportedEncodingException 
	 */
	public void setParams(Map<String, String> params) {
		this.params = params;
		StringBuilder sb = new StringBuilder();
		if ( this.params != null ) {
			Set<Entry<String,String>> ens = this.params.entrySet();
			for ( Entry<String,String> en : ens ) {
				sb.append("&");
				sb.append(en.getKey()).append("=").append(en.getValue());
			}
		}
		this.condition = sb.toString();
		if ( !this.condition.startsWith("&") ) {
			this.condition = "&" + this.condition;
		}
	}
	
	/**
	 * 生成翻页的静态HTML
	 * @return String
	 */
	public String toHTML(){
		if ( condition == null ) {
			condition = "";
		}
		int cPage = this.getCurrentPage();
		int lPage = this.getLastPage();
		int minPage = cPage>5?cPage-4:1;
		int maxPage = lPage-cPage>3?cPage+2:lPage-cPage<1?cPage:lPage;
		StringBuilder sb = new StringBuilder();
		sb.append("<div class=\"manu\">");
		sb.append(this.getPageUrl(0));
		for ( int page = minPage ; page <=maxPage ; page++ ) {
			sb.append(this.getPageUrl(page));
		}
		if ( lPage-cPage>3 ) {
			sb.append(this.getPageUrl(lPage-1));
			sb.append(this.getPageUrl(lPage));
		}
		sb.append(this.getPageUrl(-1));
		sb.append("</div>");
		
		return sb.toString();
	}
		
	/**
	 * @return String
	 */
	public String getCondition(){
		return this.condition;
	}
	
	/**
	 * @param page
	 * @return
	 */
	private String getPageUrl(int page){
		if ( condition == null || "null".equals(condition) ) {
			condition = "";
		}
		StringBuilder sb = new StringBuilder();
		if ( page == 0 ) {
			if ( this.getCurrentPage() == 1 ) {
				sb.append("<span class=\"disabled\"> <  ")
				.append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.Pages_info_1"))
				.append("</span>");
			} else {
				sb.append("<a href=\"").append(this.url).append("?")
				  .append(ParameterEnum.PAGE.toString()).append("=").append(this.getCurrentPage()-1)
				  .append(this.getCondition()).append("\"> <  ")
				.append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.Pages_info_1"))
				.append("</a>");
			}
		} else if ( page < 0 ) {
			if ( this.getCurrentPage().intValue() == this.getLastPage().intValue() ) {
				sb.append("<span class=\"disabled\">")
				.append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.Pages_info_2"))
				.append("  > </span>");
			} else {
				sb.append("<a href=\"").append(this.url).append("?")
				  .append(ParameterEnum.PAGE.toString()).append("=").append(this.getCurrentPage()+1)
				  .append(this.getCondition()).append("\">")
				.append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.Pages_info_2"))
				.append("  > </a>");
			}
		} else {
			if ( this.getCurrentPage() == page ) {
				sb.append("<span class=\"current\">").append(page).append("</span>");
			} else {
				sb.append("<a href=\"").append(this.url).append("?")
				  .append(ParameterEnum.PAGE.toString()).append("=").append(page)
				  .append(this.getCondition()).append("\">").append(page)
				  .append("</a>");
			}
		}
		Logger.getInstance().debug("Page = " + page + ";  html :"+ sb.toString(), null);
		return sb.toString();
	}
	
	/**
	 * @return int
	 */
	public int getNumberOfRows(){
		if ( null == this.getList() ) {
			return 0;
		} else {
			List<?> l = (List<?>)this.getList();
			if ( l.isEmpty() ) {
				return 0;
			} else {
				if ( l.size() < this.getPageSize() ) {
					return l.size();
				} else {
					return this.getPageSize();
				}
			}
		}
	}
}
