/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.pages;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IPojoAction;
import com.nonesole.persistence2.sql.define.ISingleValueAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.ParameterEnum;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * The second Pages class
 * @author jack lee
 * @version 1.0 - build in 2017-04-11
 */
public class Pages2<T> {

	public static final String PAGE = "p";
	public static final String START_INDEX = "start_index";
	public static final String PAGE_SIZE = "page_size";
	
	private Integer size = 10;
	private Integer page = 1;
	private Integer rows = 0;
	
	private List<T> objects;
	private String url;
	private HttpServletRequest request;
	private Map<String, Object> daoValuesMapping;
	private Page2Condtion pCondition;

	private String totalRowsDaoName;
	private String resultsDaoName;
	
	private boolean isPrepared = true;//预编译
	
	/**
	 * @param key DAO的SQL标记
	 * @param value 替换该标记的数值
	 * @return
	 */
	public Pages2<T> putValue(String key, Object value){
		if ( daoValuesMapping == null ) {
			daoValuesMapping = new HashMap<String, Object>();
		}
		daoValuesMapping.put(key, value);
		return this;
	}
	
	/**
	 * @return
	 */
	public Pages2<T> initAll(){
		if ( daoValuesMapping == null ){
			daoValuesMapping = new HashMap<String, Object>();
		}
		
		if ( pCondition != null ) {
			if ( isPrepared ) {
				pCondition.createPreparedSqlCondition();
			} else {
				pCondition.createSqlCondition();
			}
			Map<String, Object> conditionValues = pCondition.getValuesMap();
			if ( conditionValues != null && !conditionValues.isEmpty() ) {
				daoValuesMapping.putAll(pCondition.getValuesMap());//将拼好的SQL条件和数值参数传入
			}
		}

		this.initPage();
		this.initRows();
		daoValuesMapping.put(START_INDEX, this.getStartNumber());
		daoValuesMapping.put(PAGE_SIZE, size);
		
		this.initResults();
		return this;
	}
	
	/**
	 * @param request
	 */
	public Pages2<T> initPage() {
		page = PlatformTools.getIntParameter(request, PAGE);
		page = page<1?1:page;
		return this;
	}
	
	/**
	 * @param daoName
	 * @param condition 
	 */
	public Pages2<T> initRows() {
		try {
			rows = this.getRowsAction().intAction(totalRowsDaoName, daoValuesMapping);
		} catch (ExecuteSQLException e) {
			Logger.getInstance().error("There is an error on init rows of pages object.", e);
		}
		return this;
	}
	
	/**
	 * @param daoName
	 * @param condition 
	 */
	public Pages2<T> initResults() {
		try {
			objects = this.getQueryAction().pojoListAction(resultsDaoName,daoValuesMapping);
		} catch (ExecuteSQLException e) {
			Logger.getInstance().error("There is an error on init result of pages object.", e);
		}
		return this;
	}
	
	/**
	 * @return
	 */
	public ISingleValueAction getRowsAction(){
		return PersistenceFactory.getInstance().getSingleValueAction();
	}
	
	/**
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public IPojoAction<T> getQueryAction(){
		return (IPojoAction<T>) PersistenceFactory.getInstance().getPojoAction();
	}
	
	/**
	 * @return String
	 */
	public String toHTML(){
		int cPage = this.getCurrentPage();
		int lPage = this.getLastPage();
		int minPage = cPage>5?cPage-4:1;
		int maxPage = lPage-cPage>3?cPage+2:lPage-cPage<1?cPage:lPage;
		StringBuilder sb = new StringBuilder();
		sb.append("<div class=\"manu\">");
		sb.append(this.getPageUrl(0));
		for ( int page = minPage ; page <=maxPage ; page++ ) {
			sb.append(this.getPageUrl(page));
		}
		if ( lPage-cPage>3 ) {
			sb.append(this.getPageUrl(lPage-1));
			sb.append(this.getPageUrl(lPage));
		}
		sb.append(this.getPageUrl(-1));
		sb.append("</div>");
		
		return sb.toString();
	}
	
	/**
	 * @param page
	 * @return
	 */
	private String getPageUrl(int page){
		StringBuilder sb = new StringBuilder();
		if ( page == 0 ) {
			if ( this.getCurrentPage() == 1 ) {
				sb.append("<span class=\"disabled\"> <  ")
				.append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.Pages_info_1"))
				.append("</span>");
			} else {
				sb.append("<a href=\"").append(this.url).append("?")
				  .append(ParameterEnum.PAGE.toString()).append("=").append(this.getCurrentPage()-1)
				  .append(pCondition==null?"":pCondition.getUrlCondition() ).append("\"> <  ")
				.append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.Pages_info_1"))
				.append("</a>");
			}
		} else if ( page < 0 ) {
			if ( this.getCurrentPage().intValue() == this.getLastPage().intValue() ) {
				sb.append("<span class=\"disabled\">")
				.append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.Pages_info_2"))
				.append("  > </span>");
			} else {
				sb.append("<a href=\"").append(this.url).append("?")
				  .append(ParameterEnum.PAGE.toString()).append("=").append(this.getCurrentPage()+1)
				  .append(pCondition==null?"":pCondition.getUrlCondition() ).append("\">")
				.append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.Pages_info_2"))
				.append("  > </a>");
			}
		} else {
			if ( this.getCurrentPage() == page ) {
				sb.append("<span class=\"current\">").append(page).append("</span>");
			} else {
				sb.append("<a href=\"").append(this.url).append("?")
				  .append(ParameterEnum.PAGE.toString()).append("=").append(page)
				  .append(pCondition==null?"":pCondition.getUrlCondition() ).append("\">").append(page)
				  .append("</a>");
			}
		}
		return sb.toString();
	}
	
	/**
	 * @return Integer
	 */
	public Integer getPages() {
		return rows%size==0?rows/size:rows/size+1;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getPageSize() {
		return size;
	}
	
	/**
	 * @param Integer
	 */
	public Pages2<T> setPageSize(Integer size) {
		if ( size == null || size < 1 )
			size = 10;
		this.size = size;
		return this;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getCurrentPage() {
		return page<=1?1:page>=this.getLastPage()?this.getLastPage():page;
	}
	
	/**
	 * @param Integer
	 */
	public Pages2<T> setCurrentPage(Integer page) {
		this.page = page;
		return this;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getFirstPage() {
		return 1;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getPreviousPage() {
		return page<=1?1:page-1;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getNextPage() {
		return page>=this.getLastPage()?this.getLastPage():page+1;
	}
	
	/**
	 * @return Integer
	 */
	public Integer getLastPage() {
		int mod = rows%size;
		int result = rows/size;
		return mod==0?(result==0?1:result):(result+1);
	}
	
	/**
	 * @return Integer
	 */
	public int getStartNumber(){
		return (this.getCurrentPage()-1)*size;
	}
	
	/**
	 * @return int
	 */
	public int getNumberOfRows(){
		if ( null == this.objects ) {
			return 0;
		} else {
			if ( this.objects.isEmpty() ) {
				return 0;
			} else {
				if ( this.objects.size() < this.getPageSize() ) {
					return this.objects.size();
				} else {
					return this.getPageSize();
				}
			}
		}
	}
	
	/**
	 * @return the page
	 */
	public Integer getPage() {
		return page;
	}

	/**
	 * @return the rows
	 */
	public Integer getRows() {
		return rows;
	}

	/**
	 * @return the objects
	 */
	public List<T> getObjects() {
		return objects;
	}

	/**
	 * @param size the size to set
	 */
	public Pages2<T> setSize(Integer size) {
		this.size = size;
		return this;
	}

	/**
	 * @param url the url to set
	 */
	public Pages2<T> setUrl(String url) {
		this.url = url;
		return this;
	}

	/**
	 * @param request the request to set
	 */
	public Pages2<T> setRequest(HttpServletRequest request) {
		this.request = request;
		return this;
	}
	
	/**
	 * @param totalRowsDaoName the totalRowsDaoName to set
	 */
	public Pages2<T> setTotalRowsDaoName(String totalRowsDaoName) {
		this.totalRowsDaoName = totalRowsDaoName;
		return this;
	}

	/**
	 * @param resultsDaoName the resultsDaoName to set
	 */
	public Pages2<T> setResultsDaoName(String resultsDaoName) {
		this.resultsDaoName = resultsDaoName;
		return this;
	}

	/**
	 * @return the isPrepared
	 */
	public boolean isPrepared() {
		return isPrepared;
	}

	/**
	 * @param isPrepared the isPrepared to set
	 */
	public Pages2<T> setPrepared(boolean isPrepared) {
		this.isPrepared = isPrepared;
		return this;
	}
	
	/**
	 * @param pCondition the pCondition to set
	 */
	public Pages2<T> setpCondition(Page2Condtion pCondition) {
		pCondition.setPrepared(this.isPrepared);
		this.pCondition = pCondition;
		return this;
	}
	
	/**
	 * 
	 * @param objects
	 * @return
	 */
	public Pages2<T> setObjects(List<T> objects) {
		this.objects = objects;
		return this;
	}
}
