/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.safe;

import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.log.Logger;
import com.nonesole.web.platform.ActionControler;
import com.nonesole.web.platform.ParameterEnum;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.safe.ipblacklist.IBlackList;
import com.nonesole.web.platform.safe.ipwhitelist.IWhiteList;
import com.nonesole.web.platform.safe.redirect.IRedirect;
import com.nonesole.web.platform.safe.redirect.Redirect;
import com.nonesole.web.platform.safe.verificationcode.Code;
import com.nonesole.web.platform.safe.verificationcode.IVerificationCode;
import com.nonesole.web.platform.safe.wordblacklist.IWordBlackList;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * Security filter
 * @author jack lee
 * @version 1.0
 */
public class SafeWall implements Filter {

	public static Map<String,String> REDIRECT_SOURCE_TARGET;
	public static Map<String,String> REDIRECT_TARGET_SOURCE;
	public static Map<String,String> BLACK_WORD;
	public static Set<String> BLACK_IP;
	public static Set<String> WHITE_IP;
	public static Code CODE;
	public static String PAGE_404 = "404.html"; 
	public static String CODE_MAKER = "/safe/codemaker.jsp"; 
	public static String REPLACE_CHAR = "*";
	public static String LOCAL_IP = "127.0.0.1";
	
	@Override
	public void destroy() {
		
	}

	@Override
	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {
		HttpServletRequest httpRequest = (HttpServletRequest) request;
		HttpServletResponse httpResponse = (HttpServletResponse) response;
		String path = httpRequest.getServletPath();
		
		String url = null;
		if ( REDIRECT_TARGET_SOURCE != null ) {
			url = REDIRECT_TARGET_SOURCE.get(path);
			if ( !PlatformTools.isNull(url) ) {
				RequestDispatcher rd = null;
				ServletContext sc = request.getServletContext();
				rd = sc.getRequestDispatcher(url); 
				rd.forward(request, response);
				return;
			}
		}
		
		if ( REDIRECT_SOURCE_TARGET != null ) {
			url = REDIRECT_SOURCE_TARGET.get(path);
			if ( !PlatformTools.isNull(url) ) {
				this.redirect(httpRequest, httpResponse, PAGE_404);
				return;
			}
		}

		if (!PlatformTools.isServlet(httpRequest)
				&& !PlatformTools.isJSP(httpRequest)) {
			chain.doFilter(request, response);
			return;
		}
				
		boolean isWhiteIP = false;
		String ip = this.getIpAddr(httpRequest);
		if ( !LOCAL_IP.equals(ip) ) {
			if ( WHITE_IP != null && !WHITE_IP.isEmpty() ) {
				for ( String wip : WHITE_IP ) {
					if ( ip.matches(wip) ) {
						isWhiteIP = true;
						break;
					}
				}
				if ( !isWhiteIP ) {
					this.redirect(httpRequest, httpResponse, PAGE_404);
					return;
				}
			}
			
			if ( BLACK_IP != null && !BLACK_IP.isEmpty() ) {
				if ( !isWhiteIP ) {
					for ( String bip : BLACK_IP ) {
						if ( ip.matches(bip) ) {
							this.redirect(httpRequest, httpResponse, PAGE_404);
							return;
						}
					}
				}
			}
		}
		
		if ( BLACK_WORD != null && !BLACK_WORD.isEmpty() ) {
			Set<String> keyset = BLACK_WORD.keySet();
			Enumeration<String> pn = request.getParameterNames();
			if ( pn != null && keyset != null ) {
				String s = null;
				while ( pn.hasMoreElements() ) {
					s = request.getParameter(pn.nextElement());
					if ( s != null ) {
						for ( String k : keyset ) {
							if ( s.contains(k) ) {
								this.redirect(httpRequest, httpResponse, PAGE_404);
								return;
							}
						}
					}
				}
			}
		}
		
		chain.doFilter(request, response);
	}

	public static void initIPBlackList(){
		Logger logger = Logger.getInstance();
		//初始化数据
		BLACK_IP = null;
		//IP黑名单
		try {
			IBlackList bAction = (IBlackList) ActionControler.getInstance()
					.getRegistrationAction("SafeWall Object", IBlackList.class);
			if ( bAction != null ) {
				BLACK_IP = bAction.findStartedBlackList();
			}
		} catch (Exception e) {
			logger.error("There is an error on blacklist action.", e);
		}
	}
	
	public static void initIPWhiteList(){
		Logger logger = Logger.getInstance();
		//IP白名单
		WHITE_IP = null;
		try {
			IWhiteList wAction = (IWhiteList) ActionControler.getInstance()
					.getRegistrationAction("SafeWall Object", IWhiteList.class);
			if ( wAction != null ) {
				WHITE_IP = wAction.findStartedWhiteList();
			} 
			if ( WHITE_IP == null) {
				WHITE_IP = new HashSet<String>();
			}
			//WHITE_IP.add(LOCAL_IP);
		} catch (Exception e) {
			logger.error("There is an error on whitelist action.", e);
		}
	}
	
	public static void initWordBlackList(){
		Logger logger = Logger.getInstance();
		//屏蔽词
		try {
			BLACK_WORD = null;
			IWordBlackList action = (IWordBlackList) ActionControler.getInstance()
					.getRegistrationAction("SafeWall Object", IWordBlackList.class);
			if ( action != null ) {
				Set<String> words = action.findStartedWordBlackList();
				if ( words != null && !words.isEmpty() ) {
					BLACK_WORD = new HashMap<String,String>();
					for ( String w : words ) {
						BLACK_WORD.put(w, REPLACE_CHAR);
					}
				}
			}
		} catch (Exception e) {
			logger.error("There is an error on wordblacklist action.", e);
		}
	}
	
	public static void initRedirect(){
		Logger logger = Logger.getInstance();
		REDIRECT_TARGET_SOURCE = null;
		REDIRECT_SOURCE_TARGET = null;
		//入口保护
		try {
			IRedirect action = (IRedirect) ActionControler.getInstance()
					.getRegistrationAction("SafeWall Object", IRedirect.class);
			if ( action != null ) {
				List<Redirect> l = action.findStartedRedirect();
				if ( l != null && !l.isEmpty() ) {
					REDIRECT_TARGET_SOURCE = new HashMap<String,String>();
					REDIRECT_SOURCE_TARGET = new HashMap<String,String>();
					for ( Redirect obj : l ) {
						REDIRECT_TARGET_SOURCE.put(obj.getTarget_url(), obj.getSource_url());
						REDIRECT_SOURCE_TARGET.put(obj.getSource_url(), obj.getTarget_url());
					}
				}
			}
		} catch (Exception e) {
			logger.error("There is an error on redirect action.", e);
		}
	}
	
	public static void initCode(){
		Logger logger = Logger.getInstance();
		CODE  = null;
		try {
			IVerificationCode action = (IVerificationCode) ActionControler.getInstance()
					.getRegistrationAction("SafeWall Object", IVerificationCode.class);
			if ( action != null ) {
				CODE = action.getConfig();
			}
		} catch (Exception e) {
			logger.error("There is an error on code action.", e);
		}
	}
	
	@Override
	public void init(FilterConfig config) throws ServletException {
		if ( CODE == null ) {
			SafeWall.initIPBlackList();
			SafeWall.initIPWhiteList();
			SafeWall.initRedirect();
			SafeWall.initWordBlackList();
			SafeWall.initCode();
		}
	}
	
	/**
	 * @param response
	 * @param url
	 * @param message
	 */
	protected void redirect(HttpServletRequest request, HttpServletResponse response, String url) {
		try {
			response.sendRedirect(url);
		} catch (IOException e) {
			this.errorRedirect(request, response, ParameterEnum.URL_WELCOME.toString(), 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.safe.SafeWall_info_1"));
		}
	}
	
	/**
	 * @param response
	 * @param url
	 * @param message
	 */
	protected void errorRedirect(HttpServletRequest request, HttpServletResponse response, String url, String message) {
		PlatformTools.setCookieMessage(response, message);
		try {
			response.sendRedirect(url);
		} catch (IOException e) {
			this.errorRedirect(request, response, ParameterEnum.URL_WELCOME.toString(),
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.safe.SafeWall_info_1"));
		}
	}
	
	/**
	 * @param request
	 * @return
	 */
	protected String getIpAddr(HttpServletRequest request) {
		String ip = request.getHeader("x-forwarded-for");  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getHeader("Proxy-Client-IP");  
        }  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getHeader("WL-Proxy-Client-IP");  
        }  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getHeader("HTTP_CLIENT_IP");  
        }  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");  
        }  
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
            ip = request.getRemoteAddr();  
        }
		if ( ip.equals("0:0:0:0:0:0:0:1") ) {
			ip = "127.0.0.1";
		}
		return ip;
	}
	
	/**
	 * @param request
	 */
	public static void param2attrWithoutBlackWord(ServletRequest request){
		Set<String> keyset = BLACK_WORD.keySet();
		Enumeration<String> pn = request.getParameterNames();
		if ( pn != null && keyset != null ) {
			String n = null;
			String s = null;
			while ( pn.hasMoreElements() ) {
				n = pn.nextElement();
				s = request.getParameter(n);
				if ( s != null ) {
					for ( String k : keyset )
					s = s.replaceAll(k, BLACK_WORD.get(k));
					request.setAttribute(n, s);
				}
			}
		}
	}
	
	/**
	 * @param request
	 */
	public static boolean checkCode(HttpServletRequest request) {
		HttpSession session = request.getSession();
		Object sessionObj = session.getAttribute(IVerificationCode.ATTR_CODE);
		if ( PlatformTools.isNull(sessionObj) ) {
			return true;
		} else {
			Object reqestCode = request.getParameter(IVerificationCode.ATTR_CODE);
			if ( PlatformTools.isNull(reqestCode) ) {
				return false;
			} else {
				if ( sessionObj.toString().equalsIgnoreCase(reqestCode.toString()) ) {
					return true;
				} else {
					return false;
				}
			}
		}
	}
	
	/**
	 * @param request
	 */
	public static void countLoginError(HttpServletRequest request) {
		HttpSession session = request.getSession();
		Integer loginErrorNumber = 0;
		Object obj = session.getAttribute(IVerificationCode.PARAM_MAX_FAILED_LOGIN);
		if ( PlatformTools.isNull(obj) ) {
			loginErrorNumber = 1;
		} else {
			loginErrorNumber = PlatformTools.toInt(obj)+1;
		}
		session.setAttribute(IVerificationCode.PARAM_MAX_FAILED_LOGIN, loginErrorNumber);
	}
	
	/**
	 * @param request
	 */
	public static void cleanLoginErrorInfo(HttpServletRequest request) {
		HttpSession session = request.getSession();
		session.setAttribute(IVerificationCode.PARAM_MAX_FAILED_LOGIN, "");
		session.setAttribute(IVerificationCode.ATTR_CODE, "");
	}
	
	public static String getHtml(HttpServletRequest request){
		if ( SafeWall.CODE != null && SafeWall.CODE.getStatus() == 1 ) {
			HttpSession session = request.getSession();
			Integer loginErrorNumber = 0;
			Object obj = session.getAttribute(IVerificationCode.PARAM_MAX_FAILED_LOGIN);
			if ( !PlatformTools.isNull(obj) ) {
				loginErrorNumber = PlatformTools.toInt(obj);
				if ( loginErrorNumber >= SafeWall.CODE.getMax_failed_login() ) {
					StringBuilder sb = new StringBuilder();
					sb.append("<label for=\"inputPassword\" class=\"sr-only\">")
					  .append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.safe.SafeWall_info_2"))
					  .append("</label>");
					sb.append("<input type=\"text\" size=\"4\" name=\"code\" id=\"code\" class=\"form-control\" placeholder=\"")
					  .append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.safe.SafeWall_info_2"))
					  .append("\" required/>");
					sb.append("<img src=\"safe/codemaker.jsp\" id=\"codeImg\"/>&nbsp;<a href=\"javascript:;\" onclick=\"changeImg()\">")
					  .append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.safe.SafeWall_info_3"))
					  .append("</a>");
					return sb.toString();
				}
			}
		}
		return "";
	}
	
}
