/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.safe.ipwhitelist;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IListValueAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.basic.BasicAction;
import com.nonesole.web.platform.basic.ICommon;
import com.nonesole.web.platform.pages.Page2Condtion;
import com.nonesole.web.platform.pages.Page2Parameter;
import com.nonesole.web.platform.pages.Pages2;
import com.nonesole.web.platform.safe.SafeWall;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * WhiteIP Action
 * @author jack lee
 * @version 1.0
 */
public class WhiteListAction extends BasicAction implements IWhiteList {

	public WhiteListAction(){
		super();
	}
	
	@Override
	public void add(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		this.save(WhiteIP.class, request, response);
		SafeWall.initIPWhiteList();
		this.redirect(request, response, PAGE_QUERY_2);
	}

	@Override
	public void delete(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		String parameter = PARAM_WHITE_IPS;
		String url = PAGE_WHITE_LIST;
		this.deleteByIdArray(parameter, WhiteIP.class, request, response);
		SafeWall.initIPWhiteList();
		this.redirect(request, response, url);
	}

	@Override
	public void open(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		this.setStatus(PARAM_WHITE_IPS, 
				FIELD_ID, 
				MAP_KEY_CONDITION, 
				MAP_KEY_STATUS, 
				DAO_SET_STATUS, 
				request, 
				response, 
				INTEGER_OPEN_WHITE_IP);
		SafeWall.initIPWhiteList();
		this.redirect(request, response, PAGE_QUERY_2);
	}

	@Override
	public void close(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		this.setStatus(PARAM_WHITE_IPS, 
				FIELD_ID, 
				MAP_KEY_CONDITION, 
				MAP_KEY_STATUS, 
				DAO_SET_STATUS, 
				request, 
				response, 
				INTEGER_CLOSE_WHITE_IP);
		SafeWall.initIPWhiteList();
		this.redirect(request, response, PAGE_QUERY_2);
	}
	
	@Override
	public void query(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		Page2Parameter ip = this.createKeyWordParam(request, FIELD_IP);
		
		Page2Condtion condition = new Page2Condtion() {
			@Override
			public Page2Condtion createPreparedSqlCondition() {
				if (!PlatformTools.isNull(this.getUrlCondition())
						&& this.isPrepared) {
					StringBuilder sb = new StringBuilder();
					sb.append(" where ");
					int index = 1;
					for (Page2Parameter pp : this.list) {
						sb.append(pp.getComplexPreparedSqlCondition());
						if (index < this.list.size()) {
							sb.append(" and ");
						}
						index++;
					}
					this.daoConditionValue = sb.toString();
				}
				return this;
			}
		};
		condition.setDaoConditionKey(ICommon.DAO_SQL_CONDITION)
				.add(ip);

		Pages2<WhiteIP> page = new Pages2<WhiteIP>()
				.setpCondition(condition)
				.setRequest(request)
				.setUrl(PAGE_QUERY)
				.setTotalRowsDaoName(DAO_METHOD_GET_TOTAL_ROWS)
				.setResultsDaoName(DAO_METHOD_GET_RESULTS)
				.initAll();

		request.setAttribute(ICommon.ATTR_PAGES, page);
	}

	@Override
	public Set<String> findStartedWhiteList() throws Exception {
		try {
			IListValueAction action = PersistenceFactory.getInstance().getListValueAction();
			List<String> l = action.stringListAction(DAO_FIND_STARTED_WHITE_LIST, null);
			
			if ( l != null && !l.isEmpty() ) {
				Set<String> set = new HashSet<String>();
				for ( String lstr : l ) {
					set.add(PlatformTools.escapeStar(lstr));
				}
				return set;
			}
		} catch (ExecuteSQLException e) {
			logger.error("[WhiteListAction:findStartedWhiteList]", e);
		} 
		return null;
	}
}
