/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.safe.redirect;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.web.platform.safe.ISafe;
import com.nonesole.web.platform.sys.CoreException;

/**
 * Url redirect interface
 * @author jack lee
 * @version 1.0
 */
public interface IRedirect extends ISafe<Redirect> {
	
	/**
	 * Find configuration of redirect which has been started.
	 * @return
	 * @throws Exception
	 */
	public List<Redirect> findStartedRedirect() throws Exception;
	
	/**
	 * Update
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void update(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	public final static String DAO_PARAM_WORD = "redirects";
	public final static String PAGE_QUERY = "redirectlist.jsp";
	public final static String PAGE_QUERY_2 = "safe/redirectlist.jsp";
	public final static String DAO_METHOD_GET_RESULTS = "queryRedirect";
	public final static String DAO_METHOD_GET_TOTAL_ROWS = "getRedirectAllRows";
	public static final String PAGE_REDIRECT_LIST = "safe/redirectlist.jsp";
	public static final String PARAM_REDIRECTS = "redirects";
	public static final String PARAM_QUERY_KEY_WORD = "keyword";
	public static final String PARAM_QUERY_REDIRECT = "redirect01";
	public static final String PARAM_ADD = "redirect02";
	public static final String PARAM_UPDATE = "redirect03";
	public static final String PARAM_DELETE = "redirect04";
	public static final String PARAM_OPEN = "redirect05";
	public static final String PARAM_CLOSE = "redirect06";
	public static final int INTEGER_OPEN_REDIRECT = 1;
	public static final int INTEGER_CLOSE_REDIRECT = 0;
	public static final String FIELD_SOURCE_URL = "source_url";
	public static final String FIELD_TARGET_URL = "target_url";
	public static final String FIELD_ID = "id";
	public static final String FIELD_STATUS = "status";
	public static final String MAP_KEY_KEY_WORD = "#keyword#";
	public static final String MAP_KEY_URL = "url";
	public static final String MAP_KEY_STATUS = "status";
	public static final String MAP_KEY_CONDITION = "#condition";
	public static final String DAO_SET_STATUS = "setStatusForRedirect";
	public static final String DAO_QUERY_REDIRECT = "queryRedirect";
	public static final String DAO_GET_ALL_ROWS = "getRedirectAllRows";
	public static final String DAO_FIND_STARTED_REDIRECT = "findStartedRedirect";
	public static final String ATTR_PAGES = "attr_pages";
	
}
