/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.safe.verificationcode;

import java.sql.Connection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IOrmAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.ToolsAction;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.safe.SafeWall;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * Verification code action
 * @author jack lee
 * @version 1.0
 */
public class CodeAction extends ToolsAction implements IVerificationCode {
	
	public CodeAction(){
		super();
	}
	
	/* (non-Javadoc)
	 * @see com.nonesole.web.platform.safe.verificationcode.IVerificationCode#getConfig(javax.servlet.http.HttpSession, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	public void getConfig(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		//logger.debug("CodeAction:getConfig", null);
		try {
			Code code = null;
			IOrmAction action = PersistenceFactory.getInstance().getOrmAction();
			List<?> l = action.query(SQL_GET_CONFIG, Code.class);
			if ( l == null || l.isEmpty() ) {
				
			} else {
				if ( l.get(0) != null ) {
					code = (Code)l.get(0);
				}
			}
			request.setAttribute(ATTR_CODE, code);
		} catch (ExecuteSQLException e) {
			logger.error("[CodeAction:getConfig]", e);
		} 
	}

	/* (non-Javadoc)
	 * @see com.nonesole.web.platform.safe.verificationcode.IVerificationCode#saveConfig(javax.servlet.http.HttpSession, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	public void saveConfig(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		//logger.debug("CodeAction:saveConfig", null);
		try {
			Code obj = new Code();
			this.parameter2orm(obj, request);
			obj.setId(1);
			
			IOrmAction action = PersistenceFactory.getInstance().getOrmAction();
			List<?> l = action.query(SQL_GET_CONFIG, Code.class);
			Connection conn = MultiConnectionPool.getInstance().getConnection(
					MultiConnectionPool.getInstance().getDefaultDataSourceName());
			if ( l == null || l.isEmpty() ) {
				action.insert(obj, conn, true);
			} else {
				action.update(obj, conn, true);
			}
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.safe.verificationcode_CodeAction_info_1"));
			
			SafeWall.initCode();
		} catch (ExecuteSQLException | OperationsException | ClassNotFoundException e) {
			logger.error("[CodeAction:saveConfig]", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.safe.verificationcode_CodeAction_info_2"));
		} finally {
			this.redirect(request, response, PAGE_CODE_CONFIG);			
		}
	}

	@Override
	public Code getConfig() throws CoreException {
		Code code = null;
		try {
			IOrmAction action = PersistenceFactory.getInstance().getOrmAction();
			List<?> l = action.query(SQL_GET_CONFIG, Code.class);
			if ( l == null || l.isEmpty() ) {
				
			} else {
				if ( l.get(0) != null ) {
					code = (Code)l.get(0);
				}
			}
		} catch (ExecuteSQLException e) {
			logger.error("[CodeAction:getConfig]", e);
		} 
		return code;
	}

}
