/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Cookies manager
 * @author jack lee
 * @version 1.0
 */
public class LoginCookies {

	private String loginName;
	private String md5password;
	private String md5code;
	private long loginTime;

	/**
	 * Update cookies
	 * @param response
	 */
	public void updateCookies(HttpServletResponse response){
		if ( PlatformTools.isNull(loginName) || 
			 PlatformTools.isNull(md5password) ||
			 loginTime < 1l ) {
			this.clearCookies(response);
			return;
		}
		int maxAge = 60 * 60 * 24 * 30;
		StringBuilder ck = new StringBuilder();
		ck.append(loginTime).append(loginName).append(md5password);
		md5code = PlatformTools.md5(ck.toString());
		PlatformTools.setCookie(response, AccessFilter.COOKIE_KEY_TIME, String.valueOf(loginTime), maxAge, true);
		PlatformTools.setCookie(response, AccessFilter.COOKIE_KEY_USER_NAME, loginName, maxAge, true);
		PlatformTools.setCookie(response, AccessFilter.COOKIE_KEY_MD5_CODE, md5code, maxAge, true);
	}
	
	/**
	 * Clear cookies
	 * @param response
	 */
	public void clearCookies(HttpServletResponse response){
		PlatformTools.setCookie(response, AccessFilter.COOKIE_KEY_TIME, "", 0, true);
		PlatformTools.setCookie(response, AccessFilter.COOKIE_KEY_USER_NAME, "", 0, true);
		PlatformTools.setCookie(response, AccessFilter.COOKIE_KEY_MD5_CODE, "", 0, true);
	}
	
	/**
	 * Read cookies
	 * @param response
	 */
	public LoginCookies readCookies(HttpServletRequest request){
		loginName = PlatformTools.getCookie(request, AccessFilter.COOKIE_KEY_USER_NAME);
		md5code = PlatformTools.getCookie(request, AccessFilter.COOKIE_KEY_MD5_CODE);
		String timeStr = PlatformTools.getCookie(request, AccessFilter.COOKIE_KEY_TIME);
		if ( PlatformTools.isNull(timeStr) || !PlatformTools.isNumber(timeStr) ) {
			loginTime = 0l;
		} else {
			loginTime = Long.valueOf(timeStr);
		}
		return this;
	}
	
	/**
	 * Check whether or not the parameter is validate
	 * @return
	 */
	public boolean validateParameter(){
		if ( PlatformTools.isNull(loginName) || 
				 PlatformTools.isNull(md5code) ||
				 loginTime < 1l ) {
				return false;
		}
		
		long validateTime = loginTime + (60l * 60l * 24l * 30l);
		if ( System.currentTimeMillis() > validateTime ) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Check whether or not the login parameter is validate
	 * @return
	 */
	public boolean validate(){
		if ( PlatformTools.isNull(md5password) ) {
			return false;
		}
		StringBuilder ck = new StringBuilder();
		ck.append(loginTime).append(loginName).append(md5password);
		String code = PlatformTools.md5(ck.toString());
		return code.equals(this.md5code);
	}
	
	/**
	 * @return the loginName
	 */
	public String getLoginName() {
		return loginName;
	}
	/**
	 * @param loginName the loginName to set
	 */
	public LoginCookies setLoginName(String loginName) {
		this.loginName = loginName;
		return this;
	}
	/**
	 * @return the md5code
	 */
	public String getMd5code() {
		return md5code;
	}
	/**
	 * @param md5code the md5code to set
	 */
	public LoginCookies setMd5code(String md5code) {
		this.md5code = md5code;
		return this;
	}
	/**
	 * @return the loginTime
	 */
	public long getLoginTime() {
		return loginTime;
	}
	/**
	 * @param loginTime the loginTime to set
	 */
	public LoginCookies setLoginTime(long loginTime) {
		this.loginTime = loginTime;
		return this;
	}

	/**
	 * @return the md5password
	 */
	public String getMd5password() {
		return md5password;
	}

	/**
	 * @param md5password the md5password to set
	 */
	public LoginCookies setMd5password(String md5password) {
		this.md5password = md5password;
		return this;
	}
	
}
