/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.web.platform.ActionControler;
import com.nonesole.web.platform.ParameterEnum;
import com.nonesole.web.platform.sys.user.User;
import com.nonesole.web.platform.sys.user.UserEnum;

/**
 * Toolkit for platform
 * @author jack lee
 * @version 1.0
 * @date 2016-09-13
 */
public class PlatformTools {
	
	/**
	 * Find action and execute it
	 * @param actionName
	 * @param request
	 * @param response
	 */
	public static void executeAction(String funcName, HttpServletRequest request, HttpServletResponse response){
		request.setAttribute(ParameterEnum.ACTION.toString(), funcName);
		try {
			ActionControler.getInstance().doAction(request, response);
		} catch (CoreException e) {
			Logger.getInstance().error("There is an error on executing action.", e);
		}
	}
	
	/**
	 * Get list of map from reqeust.
	 * @param request
	 * @param key
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static List<Map<String,Object>> getListMapOject(HttpServletRequest request, String key){
		if ( request.getAttribute(key) != null ) {
			return (List<Map<String,Object>>)request.getAttribute(key);
		}
		return new ArrayList<Map<String,Object>>();
	}
	
	/**
	 * Get map from request
	 * @param request
	 * @param key
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Map<String,Object> getMapOject(HttpServletRequest request, String key){
		if ( request.getAttribute(key) != null ) {
			return (Map<String,Object>)request.getAttribute(key);
		}
		return new HashMap<String,Object>();
	}
	
	/**
	 * @param str
	 * @return
	 */
	public static String htmlTransform(String str){
		if ( str != null ) {
			String result =	str.replaceAll("&", "&amp;");
			result = str.replaceAll("<", "&lt;");
			result = str.replaceAll(">", "&gt;");
			result = str.replaceAll("\"", "&quot;");
			result = str.replaceAll(" ", "&nbsp;");
			result = str.replaceAll("©", "&copy;");
			result = str.replaceAll("®", "&reg;");
			result = str.replaceAll("'", "&apos;");
			result = str.replaceAll("\"", "&apos;");
			return result;
		}
		return "";
	}
	
	/**
	 * @param curTime
	 * @param dateUnit
	 * @return
	 */
	public static String getNow() {
		String dateUnit = "yyyy-MM-dd HH:mm:ss:SSS";
		SimpleDateFormat formater = new SimpleDateFormat(dateUnit);
		return formater.format(new Date(System.currentTimeMillis()));
	}
	
	/**
	 * @param curTime
	 * @param dateUnit
	 * @return
	 */
	public static String getFormatTime(long curTime,String dateUnit) {
		SimpleDateFormat formater = new SimpleDateFormat(dateUnit);
		return formater.format(new Date(curTime));
	}
		
	/**
	 * @param pageName
	 * @return
	 */
	public static final String getPathForRoot(String pageName){
		return "/"+pageName;
	}
	
	/**
	 * md5
	 * @param String message
	 * @return String
	 */
	public final static String md5(String message){ 
		MessageDigest md; 
	try { 
		md = MessageDigest.getInstance("md5"); 
	} catch (java.security.NoSuchAlgorithmException e) { 
		throw new RuntimeException(e.getMessage()); 
	} 
		md.update(message.getBytes()); 
		return new BigInteger(md.digest()).toString(); 
	} 
	
	/**
	 * SHA
	 * @param String message
	 * @return String
	 */
	public static final String sha(String message){ 
		MessageDigest md; 
	try { 
		md = MessageDigest.getInstance("SHA"); 
	} catch (java.security.NoSuchAlgorithmException e) { 
		throw new RuntimeException(e.getMessage()); 
	} 
		md.update(message.getBytes()); 
		return new BigInteger(md.digest()).toString(); 
	} 
	
	/**
	 * Set message in cookie
	 * @param response
	 * @param message
	 */
	public static void setCookieMessage(HttpServletResponse response, String message){
		setCookie(response, "message", message, 60 * 60 * 24 * 30, true);
	}
	
	/**
	 * Clear cookies
	 * @param message
	 */
	public static void clearCookieMessage(HttpServletResponse response){
		setCookie(response, "message", "", 0, true);
	}
	
	/**
	 * Read message from cookie
	 * @param message
	 */
	public static String getCookieMessage(HttpServletRequest request){
		return getCookie(request,"message");
	}
	
	/**
	 * Get name from cookies
	 * @param message
	 */
	public static String getCookie(HttpServletRequest request, String name){
		if ( request != null ) {
			Cookie[] cookies = request.getCookies();
			if ( cookies != null ) {
				for ( Cookie c : cookies ) {
					if ( name.equals(c.getName()) ) {
						try {
							return URLDecoder.decode(c.getValue(), "UTF-8");
						} catch (UnsupportedEncodingException e) {
							return c.getValue();
						}
					}
				}
			}
		}
		return "";
	}
	
	/**
	 * Set cookies
	 * @param response 
	 * @param key 
	 * @param message 
	 * @param maxAge 
	 * @param global 
	 */
	public static void setCookie(HttpServletResponse response, String key, String message, int maxAge, String path){
		if ( isNull(key) ) {
			return;
		}
		String encodeMessage = null;
		try {
			encodeMessage  = URLEncoder.encode(message, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			encodeMessage = message;
		}
		Cookie cookie = new Cookie(key, encodeMessage);
		cookie.setPath(PlatformTools.isNull(path)?"":path);
		cookie.setMaxAge(maxAge);
		response.addCookie(cookie);
	}
	
	/**
	 * Set cookie
	 * @param response 
	 * @param key 
	 * @param message 
	 * @param maxAge 
	 * @param global 
	 */
	public static void setCookie(HttpServletResponse response, String key, String message, int maxAge, boolean global){
		if ( isNull(key) ) {
			return;
		}
		String encodeMessage = null;
		try {
			encodeMessage  = URLEncoder.encode(message, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			encodeMessage = message;
		}
		Cookie cookie = new Cookie(key, encodeMessage);
		if ( global ) {
			cookie.setPath("/");
		}
		cookie.setMaxAge(maxAge);
		response.addCookie(cookie);
	}
	
	/**
	 * Check the action whether or not is update action. 
	 * @param request
	 * @return
	 */
	public final static boolean isUpdate(HttpServletRequest request,String updateKey){
		String at = request.getParameter(ParameterEnum.ACTION.toString());
		if ( null == at 
				|| "".equals(at) 
				|| null== updateKey 
				|| "".equals(updateKey)
				|| !updateKey.equals(at) ) {
			return false;
		}
		return true;
	}
	
	/**
	 * Transform object to int
	 * @param obj
	 * @return Integer
	 */
	public final static Integer toInt(Object obj) {
		if ( isNull(obj) ) {
			return 0;
		} else {
			return Integer.valueOf(obj.toString());
		}
	}
	
	/**
	 * Transform object to Integer object
	 * @param obj
	 * @return Integer
	 */
	public final static Integer page2Int(String obj) {
		if ( isNull(obj) ) {
			return 1;
		} else {
			return Integer.valueOf(obj);
		}
	}
	
	/**
	 * Get int parameter from request
	 * @param obj
	 * @return Integer
	 */
	public final static Integer getIntParameter(HttpServletRequest request, String key) {
		String s = request.getParameter(key);
		if ( isNull(s) ) {
			return 0;
		} else {
			return Integer.valueOf(s);
		}
	}
	
	/**
	 * Get long parameter from request
	 * @param obj
	 * @return Long
	 */
	public final static Long getLongParameter(HttpServletRequest request, String key) {
		String s = request.getParameter(key);
		if ( isNull(s) ) {
			return 0l;
		} else {
			return Long.valueOf(s);
		}
	}
	
	/**
	 * Get double parameter from request
	 * @param obj
	 * @return Double
	 */
	public final static Double getDoubleParameter(HttpServletRequest request, String key) {
		String s = request.getParameter(key);
		if ( isNull(s) ) {
			return 0d;
		} else {
			return Double.valueOf(s);
		}
	}
	
	/**
	 * Get float parameter from request
	 * @param obj
	 * @return Float
	 */
	public final static Float getFloatParameter(HttpServletRequest request, String key) {
		String s = request.getParameter(key);
		if ( isNull(s) ) {
			return 0f;
		} else {
			return Float.valueOf(s);
		}
	}
	
	/**
	 * Get short parameter from request
	 * @param obj
	 * @return Float
	 */
	public final static Short getShortParameter(HttpServletRequest request, String key) {
		String s = request.getParameter(key);
		if ( isNull(s) ) {
			return 0;
		} else {
			return Short.valueOf(s);
		}
	}
	
	/**
	 * Get boolean parameter from request
	 * @param obj
	 * @return Boolean
	 */
	public final static Boolean getBooleanParameter(HttpServletRequest request, String key) {
		String s = request.getParameter(key);
		if ( isNull(s) ) {
			return false;
		} else {
			return Boolean.valueOf(s);
		}
	}
		
	/**
	 * Check whether or not the class is number
	 * @param clazz
	 * @return boolean
	 */
	public final static boolean isNumber(Class<?> clazz) {
		return isInt(clazz) || isLong(clazz) || isDouble(clazz) || isFloat(clazz) || isShort(clazz);
	}
	
	/**
	 * Check whether or not the class is int
	 * @param clazz
	 * @return boolean
	 */
	public final static boolean isInt(Class<?> clazz) {
		return clazz==null?false:(clazz==int.class||clazz==Integer.class)?true:false;
	}
	
	/**
	 * Check whether or not the class is long
	 * @param clazz
	 * @return boolean
	 */
	public final static boolean isLong(Class<?> clazz) {
		return clazz==null?false:(clazz==long.class||clazz==Long.class)?true:false;
	}
	
	/**
	 * Check whether or not the class is float
	 * @param clazz
	 * @return boolean
	 */
	public final static boolean isFloat(Class<?> clazz) {
		return clazz==null?false:(clazz==float.class||clazz==Float.class)?true:false;
	}
	
	/**
	 * Check whether or not the class is double
	 * @param clazz
	 * @return boolean
	 */
	public final static boolean isDouble(Class<?> clazz) {
		return clazz==null?false:(clazz==double.class||clazz==Double.class)?true:false;
	}
	
	/**
	 * Check whether or not the class is short
	 * @param clazz
	 * @return boolean
	 */
	public final static boolean isShort(Class<?> clazz) {
		return clazz==null?false:(clazz==short.class||clazz==Short.class)?true:false;
	}
	
	/**
	 * Check whether or not the class is byte
	 * @param clazz
	 * @return boolean
	 */
	public final static boolean isByte(Class<?> clazz) {
		return clazz==null?false:(clazz==byte.class||clazz==Byte.class)?true:false;
	}
	
	/**
	 * Check whether or not the class is char
	 * @param clazz
	 * @return boolean
	 */
	public final static boolean isChar(Class<?> clazz) {
		return clazz==null?false:(clazz==char.class||clazz==Character.class)?true:false;
	}
	
	/**
	 * Check whether or not the class is boolean
	 * @param clazz
	 * @return boolean
	 */
	public final static boolean isBoolean(Class<?> clazz) {
		return clazz==null?false:(clazz==boolean.class||clazz==Boolean.class)?true:false;
	}
	
	/**
	 * Check whether or not the object is null
	 * @param s
	 * @return
	 */
	public final static boolean isNull(Object s){
		if ( null == s ) {
			return true;
		} else {
			if ( s instanceof String ) {
				if ( "".equals(s) ) {
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * Check whether or not the object is number
	 * @param obj
	 * @return
	 */
	public final static boolean isNumber(Object obj) {
		if ( obj == null )
			return false;
		if ( obj.toString().matches("(\\-|\\+)?\\d+(\\.\\d+)?") ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Check whether or not the object is long
	 * @param obj
	 * @return
	 */
	public final static boolean isLong(Object obj) {
		if ( obj == null )
			return false;
		if ( obj.toString().matches("(\\-|\\+)?\\d+") ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Check whether or not the object is email
	 * @param obj
	 * @return
	 */
	public final static boolean isEmail(String obj) {
		if ( obj == null )
			return false;
		if ( !CommonTools.checkLength(obj, 0, 50) ) {
			return false;
		}
		if ( obj.toString().matches("^\\s*\\w+(?:\\.{0,1}[\\w-]+)*@[a-zA-Z0-9]+(?:[-.][a-zA-Z0-9]+)*\\.[a-zA-Z]+\\s*$") ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Check whether or not the object is cell-phone-number
	 * @param obj
	 * @return
	 */
	public final static boolean isCellPhoneNumber(String obj) {
		if ( obj == null )
			return false;
		if ( obj.toString().matches("[\\d]{11}") ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Check whether or not the user name is validate
	 * @param obj
	 * @return
	 */
	public final static boolean validateUserName(String obj) {
		if ( obj == null )
			return false;
		if ( obj.toString().matches("[A-Za-z0-9_]{2,50}") ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Set value
	 * @param fieldName field name
	 * @param fieldType field type
	 * @param obj owner of field
	 * @param parameter value of filed
	 * @throws OperationsException
	 * */
	public final static void setFieldValue(
			String fieldName,Class<?> fieldType,Object obj,Object parameter) 
	throws OperationsException{
		Class<?> clazz = obj.getClass();
		Method m = null;
		if ( null != parameter ) {
			StringBuilder sb = new StringBuilder();
			if ( fieldName.matches("is[A-Z]+[\\w|\\W]*") ) {
				fieldName = fieldName.replace("is", "");
				sb.append("set").
					append(fieldName.substring(0,1).toUpperCase()).
					append(fieldName.substring(1, fieldName.length()));
			} else
				sb.append("set").
					append(fieldName.substring(0,1).toUpperCase()).
					append(fieldName.substring(1, fieldName.length()));
			try {
				m = clazz.getMethod(sb.toString(),fieldType);
			} catch (SecurityException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			} catch (NoSuchMethodException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			}
			//Transform string to int etc.
			if ( String.class == parameter.getClass() && String.class != fieldType ) {
				String v = parameter.toString().trim();
				if ( boolean.class == fieldType || Boolean.class == fieldType ) {
					parameter = Boolean.parseBoolean("".equals(v)?"false":v);
				} else if ( int.class == fieldType || Integer.class == fieldType ) {
					parameter = Integer.valueOf("".equals(v)?"0":v).intValue();
				} else if ( long.class == fieldType || Long.class == fieldType ) {
					parameter = Long.valueOf("".equals(v)?"0":v).longValue();
				} else if ( float.class == fieldType || Float.class == fieldType ) {
					parameter = Float.valueOf("".equals(v)?"0":v).floatValue();
				} else if ( double.class == fieldType || Double.class == fieldType ) {
					parameter = Double.valueOf("".equals(v)?"0":v).doubleValue();
				} else if ( short.class == fieldType || Short.class == fieldType ) {
					parameter = Short.valueOf("".equals(v)?"0":v).shortValue();
				} else if ( Byte.class == fieldType || byte.class == fieldType ) {
					parameter = Byte.valueOf("".equals(v)?"0":v).byteValue();
				} else if ( Character.class == fieldType || char.class == fieldType ) {
					parameter = String.valueOf(v);
				} 
			}
			try {
				m.invoke(obj, new Object[]{parameter});
			} catch (IllegalArgumentException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			} catch (IllegalAccessException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			} catch (InvocationTargetException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			}			
		}	
	}
	
	/**
	 * Get url
	 * @param url
	 * @return
	 */
	public static final String getUrlForRoot(String url){
		return url!=null?url.replace("../", ""):"";	
	}
	
	/**
	 * Get user from session
	 * @param httpRequest
	 * @return
	 */
	public static User getUser(HttpServletRequest httpRequest) {
		Object obj = httpRequest.getSession().getAttribute(UserEnum.ATTR_SESSION_USER.toString());
		if ( obj != null) {
			User u = (User)obj;
			return u;
		}
		return null;
	}
	
	/**
	 * Double to percent string
	 * @return
	 */
	public static String double2PercentString(double d){
		BigDecimal bd =	new	BigDecimal(d*100);   
		bd = bd.setScale(2,BigDecimal.ROUND_HALF_UP);   
		return bd.toString()+"%";
	}
	
	/**
	 * Double to percent string
	 * @return
	 */
	public static String double2PercentString2(double d){
		java.text.DecimalFormat df=new java.text.DecimalFormat("##.##");    
		return df.format(d*100)+"%"; 
	}
	
	/**
	 * Double to string
	 * @return
	 */
	public static String double2String(double d){
//		BigDecimal bd =	new	BigDecimal(d);   
//		bd = bd.setScale(2,BigDecimal.ROUND_HALF_UP);   
//		return bd.toString();	
		java.text.DecimalFormat df=new java.text.DecimalFormat("#.##");    
		return df.format(d); 
	}
	
	/**
	 * Get roya paltform version
	 * @param d
	 * @return
	 */
	public static String version(){
		return "Roya Platform 1.0.1";
	}

	/**
	 * Delete file
	 * @param uri
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static void deleteFile(URI uri)
			throws FileNotFoundException, IOException {
		File f = new File(uri);
		if ( f.exists() ) {
			f.delete();
		}
	}

	/**  
     * Save file 
     * @param uri  URI  
     * @param content  String 
     * @throws IOException 
     * @throws FileNotFoundException 
     */  
    public static void saveFileFromString(URI uri, String content)
    throws FileNotFoundException, IOException {  
        RandomAccessFile randomFile = 
        	new RandomAccessFile(createFile(uri), "rw"); 
        randomFile.seek(0);   
        //randomFile.writeBytes(content);   
        randomFile.write(content.getBytes("utf-8"));
        randomFile.close();
        randomFile = null;
    }   
    
    /**
	 * Create new file
	 * @param uri URI
	 * @return File
	 * @throws IOException 
	 * */
	public static File createFile(URI uri) throws IOException{
		if ( null == uri )
			throw new IOException("URI is null.");
		File file = new File(uri);
		if ( !file.exists() ) {
			createFolder(file.getParentFile().toURI());
			file.createNewFile();
		}
		if ( !file.isFile() )
			throw new IOException(
					new StringBuilder()
						.append("File \"")
						.append(uri.toString())
						.append("\" is not exist.")
						.toString());
		return file;		
	}
	
	/**
	 * Create new folder
	 * @param uri URI
	 * @return File(folder)
	 * @throws IOException 
	 * */
	public static File createFolder(URI uri) throws IOException{
		if ( null == uri ) 
			throw new IOException("URI is null.");
		File file = new File(uri);
		if ( !file.exists() )
			file.mkdirs();
		if ( !file.isDirectory() ) 
			throw new IOException(
					new StringBuilder()
						.append("Folder \"")
						.append(uri.toString())
						.append("\" is not exist.")
						.toString());
		return file;		
	}
	
	/**
	 * Get parameter from request
	 * @param request
	 * @param key
	 * @return
	 */
	public static String getRequestParameter(HttpServletRequest request, String paramKey){
		String paramValue = null;
		if ( request.getQueryString() == null ) {
			paramValue = request.getParameter(paramKey);
		} else {
			String v = request.getParameter(paramKey);
			if ( !PlatformTools.isNull(v) ) {
				try {
					paramValue = new String(v.getBytes("ISO-8859-1"),"UTF-8");
				} catch (UnsupportedEncodingException e) {
					Logger.getInstance().error("Can not encode string from iso-8859-1 to utf-8", e);
				}
			} else {
				paramValue = v;
			}
		}
		return paramValue==null?"":paramValue;
	}
	
	/**
	 * String to time with Long type
	 * @param time  
	 * @param model
	 * @return 
	 * @throws ParseException
	 */
	public static long string2time(String time, String model) throws ParseException{
		if ( isNull(model) ) {
			model = "yyyy-MM-dd"; //yyyy/MM/dd HH:mm:ss.SSS
		}
		if ( isNull(time) ) {
			return 0l;
		}
		SimpleDateFormat formater = new SimpleDateFormat(model);
		Date newDate = formater.parse(time);
		return newDate.getTime();
	}
	
	/**
	 * Long to string of time
	 * @param time 
	 * @param model 
	 * @return 
	 * @throws ParseException
	 */
	public static String time2string(long time, String model) throws ParseException{
		if ( isNull(model) ) {
			model = "yyyy-MM-dd"; //yyyy/MM/dd HH:mm:ss.SSS
		}
		if ( isNull(time) ) {
			return "";
		}
		SimpleDateFormat formater = new SimpleDateFormat(model);
		Date newDate = new Date(time);
		return formater.format(newDate);
	}
	
	/** 
	 * Wildcard
	 * @param keyword 
	 * @return 
	 */  
	public static String escapeStar(String keyword) {
		if ( keyword == null ) {
			return "";
		}
		return keyword.replaceAll("[*]+", "*").replaceAll("[*]{1}", "[0-9.]*");
	}
	
	/**
	 * Check whether or not The resource is servlet.
	 * @param httpRequest
	 * @return
	 */
	public static boolean isServlet(HttpServletRequest httpRequest) {
		String path = httpRequest.getServletPath();
		if ( AccessFilter.servletRegistrations != null ) {
			path = path.substring(1);
			if ( AccessFilter.servletRegistrations.get(path) != null ) {
				Logger.getInstance().debug("AccessFilter:isServlet:"+path, null);
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Check whether or not The resource is jsp.
	 * @param httpRequest
	 * @return
	 */
	public static boolean isJSP(HttpServletRequest httpRequest) {
		String path = httpRequest.getServletPath();
		if ( path.endsWith(".jsp") ) {
			Logger.getInstance().debug("AccessFilter:isJSP:"+path, null);
			return true;
		}
		return false;
	}
	
	/**
	 * Load file
	 * @param path
	 * @param lines
	 * @return
	 */
	public static String loadFile(String path, int lines) {
		StringBuilder sb = new StringBuilder();
		if ( lines < 1 ) {
			lines = 500;
		}
		if ( isNull(path) ) {
			return sb.toString();
		}
		File f = new File(path);
		if ( !f.isFile() ) {
			return sb.toString();
		}
		InputStream is = null;
		InputStreamReader isr = null;
		BufferedReader br = null;
		
		try {
			is = new FileInputStream(f);
			isr = new InputStreamReader(is,"utf-8");
			br = new BufferedReader(isr);
			String line = null;
			int rows = 0;
			while( null != (line = br.readLine()) ) {
				if ( rows > lines ) {
					break;
				}
				sb.append(line).append("\r\n");
				rows++;
			}
			br.close();
			isr.close();
			is.close();
			br = null;
			isr = null;
			is = null;
		} catch ( IOException e) {
			Logger.getInstance().error("Can not load file.", e);
		}
		return sb.toString();
	}
	
	/**
	 * Download file
	 * @param path
	 * @param response
	 * @return
	 */
	public static HttpServletResponse download(String path, HttpServletResponse response) {
        try {
            File file = new File(path);
            String filename = file.getName();
            //String ext = filename.substring(filename.lastIndexOf(".") + 1).toUpperCase();

            InputStream fis = new BufferedInputStream(new FileInputStream(path));
            byte[] buffer = new byte[fis.available()];
            fis.read(buffer);
            fis.close();
            
            response.reset();
            response.addHeader("Content-Disposition", "attachment;filename=" + new String(filename.getBytes()));
            response.addHeader("Content-Length", "" + file.length());
            response.setContentType("application/octet-stream");
            OutputStream toClient = new BufferedOutputStream(response.getOutputStream());
            toClient.write(buffer);
            toClient.flush();
            toClient.close();
        } catch (IOException e) {
        	Logger.getInstance().error("Can not download file.", e);
        }
        return response;
    }

	/**
	 * Download local file
	 * @param path 
	 * @param response
	 * @throws FileNotFoundException
	 */
	public static void downloadLocalFile(String path, String fileName, HttpServletResponse response) throws FileNotFoundException {
        InputStream inStream = new FileInputStream(path);// 文件的存放路径
               
        response.reset();
        response.setContentType("bin");
        response.addHeader("Content-Disposition", "attachment; filename=\"" + fileName + "\"");
        byte[] b = new byte[100];
        int len;
        
        try {
        	OutputStream os = response.getOutputStream();
            while ((len = inStream.read(b)) > 0)
            	os.write(b, 0, len);
            inStream.close();
        } catch (IOException e) {
        	Logger.getInstance().error("Can not download file.", e);
        }
    }
	
	/**
	 * Download local file
	 * @param path 本地文件路径（例如e:/test.log）
	 * @param response
	 * @throws FileNotFoundException
	 */
	public static void downloadLocalFile(String path,HttpServletResponse response) throws FileNotFoundException {
        File file = new File(path);
        InputStream inStream = new FileInputStream(path);// 文件的存放路径
        
        response.reset();
        response.setContentType("bin");
        response.addHeader("Content-Disposition", "attachment; filename=\"" + file.getName() + "\"");
        byte[] b = new byte[100];
        int len;
        
        try {
        	OutputStream os = response.getOutputStream();
            while ((len = inStream.read(b)) > 0)
            	os.write(b, 0, len);
            inStream.close();
        } catch (IOException e) {
        	Logger.getInstance().error("Can not download file.", e);
        }
    }

    /**
     * Download file of web url
     * @param url 
     * @param response
     * @throws MalformedURLException
     */
	public static void downloadNetFile(URL url, String targetFilePath, HttpServletResponse response) 
			throws MalformedURLException {
        int bytesum = 0;
        int byteread = 0;

        try {
            URLConnection conn = url.openConnection();
            InputStream inStream = conn.getInputStream();
            //FileOutputStream fs = new FileOutputStream("c:/abc.gif");
            FileOutputStream fs = new FileOutputStream(targetFilePath);

            byte[] buffer = new byte[1204];
            while ((byteread = inStream.read(buffer)) != -1) {
                bytesum += byteread;
                System.out.println(bytesum);
                fs.write(buffer, 0, byteread);
            }
            fs.flush();
            fs.close();
        } catch (IOException e) {
            Logger.getInstance().error("Can not download file.", e);
        } 
    }
	
    /**
     * Download and open file
     * @param filePath
     * @param response
     * @param isOnline 
     */
	public static void downloadOnline(String filePath, HttpServletResponse response, boolean isOnLine) throws Exception {
        File f = new File(filePath);
        if (!f.exists()) {
            response.sendError(404, "File not found!");
            return;
        }
        BufferedInputStream br = new BufferedInputStream(new FileInputStream(f));
        byte[] buf = new byte[1024];
        int len = 0;

        response.reset(); 
        if (isOnLine) { 
            URL u = new URL("file:///" + filePath);
            response.setContentType(u.openConnection().getContentType());
            response.setHeader("Content-Disposition", "inline; filename=" + f.getName());
           
        } else { 
            response.setContentType("application/x-msdownload");
            response.setHeader("Content-Disposition", "attachment; filename=" + f.getName());
        }
        OutputStream out = response.getOutputStream();
        while ((len = br.read(buf)) > 0)
            out.write(buf, 0, len);
        br.close();
        out.close();
    }
	
	/**
	 * Save file
	 * @param file
	 * @param content
	 * @throws Exception
	 */
	public static void saveFileFromString(File file, String content)
			throws Exception {
		if ( file == null || content == null )
			throw new NullPointerException("input value is null.");
		if ( !file.exists() ) {
			file.createNewFile();
		}
		RandomAccessFile randomFile = new RandomAccessFile(file, "rw");
		long fileLength = randomFile.length();
		randomFile.seek(fileLength);
		randomFile.writeBytes(content);
		randomFile.close();
		randomFile = null;
	}
	
	
	public static void main(String[] args) {
		System.out.println(PlatformTools.validateUserName("&#120;&#120;&#120;&#59;&#39;"));
	}
}
