/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.XMLHandler;
import com.nonesole.web.platform.ActionControler;

/**
 * XML Parser for platform
 * @author jack lee
 * @version 1.0
 * @date 2016-10-27
 */
public class PlatformXMLParser {

	private XMLHandler handler = null;
	private Document doc = null;
	public static final String PLATFORM = "platform";
	public static final String CACHE = "cache";
	public static final String CACHE_ACTION = "action";
	public static final String CACHE_ACTION_NAME = "name";
	public static final String CACHE_ACTION_ENABLE = "enable";
	public static final String CACHE_ACTION_INTEGERFACE = "interface";
	public static final String CACHE_ACTION_CLASS = "class";
	public static final String CACHE_ACTION_METHOD = "method";
	public static final String CACHE_ACTION_METHOD_NAME = "name";
	public static final String CACHE_ACTION_METHOD_PARAMETER = "parameter";
	public static final String CACHE_ACTION_PAGE = "page";
	public static final String CACHE_ACTION_PAGE_ENTRANCE = "entrance";
	public static final String CACHE_ACTION_PAGE_URL = "url";
	public static final String CACHE_PUBLIC_PAGE = "public_page";
	public static final String CACHE_PUBLIC_PAGE_URL = "url";
	public static final String CACHE_PUBLIC_PAGE_NAME = "url";
	public static final String CACHE_PUBLIC_PAGE_ENABLE = "enable";
	
	private Logger logger;
	
	/**
	 * Constructor
	 * @throws Exception
	 */
	public PlatformXMLParser() throws Exception{
		logger = Logger.getInstance();;
		handler = XMLHandler.getInstance();
		doc = handler.loadStreamFile("roya.xml");
	}
	
	/**
	 * init catch
	 * @throws ClassNotFoundException
	 */
	public void init() throws ClassNotFoundException{
		if ( doc != null ) {
			List<Node> cacheNodes = this.handler.getChildNodes(doc, CACHE);
			if ( cacheNodes != null && !cacheNodes.isEmpty() ) {
				Node cacheNode = cacheNodes.get(0);
				List<Node> actionNodes = this.handler.getChildNodes(cacheNode, CACHE_ACTION);
				if ( actionNodes != null && !actionNodes.isEmpty() ) {
					logger.debug("Parse action node and init cache.", null);
					for ( Node actionNode : actionNodes ) {
						this.parseAction(actionNode);
					}
				}
				logger.debug("Parse public page node and init cache.", null);
				this.initPublicPage(cacheNode);
			}
		}
	}
	
	/**
	 * Parse action node
	 * @param actionNode
	 * @throws ClassNotFoundException
	 */
	protected void parseAction(Node actionNode) throws ClassNotFoundException{
		if ( actionNode == null ) {
			return;
		}
		String enableNodeAttrbute = this.handler.getNodeAttribute(actionNode, CACHE_ACTION_ENABLE);
		boolean enable = enableNodeAttrbute==null?false:"true".equals(enableNodeAttrbute)?true:false;
		if ( enable ) {
			String actionName = this.handler.getNodeAttribute(actionNode, CACHE_ACTION_NAME);
			if ( actionName != null ) {
				//interface , class
				String interfaceName = this.getNodeTextContent(actionNode, CACHE_ACTION_INTEGERFACE);
				String className = this.getNodeTextContent(actionNode, CACHE_ACTION_CLASS);
				if ( "".equals(interfaceName) || "".equals(className) ){
					return;
				}
				List<Node> methodNodes = this.handler.getChildNodes(actionNode, CACHE_ACTION_METHOD);
				if ( methodNodes == null ) {
					return;
				}
				//method
				List<String> names = new ArrayList<String>();
				List<String> parameters = new ArrayList<String>();
				String methodName = null;
				String methodParameter = null;
				for ( Node method : methodNodes ) {
					methodName = this.handler.getNodeAttribute(method, CACHE_ACTION_METHOD_NAME);
					methodParameter = this.handler.getNodeAttribute(method, CACHE_ACTION_METHOD_PARAMETER);
					if ( PlatformTools.isNull(methodName) || 
							PlatformTools.isNull(methodParameter) ) {
						continue;
					}
					names.add(methodName);
					parameters.add(methodParameter);
				}
				if ( names.isEmpty() ) {
					return;
				}
				
				ActionControler.interfaceMap.put(actionName, Class.forName(interfaceName));
				ActionControler.instanceMap.put(actionName, Class.forName(className));
				ActionControler.threadlocalMap.put(actionName, new ThreadLocal<Object>());//为了保证灵活性，只能强制转换
				for ( int i = 0 ; i < parameters.size() ; i++ ) {
					ActionControler.function2ObjectMap.put(parameters.get(i), actionName);
					ActionControler.function2MethodMap.put(parameters.get(i), names.get(i));
				}
				
				List<String> pageUrls = new ArrayList<String>();
				List<Boolean> entrance = new ArrayList<Boolean>();
				List<String> pageNames = new ArrayList<String>();
				String urlValue = null;
				String entranceValue = null;
				String pageNameValue = null;
				List<Node> pageNodes = this.handler.getChildNodes(actionNode, CACHE_ACTION_PAGE);
				if ( pageNodes != null ) {
					for ( Node page : pageNodes ) {
						entranceValue = this.handler.getNodeAttribute(page, CACHE_ACTION_PAGE_ENTRANCE);
						urlValue = this.handler.getNodeAttribute(page, CACHE_ACTION_PAGE_URL);
						pageNameValue = this.handler.getNodeAttribute(page, CACHE_PUBLIC_PAGE_NAME);
						if ( PlatformTools.isNull(urlValue) ) {
							continue;
						}
						pageUrls.add(urlValue);
						entrance.add(entranceValue==null?false:"true".equals(entranceValue)?true:false);
						pageNames.add(pageNameValue==null?"":pageNameValue);
					}
				}
				
				for ( int i = 0 ; i < pageUrls.size() ; i++ ) {
					ActionControler.getInstance().setAdminBackPageUrl(pageUrls.get(i));
				}
			}
		}
		
	}
	
	/**
	 * init public page cache
	 * @param cacheNode
	 */
	protected void initPublicPage(Node cacheNode){
		List<String> publicPageUrls = new ArrayList<String>();
		List<Boolean> publicPageEnable = new ArrayList<Boolean>();
		String publicPageUrlValue = null;
		String publicPageEnableValue = null;
		List<Node> publicPageNodes = this.handler.getChildNodes(cacheNode, CACHE_PUBLIC_PAGE);
		if ( publicPageNodes != null ) {
			for ( Node page : publicPageNodes ) {
				publicPageUrlValue = this.handler.getNodeAttribute(page, CACHE_PUBLIC_PAGE_URL);
				publicPageEnableValue = this.handler.getNodeAttribute(page, CACHE_PUBLIC_PAGE_ENABLE);
				if ( PlatformTools.isNull(publicPageUrlValue) ) {
					continue;
				}
				publicPageUrls.add(publicPageUrlValue);
				publicPageEnable.add(publicPageEnableValue==null?false:"true".equals(publicPageEnableValue)?true:false);
			}
		}
		
		for ( int i = 0 ; i < publicPageUrls.size() ; i++ ) {
			ActionControler.getInstance().setCommonPageUrl(publicPageUrls.get(i));
		}
	}
	
	/**
	 * Get text form node
	 * @param parentNode
	 * @param childNodeName
	 * @return
	 */
	private String getNodeTextContent(Node parentNode, String childNodeName){
		if ( parentNode != null && childNodeName != null && !"".equals(childNodeName) ) {
			List<Node> childNodes = this.handler.getChildNodes(parentNode, childNodeName);
			if ( childNodes == null || childNodes.isEmpty() ) {
				return "";
			}
			Node childNode = childNodes.get(0);
			String childNodeValue = childNode.getTextContent();
			return childNodeValue==null?"":childNodeValue;
		}
		return "";
	}
}
