/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/**
 * Toolkit for properties files
 * @author jack lee
 * @version 1.0
 */
public class PropertiesTools {

	/**
	 * Read properties stream
	 * @param file
	 * @return
	 * @throws Exception
	 */
	public static String readStream(InputStream is) throws Exception{
		//InputStreamReader isReader = new InputStreamReader(is,"utf-8");
		InputStreamReader isReader = new InputStreamReader(is);
		BufferedReader br = new BufferedReader(isReader);
		String line = "";
		String[] values = null;
		StringBuilder sb = new StringBuilder(); 
		while ((line = br.readLine()) != null) {
			if ( !"".equals(line.trim()) ) {
				values = line.trim().split("=");
				if ( values.length == 2 ) {
					sb.append(values[0]).append("=").append(values[1]).append("\r\n");
				}
			}
		}
		br.close();
		isReader.close();
		is.close();
		return sb.toString();
	}
	
	/**
	 * Read properties file
	 * @param file
	 * @return
	 * @throws Exception
	 */
	public static String readFile(File file) throws Exception{
		InputStream is = new FileInputStream(file);
		//InputStreamReader isReader = new InputStreamReader(is,"utf-8");
		InputStreamReader isReader = new InputStreamReader(is);
		BufferedReader br = new BufferedReader(isReader);
		String line = "";
		String[] values = null;
		StringBuilder sb = new StringBuilder(); 
		while ((line = br.readLine()) != null) {
			if ( !"".equals(line.trim()) ) {
				values = line.trim().split("=");
				if ( values.length == 2 ) {
					sb.append(values[1]).append("\r\n");
				}
			}
		}
		br.close();
		isReader.close();
		is.close();
		return sb.toString();
	}
	
	/**
	 * Read key
	 * @param file 文件对象
	 * @return Key值列表
	 * @throws Exception
	 */
	public static List<String> readKey(File file) throws Exception{
		List<String> str = new ArrayList<String>();
		InputStream is = new FileInputStream(file);
		//InputStreamReader isReader = new InputStreamReader(is,"utf-8");
		InputStreamReader isReader = new InputStreamReader(is);
		BufferedReader br = new BufferedReader(isReader);
		String line = "";
		String[] values = null;
		while ((line = br.readLine()) != null) {
			if ( !"".equals(line.trim()) ) {
				values = line.trim().split("=");
				if ( values.length == 2 ) {
					str.add(values[0]);
				}
			}
		}
		br.close();
		isReader.close();
		is.close();
		return str;
	}
	
	/**
	 * Read value
	 * @param file
	 * @return 
	 * @throws Exception
	 */
	public static List<String> readValue(File file) throws Exception{
		List<String> str = new ArrayList<String>();
		InputStream is = new FileInputStream(file);
		InputStreamReader isReader = new InputStreamReader(is,"utf-8");
		//InputStreamReader isReader = new InputStreamReader(is);
		BufferedReader br = new BufferedReader(isReader);
		String line = "";
		while ((line = br.readLine()) != null) {
			if ( !"".equals(line.trim()) ) {
				str.add(line.trim());
			}
		}
		br.close();
		isReader.close();
		is.close();
		return str;
	}
	
	/**
	 * Transform list to key-value string
	 * @param keys
	 * @param values
	 * @return
	 * @throws Exception
	 */
	public static String key2value(List<String> keys,List<String> values) throws Exception{
		if ( keys.size() != values.size() ) {
			throw new Exception("Key and value are not match.");
		}
			
		StringBuilder sb = new StringBuilder(); 
		for( int i = 0 ; i < keys.size() ; i ++ ) {
			sb.append(keys.get(i)).append("=").append(values.get(i)).append("\r\n");
		}
		return sb.toString();
	}
	

	/**
	 * Save
	 * @param prop 
	 * @param uri 
	 * @param charset 
	 * @return
	 */
	public static boolean saveProperties(Properties prop, URI uri, String charset) 
			throws IOException {
		File file = new File(uri);
		if ( !file.exists() || !file.isFile() ) {
			return false;
		}
		if ( PlatformTools.isNull(charset) ) {
			charset = "utf-8";
		}
		FileOutputStream fos = new FileOutputStream(file);
		OutputStreamWriter osw = new OutputStreamWriter(fos,charset);
		PrintWriter pw  = new PrintWriter(osw);
		prop.store(pw, "Save by Roya on " + PlatformTools.getNow() +".");
		pw.close();
		osw.close();
		fos.close();
		pw = null;
		osw = null;
		fos = null;
		return true;
	}
	
	/**
	 * Save in unicode
	 * @param prop 
	 * @param uri 
	 * @return
	 */
	public static boolean saveUnicodeProperties(Properties prop, URI uri) 
			throws IOException {
		File file = new File(uri);
		if ( !file.exists() || !file.isFile() ) {
			return false;
		}
		FileOutputStream fos = new FileOutputStream(file);
		prop.store(fos, "Save by Roya on " + PlatformTools.getNow() +".");
		fos.close();
		fos = null;
		return true;
	}
	
	/**
	 * Save
	 * @param theString 
	 * @param escapeSpace 
	 * @param escapeUnicode 
     */
    public static String saveConvert(String theString,
                               boolean escapeSpace,
                               boolean escapeUnicode) {
        int len = theString.length();
        int bufLen = len * 2;
        if (bufLen < 0) {
            bufLen = Integer.MAX_VALUE;
        }
        StringBuffer outBuffer = new StringBuffer(bufLen);

        for(int x=0; x<len; x++) {
            char aChar = theString.charAt(x);
            // Handle common case first, selecting largest block that
            // avoids the specials below
            if ((aChar > 61) && (aChar < 127)) {
                if (aChar == '\\') {
                    outBuffer.append('\\'); outBuffer.append('\\');
                    continue;
                }
                outBuffer.append(aChar);
                continue;
            }
            switch(aChar) {
                case ' ':
                    if (x == 0 || escapeSpace)
                        outBuffer.append('\\');
                    outBuffer.append(' ');
                    break;
                case '\t':outBuffer.append('\\'); outBuffer.append('t');
                          break;
                case '\n':outBuffer.append('\\'); outBuffer.append('n');
                          break;
                case '\r':outBuffer.append('\\'); outBuffer.append('r');
                          break;
                case '\f':outBuffer.append('\\'); outBuffer.append('f');
                          break;
                case '=': // Fall through
                case ':': // Fall through
                case '#': // Fall through
                case '!':
                    outBuffer.append('\\'); outBuffer.append(aChar);
                    break;
                default:
                    if (((aChar < 0x0020) || (aChar > 0x007e)) & escapeUnicode ) {
                        outBuffer.append('\\');
                        outBuffer.append('u');
                        outBuffer.append(toHex((aChar >> 12) & 0xF));
                        outBuffer.append(toHex((aChar >>  8) & 0xF));
                        outBuffer.append(toHex((aChar >>  4) & 0xF));
                        outBuffer.append(toHex( aChar        & 0xF));
                    } else {
                        outBuffer.append(aChar);
                    }
            }
        }
        return outBuffer.toString();
    }
    
    /**
     * Convert a nibble to a hex character
     * @param   nibble  the nibble to convert.
     */
    private static char toHex(int nibble) {
        return hexDigit[(nibble & 0xF)];
    }

    /** A table of hex digits */
    private static final char[] hexDigit = {
        '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
    };
	
	/**
	 * Transform string to unicode
	 * @param string
	 * @return unicode string
	 */
	public static String string2Unicode(String string) {
	    StringBuffer unicode = new StringBuffer();
	    for (int i = 0; i < string.length(); i++) {
	        char c = string.charAt(i);
	        unicode.append("\\u" + Integer.toHexString(c));
	    }
	    return unicode.toString();
	}
	
	/**
	 * Transform unicode to string
	 * @param  unicode string
	 * @return string
	 */
	public static String unicode2String(String unicode) {
	    StringBuffer string = new StringBuffer();
	    String[] hex = unicode.split("\\\\u");
	    for (int i = 1; i < hex.length; i++) {
	        int data = Integer.parseInt(hex[i], 16);
	        string.append((char) data);
	    }
	    return string.toString();
	}

	/**
	 * Transform string to unicode
	 * @param string 
	 * @return unicode string
	 */
	public static String gbEncoding(final String gbString) {
		char[] utfBytes = gbString.toCharArray();
		String unicodeBytes = "";
		for (int byteIndex = 0; byteIndex < utfBytes.length; byteIndex++) {
			String hexB = Integer.toHexString(utfBytes[byteIndex]);
			if (hexB.length() <= 2) {
				hexB = "00" + hexB;
			}
			unicodeBytes = unicodeBytes + "\\u" + hexB;
		}
		System.out.println("unicodeBytes is: " + unicodeBytes);
		return unicodeBytes;
	}

	/**
	 * Transform unicode to string
	 * @param  unicode string
	 * @return string
	 */
	public static String decodeUnicode(final String dataStr) {
		int start = 0;
		int end = 0;
		final StringBuffer buffer = new StringBuffer();
		while (start > -1) {
			end = dataStr.indexOf("\\u", start + 2);
			String charStr = "";
			if (end == -1) {
				charStr = dataStr.substring(start + 2, dataStr.length());
			} else {
				charStr = dataStr.substring(start + 2, end);
			}
			char letter = (char) Integer.parseInt(charStr, 16); // 16进制parse整形字符串。
			buffer.append(new Character(letter).toString());
			start = end;
		}
		return buffer.toString();
	}
	
	
}
