/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.database;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.config.bean.OSMObject;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.define.IOrmAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * Database action
 * @author jack lee
 * @version 1.0
 */
public class DatabaseAction implements IDatabase {
	
	protected Logger logger;
	
	public static Map<String,Class<?>> table2pojo;
	
	public DatabaseAction(){
		logger = logger.getInstance();
	}
	
	@Override
	public Database getDatabaseInfo() throws CoreException {
		Connection conn = null;
		Database database = new Database();
		try {
			conn = MultiConnectionPool.getInstance().getConnection(
							MultiConnectionPool.getInstance().getDefaultDataSourceName());
			
			DatabaseMetaData metadata = conn.getMetaData();
			database.setDirverVersion(metadata.getDriverVersion());
			database.setProductName(metadata.getDatabaseProductName());
			database.setProductVersion(metadata.getDatabaseProductVersion());
			database.setCatalog(conn.getCatalog());
			
			ResultSet rscol = null;
			ResultSet rskey = null;
			TableBean t = null;
			FieldBean f = null;		
			List<FieldBean> fs = null;
			Map<String,FieldBean> fm = null;
			String colName = null;
			List<TableBean> tables = new ArrayList<TableBean>();
			
			ResultSet rstable = metadata.getTables(database.getCatalog(), null, null, new String[]{"TABLE"});
            while (rstable.next()) {   
            	t = new TableBean();
            	t.setName(rstable.getString("TABLE_NAME"));
            	t.setDescription(rstable.getString("REMARKS"));
            	
            	rscol = metadata.getColumns(database.getCatalog(), null, t.getName(), null);

            	fs = new ArrayList<FieldBean>();
            	fm = new HashMap<String,FieldBean>();
            	while ( rscol.next() ) {
//            		ResultSetMetaData rsmd = rscol.getMetaData();
//            		int size = rsmd.getColumnCount();
//            		for ( int i = 1 ; i < size ; i++ ) {
//            			//System.out.println(rsmd.getColumnName(i)+" = "+rscol.getString(i) + " >> autoIncrement : " + rsmd.isAutoIncrement(i));
//            		}
            		f = new FieldBean();
            		f.setJdbcType(rscol.getInt("DATA_TYPE"));
            		f.setName(rscol.getString("COLUMN_NAME"));
            		f.setSize(rscol.getInt("COLUMN_SIZE"));
            		f.setNotNull(!rscol.getBoolean("IS_NULLABLE"));
            		f.setAutoIncrement(rscol.getBoolean("IS_AUTOINCREMENT"));
            		f.setRemarks(rscol.getString("REMARKS"));
            		f.setUnique(false);
            		f.setPrimaryKey(false);
            		fs.add(f);
            		fm.put(f.getName(), f);
            	}
            	rscol.close();
            	rscol = null;
            	
            	rskey = metadata.getPrimaryKeys(database.getCatalog(), null, t.getName());
            	while ( rskey.next() ) {
            		ResultSetMetaData rsmd = rskey.getMetaData();
//            		int size = rsmd.getColumnCount();
//            		for ( int i = 1 ; i < size ; i++ ) {
//            			
//            			//System.out.println(rsmd.getColumnName(i)+" = "+ rskey.getString(i) + " >> autoIncrement : " + rsmd.isAutoIncrement(i));
//            		}
//            		System.out.println(rskey.getString("TABLE_NAME") + " >> " + rskey.getString("COLUMN_NAME"));
            		colName = rskey.getString("COLUMN_NAME");
            		if ( fm.get(colName) != null ) {
            			fm.get(colName).setPrimaryKey(true);
            		}
            	}
            	rskey.close();
            	rskey = null;
            	
            	t.setFields(fs);
            	tables.add(t);
            }  
            database.setTables(tables);
            rstable.close();   
            rstable = null;
		} catch (Exception e) {
			logger.error("Can not load database info.", e);
		} finally {
			if ( conn != null ) {
				try {
					conn.close();
				} catch (SQLException e) {
					Logger.getInstance().error("There is an error on closing connection.", e);
				}
			}
		}
		return database;
	}
	
	/* (non-Javadoc)
	 * @see com.nonesole.web.platform.sys.database.IDatabase#getDatabaseInfo()
	 */
	@Override
	public void getDatabaseInfo(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		request.setAttribute(IDatabase.ATTR_DATA_BASE_INFO, this.getDatabaseInfo());
	}

	/* (non-Javadoc)
	 * @see com.nonesole.web.platform.sys.database.IDatabase#fullBackup()
	 */
	@Override
	public void fullBackup(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Database database = this.getDatabaseInfo();
		Connection conn = null;
		try {
			conn = MultiConnectionPool.getInstance().getConnection(
					MultiConnectionPool.getInstance().getDefaultDataSourceName());
		} catch (OperationsException e) {
			logger.error("Can not get connection.", e);
		}
		StringBuilder sb = new StringBuilder();
		sb.append("# Roya(nonesole.com) for ")
		  .append(database.getProductName())
		  .append(" ")
		  .append(database.getProductVersion()).append("\r\n");
		sb.append("# ---------------------------------------").append("\r\n");
		try {
			sb.append("# URL : ").append(conn.getMetaData().getURL()).append("\r\n");
		} catch (SQLException e) {
			logger.error("Can not get metadata from connection.", e);
		}
		sb.append("# Backup time: ").append(PlatformTools.getNow()).append("\r\n");
		sb.append("\r\n").append("\r\n");
		
		//SET FOREIGN_KEY_CHECKS=0;
		sb.append("#SET FOREIGN_KEY_CHECKS=0;").append("\r\n");
		sb.append("\r\n");
		
		//DROP DATABASE IF EXISTS `platform`;
		//CREATE DATABASE `platform`
		//CHARACTER SET 'utf8'
		//COLLATE 'utf8_general_ci';
		//USE `platform`;
		
		sb.append("# ").append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.database.DatabaseAction_info_1")).append("\r\n");
		sb.append("#DROP DATABASE IF EXISTS platform;").append("\r\n");
		sb.append("#CREATE DATABASE platform").append("\r\n");
		sb.append("#    CHARACTER SET 'utf8'").append("\r\n");
		sb.append("#    COLLATE 'utf8_general_ci';").append("\r\n");
		sb.append("\r\n");
		
		sb.append("USE platform;").append("\r\n");
		sb.append("\r\n");
		
		if ( database != null ) {
			List<TableBean> tables = database.getTables();
			if ( tables != null ) {
				for ( TableBean table : tables ) {
					//#
					//# Structure for the `access_info` table : 
					//#
					sb.append("#").append("\r\n")
					  .append("# Structure for the `").append(table.getName()).append("` table :").append("\r\n")
					  .append("#").append("\r\n");
					sb.append("\r\n");
					try {
						sb.append("# ").append(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.database.DatabaseAction_info_1")).append("\r\n");
						sb.append("#").append(this.getDeleteSql(table)).append("\r\n");
						sb.append(this.getCreateSql(table)).append("\r\n");
					} catch (Exception e) {
						logger.error("Can not create sql.", e);
					}
				}
				
				if ( table2pojo == null ) {
					table2pojo = new HashMap<String,Class<?>>();
					List<OSMObject> olist = ConfigLoader.getInstance().getOSMList();
					if ( olist != null ) {
						for ( OSMObject obj : olist ) {
							try {
								table2pojo.put(obj.getTableName(), Class.forName(obj.getClassName()));
							} catch (ClassNotFoundException e) {
								logger.error("Can not create table2pojo map.", e);
							}
						}
					}
					
				}
				
				IOrmAction action = PersistenceFactory.getInstance().getOrmAction();
				Class<?> clazz = null;
				List<?> objs = null;
				//获取插入数据SQL
				for ( TableBean table : tables ) {
					clazz = table2pojo.get(table.getName());
					if ( clazz != null ) {
						//#
						//# Data for the `access_info` table  (LIMIT 0,500)
						//#
						sb.append("#").append("\r\n")
						  .append("# Data for the `").append(table.getName()).append("` table").append("\r\n")
						  .append("#").append("\r\n");
						sb.append("\r\n");
						
						try {
							if ( table.getName().equals("sys_menu") || table.getName().equals("sys_users_info") ) {
								logger.debug(">>> " + table.getName(), null);
							}
							objs = action.query("select * from " + table.getName(), clazz);
							sb.append(this.getInsertSqlByList(table, objs)).append("\r\n");
							sb.append("\r\n");
						} catch (ExecuteSQLException e) {
							logger.error("Can not query data of table " + table.getName(), e);
						}
					}
				}
			}
		}
		
		//logger.debug("sql >>>>> \r\n" + sb.toString(), null);
		request.setAttribute(IDatabase.ATTR_DATA_BASE_SQL, sb.toString());
	}

	/**
	 * Get insert sql 
	 * @param table
	 * @param obj
	 * @return
	 */
	protected String getInsertSqlByList(TableBean table, List<?> objs) {
		if ( table == null || objs == null || objs.isEmpty() ) {
			return "";
		}
		StringBuilder sb = new StringBuilder();
		for ( Object obj : objs ) {
			sb.append(this.getInsertSqlByObject(table, obj));
			sb.append("\r\n");
		}
		return sb.toString();
	}
	
	/**
	 * Get delete sql
	 * @param table
	 * @param obj
	 * @return
	 */
	protected String getInsertSqlByObject(TableBean table, Object obj) {
		Class<?> c = obj.getClass();
		java.lang.reflect.Field[] fields = c.getDeclaredFields();
		if ( fields == null ) {
			return "";
		}
		String name;
		Object value;
		Class<?> type = null;
		StringBuilder dbField = new StringBuilder();
		StringBuilder values = new StringBuilder();
		for( Field field : fields ){
			name = field.getName();
			type = field.getType();
			int modifier = field.getModifiers();
			if ( Modifier.isFinal(modifier) ||
					(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
				continue;
			} 
			dbField.append(name).append(",");
			try {
				value = DefaultBaseAction.invoke(field, obj);
				this.handleCharacter(type,values,value);
			} catch (Exception e) {
				logger.error("There is an error on obj2sql for insert action.", e);
			}
			values.append(",");			
		} 
		int point = dbField.lastIndexOf(",");
		dbField.delete(point, point+1);
		point = values.lastIndexOf(",");
		values.delete(point, point+1);
		
		//sql
		StringBuilder sql = new StringBuilder();
		sql.append("insert into")
			.append(" ")
			.append(table.getName())
			.append("(")
			.append(dbField.toString())
			.append(") values (")
			.append(values.toString())
			.append(");");
		return sql.toString();
	}
	
	/**
	 * Check field whether or not needs '' around it's value
	 * @param fieldType  field type
	 * @param sb string which needs join
	 * @param value value which will join the string
	 * @throws ExecuteSQLException
	 * */
	protected void handleCharacter(
			Class<?> fieldType,StringBuilder sb,Object value) 
	throws ExecuteSQLException{
		if ( CommonTools.isNull(fieldType) ) {
			throw new ExecuteSQLException("Fieldtype is null.");
		}
		if ( CommonTools.isNull(sb) ) {
			throw new ExecuteSQLException("StringBuilder is null.");
		}
		if ( CommonTools.isNull(value) ) {
			if ( fieldType == String.class 
					|| fieldType == Date.class 
					|| fieldType == Time.class
					|| fieldType == Timestamp.class 
					|| fieldType == Character.class
					|| fieldType == char.class)	{
				value = "";
			} else if ( boolean.class == fieldType || Boolean.class == fieldType
					 	|| int.class == fieldType || Integer.class == fieldType 
					 	|| long.class == fieldType || Long.class == fieldType
					 	|| float.class == fieldType || Float.class == fieldType
					 	|| double.class == fieldType || Double.class == fieldType
					 	|| short.class == fieldType || Short.class == fieldType
					 	|| Byte.class == fieldType || byte.class == fieldType
					 	|| BigDecimal.class == fieldType 
					 	|| BigInteger.class == fieldType ) {
				value = 0;
			} else if ( fieldType.isArray() ) {
				value = "";
			} else {
				value = "";
			}
		}
		
		if ( fieldType == String.class 
				|| fieldType == Date.class 
				|| fieldType == Time.class
				|| fieldType == Timestamp.class 
				|| fieldType == Character.class
				|| fieldType == char.class)	{
			sb.append("'").append(this.tranformAscii(value)).append("'");
		} else if ( boolean.class == fieldType || Boolean.class == fieldType
				 	|| int.class == fieldType || Integer.class == fieldType 
				 	|| long.class == fieldType || Long.class == fieldType
				 	|| float.class == fieldType || Float.class == fieldType
				 	|| double.class == fieldType || Double.class == fieldType
				 	|| short.class == fieldType || Short.class == fieldType
				 	|| Byte.class == fieldType || byte.class == fieldType
				 	|| BigDecimal.class == fieldType 
				 	|| BigInteger.class == fieldType ) {
			sb.append(value);
		} else if ( fieldType.isArray() ) {
			Class<?> arrayType = fieldType.getComponentType();
			int length = Array.getLength(value);
			if ( arrayType.isPrimitive() ) {
				//if value is primitive
				for ( int i = 0 ; i < length ; i++ ) {
					sb.append(Array.get(value, i));
				}
			} else {
				for ( int i = 0 ; i < length ; i++ ) {
					sb.append(Array.get(value, i).toString());
				}
			}
			
		}else {
			sb.append(value.toString());
		}
	}
	
	protected String tranformAscii(Object value){
		if ( value != null && !"".equals(value) ) {
			String s = value.toString();
			if ( s.indexOf("html") > -1 ) {
				System.out.println(s);
			}
			s = s.replace("\\", "\\\\");
			s = s.replaceAll("\t", "\\\\t");
			s = s.replaceAll("\n", "\\\\n");
			s = s.replaceAll("\r", "\\\\r");
			s = s.replaceAll("\b", "\\\\b");
			s = s.replaceAll("\"", "\\\\\"");
			s = s.replaceAll("'", "''");
			s = s.replaceAll("\f", "\\\\f");
			return s;
		}
		return "";
	}
	
	/**
	 * Get create sql
	 * @param t
	 * @return
	 * @throws Exception
	 */
	protected String getCreateSql(TableBean t) throws Exception {
		StringBuilder sb = new StringBuilder();
		sb.append("create table if not exists ").append(t.getName()).append("(");
		sb.append("\r\n");
		List<FieldBean> fs = t.getFields();
		StringBuilder pk = new StringBuilder();
		boolean isCompositePrimaryKey = t.isCompositePrimaryKey();
		if ( isCompositePrimaryKey ) {
			pk.append("primary key (");
		}
		for ( FieldBean f : fs ) {
			sb.append("  `").append(f.getName()).append("` ")
			  .append(f.getJdbcTypeName());
			if ( f.getJdbcType() == Types.VARCHAR ) {
				sb.append("(").append(f.getSize()).append(")");
			}
			sb.append(" ");
			if ( f.isPrimaryKey() ) {
				if ( !isCompositePrimaryKey ) {
					sb.append(" primary key ");
				} else {
					pk.append("`").append(f.getName()).append("`,");
				}
			} else {
				if ( f.isNotNull() ) {
					sb.append(" not null ");
				}
				if ( f.isAutoIncrement() ) {
					sb.append(" auto_increment ");
				}
				if ( f.isUnique() ) {
					sb.append(" identity ");
				}
			}
			sb.append(" comment '").append(f.getRemarks()).append("'");
			sb.append(",");		
			sb.append("\r\n");
		}
		if ( isCompositePrimaryKey ) {
			if ( pk.lastIndexOf(",") > -1 )
				pk.deleteCharAt(pk.lastIndexOf(","));
			pk.append(")");
			sb.append(pk.toString()).append("\r\n");
		} else {
			if ( sb.lastIndexOf(",") > -1 )
				sb.deleteCharAt(sb.lastIndexOf(","));
		}
		//System.out.println(sb.lastIndexOf(",")+","+sb.capacity());
		sb.append(")");
		sb.append(" charset=utf8 comment='")
		  .append(t.getDescription()).append("';");
		sb.append("\r\n");
		return sb.toString();
	}

	/**
	 * Get drop-table-sql
	 * @param t
	 * @return
	 * @throws Exception
	 */
	protected String getDeleteSql(TableBean t) throws Exception {
		StringBuilder sb = new StringBuilder();
		sb.append("drop table if exists ").append(t.getName()).append(";");
		sb.append("\r\n");
		//logger.debug("Delete sql : " + sb.toString(), null);
		return sb.toString();
	}

	/**
	 * Get alert-table-sql
	 * @param t
	 * @return
	 * @throws Exception
	 */
	protected String getAlertSql(TableBean t) throws Exception {
		StringBuilder sb = new StringBuilder();
		sb.append(this.getCreateSql(t));
		List<FieldBean> fs = t.getFields();
		for ( FieldBean f : fs ) {
			sb.append("alter table ").append(t.getName())
			  .append(" add column if not exists ").append(f.getName())
			  .append(" ").append(f.getJdbcTypeName());
			if ( f.getJdbcType() == Types.VARCHAR ) {
				sb.append("(").append(f.getSize()).append(")");
			}
			sb.append(" ");
			if ( f.isPrimaryKey() ) {
				sb.append(" primary key ");
			} else {
				if ( f.isNotNull() ) {
					sb.append(" not null ");
				}
				if ( f.isAutoIncrement() ) {
					sb.append(" auto_increment ");
				}
				if ( f.isUnique() ) {
					sb.append(" identity ");
				}
			}
			sb.append("");
			sb.append("\r\n");
		}
		sb.append(" charset=utf8 comment='")
		  .append(t.getDescription()).append("';");
		sb.append("\r\n");
		logger.debug("Alert sql : " + sb.toString(), null);
		return sb.toString();
	}
	
	/**
	 * Print in the console
	 * @param rs
	 * @throws SQLException 
	 */
	protected void print(ResultSet rs) throws SQLException{
		ResultSetMetaData rsmd = rs.getMetaData();
    	for ( int i = 1 ;i <= rsmd.getColumnCount() ; i++ ) {
    		//logger.debug("===>>> ^^^^^^^^^^^^^^^^", null);
    		//logger.debug("===>>> column name = " + rsmd.getColumnName(i), null);
    		//logger.debug("===>>> column classname = " + rsmd.getColumnClassName(i), null);
    		//logger.debug("===>>> column label = " + rsmd.getColumnLabel(i), null);
    		//logger.debug("===>>> column display size = " + rsmd.getColumnDisplaySize(i), null);
    		//logger.debug("===>>> column type = " + rsmd.getColumnType(i), null);
    		//logger.debug("===>>> column type name = " + rsmd.getColumnTypeName(i), null);
    		//logger.debug("===>>> column precision = " + rsmd.getPrecision(i), null);
    		//logger.debug("===>>> column scale = " + rsmd.getScale(i), null);
    	}
	}

	
	
}
