/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.database;

import java.io.IOException;
import java.sql.Types;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;

import com.nonesole.log.Logger;

/**
 * Database tools
 * @author jack lee
 * @version 0.1
 */
public class DatabaseUtil {

	public static Map<String,String> java2db;
	
	public static Map<String,String> db2java;
	
	public static Map<String,Integer> dbTypeInt;
	
	public static Map<Integer,String> dbTypeStr;
	
	static {
		java2db = new HashMap<String,String>();
		db2java = new HashMap<String,String>();
		dbTypeInt = new HashMap<String,Integer>();
		dbTypeStr = new HashMap<Integer,String>();
		
		Properties prop = new Properties();
		try {
			prop.load(DatabaseUtil.class
					.getResourceAsStream("relation.properties"));
		} catch (IOException e) {
			Logger.getInstance().error("There is an error on loading properties of database.", e);
			//e.printStackTrace();
		}
		Set<Entry<Object,Object>> ens = prop.entrySet();
		for ( Entry<Object,Object> en : ens ) {
			java2db.put((String)en.getKey(), (String)en.getValue());
			db2java.put((String)en.getValue(), (String)en.getKey());
			int type = -1;
			if ( "TIMESTAMP".equals(en.getValue()) ) {
				type = Types.TIMESTAMP;
			} else if ( "CLOB".equals(en.getValue()) ) {
				type = Types.CLOB;
			} else if ( "BLOB".equals(en.getValue()) ) {
				type = Types.BLOB;
			} else if ( "VARCHAR".equals(en.getValue()) ) {
				type = Types.VARCHAR;
			} else if ( "BIT".equals(en.getValue()) ) {
				type = Types.BIT;
			} else if ( "INTEGER".equals(en.getValue()) ) {
				type = Types.INTEGER;
			} else if ( "BIGINT".equals(en.getValue()) ) {
				type = Types.BIGINT;
			} else if ( "REAL".equals(en.getValue()) ) {
				type = Types.REAL;
			} else if ( "DOUBLE".equals(en.getValue()) ) {
				type = Types.DOUBLE;
			} else if ( "ARRAY".equals(en.getValue()) ) {
				type = Types.ARRAY;
			} else if ( "STRUCT".equals(en.getValue()) ) {
				type = Types.STRUCT;
			} else if ( "REF".equals(en.getValue()) ) {
				type = Types.REF;
			} else if ( "DATALINK".equals(en.getValue()) ) {
				type = Types.DATALINK;
			} else if ( "JAVA_OBJECT".equals(en.getValue()) ) {
				type = Types.JAVA_OBJECT;
			} else if ( "ROWID".equals(en.getValue()) ) {
				type = Types.ROWID;
			} else if ( "NCLOB".equals(en.getValue()) ) {
				type = Types.NCLOB;
			} else if ( "SQLXML".equals(en.getValue()) ) {
				type = Types.SQLXML;
			} else if ( "NUMERIC".equals(en.getValue()) ) {
				type = Types.NUMERIC;
			} else if ( "TINYINT".equals(en.getValue()) ) {
				type = Types.TINYINT;
			} else if ( "SMALLINT".equals(en.getValue()) ) {
				type = Types.SMALLINT;
			} else if ( "VARBINARY".equals(en.getValue()) ) {
				type = Types.VARBINARY;
			} else if ( "DATE".equals(en.getValue()) ) {
				type = Types.DATE;
			} else if ( "TIME".equals(en.getValue()) ) {
				type = Types.TIME;
			} else {
				type = Types.VARCHAR;
			}
			
			dbTypeInt.put((String)en.getValue(), type);
			dbTypeStr.put(type, (String)en.getValue());
		}
		
	}
	
	/**
	 * Get data type
	 * @param javaType 
	 * @return 
	 */
	public static String getDataType(String javaType) {
		String dt = java2db.get(javaType);
		return dt==null?"NOT-STANDARD-JDBC4-DATA-TYPE":dt;
	}
	
	/**
	 * Get java type
	 * @param dataType 
	 * @return 
	 */
	public static String getJavaType(String dataType) {
		String jt = db2java.get(dataType);
		return jt==null?"java.lang.String":jt;
	}
	
	/**
	 * Get string of data type
	 * @param dataType 
	 * @return 
	 */
	public static String getDataTypeString(Integer dataType) {
		String dts = dbTypeStr.get(dataType);
		return dts==null?"NOT-STANDARD-JDBC4-DATA-TYPE":dts;
	}
	
	/**
	 * Get constant of data type
	 * @param dataType 
	 * @return 
	 */
	public static Integer getDataTypeInteger(String dataType) {
		Integer dti = dbTypeInt.get(dataType);
		return dti==null?Types.VARCHAR:dti;
	}
}
