/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.menu;

import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.web.platform.basic.IPojo;


/**
 * Menu POJO
 * @author jack lee
 * @version 1.0
 * @date 2016-10-10
 */
public class Menu extends ExtensionMenu  implements IPojo {

	private int id;
	private String name;
	private String url;
	private int pid;
	private String ico;

	/**
	 * @return the selected
	 */
	public boolean isSelected() {
		return selected;
	}
	/**
	 * @param selected the selected to set
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}
	/**
	 * @return the ico
	 */
	public String getIco() {
		return ico==null?"":ico;
	}
	/**
	 * @param ico the ico to set
	 */
	public void setIco(String ico) {
		this.ico = ico;
	}
	/**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}
	/**
	 * @param url the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}
	/**
	 * @return the pid
	 */
	public int getPid() {
		return pid;
	}
	/**
	 * @param pid the pid to set
	 */
	public void setPid(int pid) {
		this.pid = pid;
	}
	/**
	 * @return the level
	 */
	public int getLevel() {
		return level;
	}
	/**
	 * @param level the level to set
	 */
	public void setLevel(int level) {
		this.level = level;
	}
	/**
	 * @return the leaf
	 */
	public int getLeaf() {
		return leaf;
	}
	/**
	 * @param leaf the leaf to set
	 */
	public void setLeaf(int leaf) {
		this.leaf = leaf;
	}
	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}
	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}
	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * @return
	 */
	public boolean validate(int action){
		if ( action == INSERT ) {
			return !CommonTools.isNull(this.name) 
					|| this.level == 0 
					|| this.leaf == 0;
		} else if ( action == UPDATE ) {
			if ( id < 1 && CommonTools.isNull(this.name) ) {
				return false;
			}
		} else if ( action == DELETE ) {
			if ( id < 1 ) {
				return false;
			}
		}
		return true;
	}
	
	public static final int INSERT = 1;
	public static final int UPDATE = 2;
	public static final int DELETE = 3;
	
	/**
	 * @return
	 */
	public String toString(){
		if ( level < 1 ) {
			return this.name;
		}
		StringBuilder sb = new StringBuilder();
		for ( int i = 1 ; i < level ; i++ ) {
			sb.append("&nbsp;&nbsp;&nbsp;&nbsp;");
		}
		if ( this.leaf == 0 )
			sb.append("<img src=\"../img/folder01.png\"/>");
		else
			sb.append("<img src=\"../img/file01.png\"/>");
		sb.append(this.name);
		return sb.toString();
	}
	
	/**
	 * @param obj
	 */
	public boolean equals(Object obj){
		if ( obj == null ) {
			return false;
		} else {
			if ( !(obj instanceof Menu) ) {
				return false;
			} else {
				return this.id == ((Menu)obj).getId();
			}
		}
	}
	@Override
	public int getObjectId() {
		return id;
	}
}
