/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.menu;

import java.sql.Connection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IPojoAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.ActionControler;
import com.nonesole.web.platform.basic.BasicAction;
import com.nonesole.web.platform.basic.ICommon;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.pages.Page2Condtion;
import com.nonesole.web.platform.pages.Page2Parameter;
import com.nonesole.web.platform.pages.Pages2;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;
import com.nonesole.web.platform.sys.rightset.RightSetEnum;

/**
 * Menu action
 * @author jack lee
 * @version 1.0
 * @date 2016-10-09
 */
public class MenuAction extends BasicAction implements IMenu {

	public MenuAction(){
		super();
	}
	
	@Override
	public void add(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		this.save(Menu.class, request, response);
		ActionControler.getInstance().init();
		this.redirect(request, response, PAGE_QUERY_2);
	}

	@Override
	public void update(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		this.update(Menu.class, request, response);
		ActionControler.getInstance().init();
		this.redirect(request, response, PAGE_QUERY_2);
	}

	@Override
	public void query(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Page2Parameter name = this.createKeyWordParam(request, DAO_PARAM_NAME);
		
		Page2Condtion condition = new Page2Condtion() {
			@Override
			public Page2Condtion createPreparedSqlCondition() {
				if (!PlatformTools.isNull(this.getUrlCondition())
						&& this.isPrepared) {
					StringBuilder sb = new StringBuilder();
					sb.append(" where ");
					int index = 1;
					for (Page2Parameter pp : this.list) {
						sb.append(pp.getComplexPreparedSqlCondition());
						if (index < this.list.size()) {
							sb.append(" and ");
						}
						index++;
					}
					this.daoConditionValue = sb.toString();
				}
				return this;
			}
		};
		condition.setDaoConditionKey(ICommon.DAO_SQL_CONDITION)
				.add(name);

		Pages2<Menu> page = new Pages2<Menu>()
				.setpCondition(condition)
				.setRequest(request)
				.setUrl(PAGE_QUERY)
				.setTotalRowsDaoName(DAO_METHOD_GET_TOTAL_ROWS)
				.setResultsDaoName(DAO_METHOD_GET_RESULTS)
				.initAll();

		if ( page.getObjects() != null ) {
			MenuTree mt = new MenuTree(page.getObjects());
			page.setObjects(mt.getObjects(request.getLocale()));
		}
		request.setAttribute(ICommon.ATTR_PAGES, page);
	}
	
	@Override
	public void delete(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		String parameter = MenuEnum.PARAM_MENU_ID.toString();
		String sqlPart = RightSetEnum.PARAM_CONDITION_ID.toString();
		String url = MenuEnum.URL_MENUS.toString();
		boolean validate = true;
		try {
			validate = validate && this.checkRelation(RightSetEnum.DAO_FIND_FUNCTIONS_BY_MENU_ID.toString(), parameter, sqlPart, url, request, response);
			validate = validate && this.checkRelation(RightSetEnum.DAO_FIND_ROLES_BY_MENU_ID.toString(), parameter, sqlPart, url, request, response);
			validate = validate && this.checkChildNode(MenuEnum.DAO_GET_ALL_ROWS.toString(), parameter, DAO_PARAM_CONDITION, url, request, response);
		} catch (ExecuteSQLException e) {
			Logger.getInstance().error("There is an error on deleting menu.", e);
			//e.printStackTrace();
		}
		if ( validate ) {
			this.deleteByIdArray(parameter, Menu.class, request, response);
			ActionControler.getInstance().init();
			this.redirect(request, response, url);
		}
	}

	@Override
	public void getMenuDetail(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		@SuppressWarnings("unchecked")
		IPojoAction<Menu> acts = (IPojoAction<Menu>) PersistenceFactory.getInstance().getPojoAction();
		Map<String,Object> c = new HashMap<String,Object>();
		c.put(MenuEnum.PARAM_MENU_ID.toString(), 
				PlatformTools.toInt(request.getParameter(MenuEnum.PARAM_MENU_ID.toString())));
		Menu r = null;
		Map<String, Object> emptyMap = new HashMap<String, Object>();
		MenuTree mt = null;
		List<Menu> l = null;
		try {
			Connection conn = null;
			try {
				conn = MultiConnectionPool.getInstance().getConnection(
						MultiConnectionPool.getInstance().getDefaultDataSourceName());
			} catch (OperationsException e) {
				PlatformTools.setCookieMessage(response, 
						LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.menu.MenuAction_info_11"));
				return;
			}
			
			r = acts.pojoAction(MenuEnum.DAO_GET_MENU_DETAILS.toString(), c, conn, false);
			l = acts.pojoListAction(MenuEnum.DAO_FIND_MENUS.toString(),emptyMap);
		} catch (ExecuteSQLException e) {
			logger.error("[MenuAction:getMenuDetail]", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.menu.MenuAction_info_5"));
		}
		if ( l != null ) {
			mt = new MenuTree(l);
		}		
		request.setAttribute(MenuEnum.ATTR_MENU_TREE.toString(), mt);
		request.setAttribute(MenuEnum.ATTR_MENU_DETAIL.toString(), r);
	}
	
	@Override
	public void getMenuTree(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		@SuppressWarnings("unchecked")
		IPojoAction<Menu> acts = (IPojoAction<Menu>) PersistenceFactory.getInstance().getPojoAction();
		Map<String, Object> emptyMap = new HashMap<String, Object>();
		MenuTree mt = null;
		List<Menu> l = null;
		try {
			l = acts.pojoListAction(MenuEnum.DAO_FIND_MENUS.toString(),emptyMap);
		} catch (ExecuteSQLException e) {
			logger.error("[MenuAction:getMenuDetail]", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.menu.MenuAction_info_5"));
		}
		if ( l != null ) {
			mt = new MenuTree(l);
		}		
		request.setAttribute(MenuEnum.ATTR_MENU_TREE.toString(), mt);
	}
}
