/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.rightset;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.web.platform.sys.CoreException;

/**
 * RightSet interface
 * @author jack lee
 * @version 1.0
 * @date 2016-10-15
 */
public interface IRightSet {

	/**
	 * Find user-role-relation by user's id
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void findUserRoleByUserId(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Update relations of user and roles
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void updateUserRoleForUser(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Find user-role-relation by role's id
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void findUserRoleByRoleId(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Update relations of users and role
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void updateUserRoleForRole(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Find role-menu-relation by role's id
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void findRoleMenuByRoleId(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Update relations of menus and role
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void updateRoleMenuForRole(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Find role-menu-relation by menu's id
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void findRoleMenuByMenuId(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Update relations of menu and roles
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void updateRoleMenuForMenu(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Find menu-function-relation by menu's id
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void findMenuFunctionByMenuId(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Update relations of functions and menu
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void updateMenuFunctionForMenu(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Find menu-function-relation by function's id
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void findMenuFunctionByFunctionId(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Update relations of function and menus
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void updateMenuFunctionForFunction(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
	/**
	 * Find relations of urls and roles
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public Map<String,Integer> getUrl2RoleIdRelation() throws CoreException;
	
	/**
	 * Find relations of parameter-function-codes and roles
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public Map<String,Integer> getActionParameter2RoleIdRelation() throws CoreException;
	
	/**
	 * Find url of page for menus
	 * @param session
	 * @param request
	 * @param response
	 * @throws CoreException
	 */
	public void getFunctionPageUrlForMenuItem(HttpSession session,
			HttpServletRequest request,
			HttpServletResponse response) throws CoreException;
	
}
