/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.rightset;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.define.IListValueAction;
import com.nonesole.persistence2.sql.define.IMapAction;
import com.nonesole.persistence2.sql.define.IOrmAction;
import com.nonesole.persistence2.sql.define.IPojoAction;
import com.nonesole.persistence2.sql.define.IVoidAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.ActionControler;
import com.nonesole.web.platform.ToolsAction;
import com.nonesole.web.platform.ParameterEnum;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;
import com.nonesole.web.platform.sys.function.Function;
import com.nonesole.web.platform.sys.function.FunctionEnum;
import com.nonesole.web.platform.sys.menu.Menu;
import com.nonesole.web.platform.sys.menu.MenuEnum;
import com.nonesole.web.platform.sys.menu.MenuTree;
import com.nonesole.web.platform.sys.role.Role;
import com.nonesole.web.platform.sys.role.RoleEnum;
import com.nonesole.web.platform.sys.user.User;
import com.nonesole.web.platform.sys.user.UserEnum;

/**
 * RightSet action
 * @author jack lee
 * @version 1.0
 * @date 2016-10-15
 */
public class RightSetAction extends ToolsAction implements IRightSet {

	public RightSetAction(){
		super();
	}
	
	/**
	 * Get id list
	 * @param request
	 * @return
	 */
	protected List<Integer> getId(HttpServletRequest request,String parameter){
		List<Integer> l = new ArrayList<Integer>();
		String fromIdStr = request.getParameter(parameter);
		if ( fromIdStr != null && !"".equals(fromIdStr) ) {
			String[] array = fromIdStr.split(",");
			for ( String s : array ) {
				if ( CommonTools.isNumber(s) ) {
					l.add(Integer.valueOf(s));
				}
			}
		}
		return l;
	}
	
	/**
	 * Get insert object
	 * @param actionName
	 * @param fromId
	 * @param targetId
	 * @return
	 */
	protected List<?> getInsertObjects(String actionName, List<Integer> fromId, List<Integer> targetId) {
		if ( RightSetEnum.METHOD_UPDATE_USER_ROLE_FOR_USER.toString().equals(actionName) ) {
			List<UserRole> list = new ArrayList<UserRole>();
			UserRole obj = null;
			for ( Integer fId : fromId ) {
				for ( Integer tId : targetId ) {
					obj = new UserRole();
					obj.setUser_id(fId);
					obj.setRole_id(tId);
					list.add(obj);
				}
			}
			return list;
		} else if ( RightSetEnum.METHOD_UPDATE_USER_ROLE_FOR_ROLE.toString().equals(actionName) ) {
			List<UserRole> list = new ArrayList<UserRole>();
			UserRole obj = null;
			for ( Integer fId : fromId ) {
				for ( Integer tId : targetId ) {
					obj = new UserRole();
					obj.setUser_id(tId);
					obj.setRole_id(fId);
					list.add(obj);
				}
			}
			return list;
		} else if ( RightSetEnum.METHOD_UPDATE_ROLE_MENU_FOR_ROLE.toString().equals(actionName) ) {
			List<RoleMenu> list = new ArrayList<RoleMenu>();
			RoleMenu obj = null;
			for ( Integer fId : fromId ) {
				for ( Integer tId : targetId ) {
					obj = new RoleMenu();
					obj.setRole_id(fId);
					obj.setMenu_id(tId);
					list.add(obj);
				}
			}
			return list;
		} else if ( RightSetEnum.METHOD_UPDATE_ROLE_MENU_FOR_MENU.toString().equals(actionName) ) {
			List<RoleMenu> list = new ArrayList<RoleMenu>();
			RoleMenu obj = null;
			for ( Integer fId : fromId ) {
				for ( Integer tId : targetId ) {
					obj = new RoleMenu();
					obj.setRole_id(tId);
					obj.setMenu_id(fId);
					list.add(obj);
				}
			}
			return list;
		} else if ( RightSetEnum.METHOD_UPDATE_MENU_FUNCTION_FOR_MENU.toString().equals(actionName) ) {
			List<MenuFunction> list = new ArrayList<MenuFunction>();
			MenuFunction obj = null;
			for ( Integer fId : fromId ) {
				for ( Integer tId : targetId ) {
					obj = new MenuFunction();
					obj.setMenu_id(fId);
					obj.setFunction_id(tId);
					list.add(obj);
				}
			}
			return list;
		} else if ( RightSetEnum.METHOD_UPDATE_MENU_FUNCTION_FOR_FUNCTION.toString().equals(actionName) ) {
			List<MenuFunction> list = new ArrayList<MenuFunction>();
			MenuFunction obj = null;
			for ( Integer fId : fromId ) {
				for ( Integer tId : targetId ) {
					obj = new MenuFunction();
					obj.setMenu_id(tId);
					obj.setFunction_id(fId);
					list.add(obj);
				}
			}
			return list;
		} 
		return null;
	}
	
	/**
	 * Common update method for no-tree function
	 * @param request
	 */
	protected void commonUpdate(HttpServletRequest request, HttpServletResponse response, 
			String deleteActionName, String updateActionName, String url) {
		logger.debug("RightSetAction:commonUpdate", null);
		List<Integer> fromId = this.getId(request, RightSetEnum.PARAM_FROM_ID.toString());
		List<Integer> targetId = this.getId(request, RightSetEnum.PARAM_TARGET_ID.toString());
		if ( !fromId.isEmpty() ) {
			Connection conn = this.getDefaultConnection(request, response, url);
			if ( conn != null ) {
				this.deleteRelation(fromId, conn, deleteActionName, request, response, url);
				if ( !targetId.isEmpty() ) {
					this.addRelation(fromId, targetId, conn, updateActionName, request, response, url);
				}
				ActionControler.getInstance().init();
			}
		}
		this.redirect(request, response, url);
	}
	
	/**
	 * Delete relation
	 * @param fromId
	 * @param conn
	 * @param deleteActionName
	 * @param response
	 * @param url
	 */
	protected void deleteRelation(List<Integer> fromId, Connection conn, 
			String deleteActionName, HttpServletRequest request, HttpServletResponse response, String url){
		StringBuilder sb = new StringBuilder();
		for ( Integer fId : fromId ) {
			sb.append(fId).append(",");
		}
		String idStr = sb.toString();
		if ( "".equals(idStr) ){
			idStr = "-1";
		} else {
			idStr = idStr.substring(0, idStr.length()-1);
		}
		IVoidAction dAction = PersistenceFactory.getInstance().getVoidAction();
		Map<String, Object> c = new HashMap<String, Object>();
		c.put(RightSetEnum.PARAM_ID_ARRAY.toString(), idStr);
		try {
			DefaultBaseAction.commitConnection(conn, false);
			dAction.voidAction(deleteActionName, c, conn, false);
		} catch (ExecuteSQLException e) {
			logger.error("There is an error on deleting object.", e);
			this.errorRedirect(request, response, url, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_1"));
			return;
		}
	}
	
	/**
	 * Add relations
	 * @param fromId
	 * @param targetId
	 * @param conn
	 * @param updateActionName
	 * @param response
	 * @param url
	 */
	protected void addRelation(List<Integer> fromId, List<Integer> targetId, Connection conn, 
			String updateActionName, HttpServletRequest request, HttpServletResponse response, String url){
		List<?> list = this.getInsertObjects(updateActionName,fromId,targetId);
		if ( list == null ) {
			try {
				conn.rollback();
				conn.close();
			} catch (SQLException e) {
				logger.error("The connection can not rollback.", e);
				this.errorRedirect(request, response, url, 
						LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_2"));
				return;
			}
			return;
		}
		IOrmAction iAction = (IOrmAction) PersistenceFactory.getInstance().getOrmAction();
		try {
			iAction.batchInsert(list, conn);
			DefaultBaseAction.commitConnection(conn, true);
		} catch (ExecuteSQLException e) {
			logger.error("There is an error on batch inserting objects.", e);
			this.errorRedirect(request, response, url, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_3"));
			return;
		}
	}
	
	/**
	 * @param response
	 * @param url
	 */
	protected void redirect(HttpServletRequest request, HttpServletResponse response, String url){
		try {
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_4"));
			response.sendRedirect(url);
		} catch (IOException e) {
			logger.error("There is an error on redirecting url.", e);
			this.errorRedirect(request, response, ParameterEnum.URL_WELCOME.toString(), 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_5"));
			return;
		}
	}
	
	
		
	/**
	 * Common update method for menu
	 * @param request 
	 * @param response 
	 * @param deleteActionName 
	 * @param updateActionName 
	 * @param menuIdisFromId 
	 * @param onlyLeaf 
	 */
	protected void menuUpdate(HttpServletRequest request, HttpServletResponse response, 
			String deleteActionName, String updateActionName, String url, boolean menuIdisFromId, boolean onlyLeaf) {
		logger.debug("RightSetAction:treeUpdate", null);
		List<Integer> fromId = this.getId(request, RightSetEnum.PARAM_FROM_ID.toString());
		List<Integer> targetId = this.getId(request, RightSetEnum.PARAM_TARGET_ID.toString());
		
		if ( !fromId.isEmpty() ) {
			Connection conn = this.getDefaultConnection(request, response, url);
			if ( conn != null ) {
				if ( ActionControler.allMenuList != null ) {
					try {
						MenuTree mt = new MenuTree(ActionControler.allMenuList);
						if ( menuIdisFromId ) {
							if ( onlyLeaf ) {
								fromId = mt.getSelectedLeafNodesId(fromId);
							} else {
								fromId = mt.getSelectedNodesId(fromId);
							}
						} else {
							if ( onlyLeaf ) {
								targetId = mt.getSelectedLeafNodesId(targetId);
							} else {
								targetId = mt.getSelectedNodesId(targetId);
							}
						}
					} catch (CoreException e) {
						logger.error("MenuTree has an error.", e);
						this.errorRedirect(request, response, url, 
								LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_6"));
						return;
					}
				}
				
				this.deleteRelation(fromId, conn, deleteActionName, request, response, url);
					
				this.addRelation(fromId, targetId, conn, updateActionName, request, response, url);
				ActionControler.getInstance().init();
			}
		}
		
		this.redirect(request, response, url);
	}
	
	@Override
	public void updateUserRoleForUser(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:updateUserRoleForUser", null);
		this.commonUpdate(request, response, 
				RightSetEnum.DAO_DELETE_USER_ROLE_BY_USER_ID.toString(), 
				RightSetEnum.METHOD_UPDATE_USER_ROLE_FOR_USER.toString(), 
				UserEnum.URL_USERS.toString());
	}

	@Override
	public void updateUserRoleForRole(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:updateUserRoleForRole", null);
		this.commonUpdate(request, response, 
				RightSetEnum.DAO_DELETE_USER_ROLE_BY_ROLE_ID.toString(), 
				RightSetEnum.METHOD_UPDATE_USER_ROLE_FOR_ROLE.toString(), 
				RoleEnum.URL_ROLES.toString());
	}

	@Override
	public void updateRoleMenuForRole(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:updateRoleMenuForRole", null);
		this.menuUpdate(request, response, 
				RightSetEnum.DAO_DELETE_ROLE_MENU_BY_ROLE_ID.toString(), 
				RightSetEnum.METHOD_UPDATE_ROLE_MENU_FOR_ROLE.toString(), 
				RoleEnum.URL_ROLES.toString(),false,false);
	}

	@Override
	public void updateRoleMenuForMenu(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:updateRoleMenuForMenu", null);
		this.menuUpdate(request, response, 
				RightSetEnum.DAO_DELETE_ROLE_MENU_BY_MENU_ID.toString(), 
				RightSetEnum.METHOD_UPDATE_ROLE_MENU_FOR_MENU.toString(), 
				MenuEnum.URL_MENUS.toString(),true,true);
	}

	@Override
	public void updateMenuFunctionForMenu(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:updateRoleMenuForMenu", null);
		this.menuUpdate(request, response, 
				RightSetEnum.DAO_DELETE_MENU_FUNCTION_BY_MENU_ID.toString(), 
				RightSetEnum.METHOD_UPDATE_MENU_FUNCTION_FOR_MENU.toString(), 
				MenuEnum.URL_MENUS.toString(),true,true);
	}

	@Override
	public void updateMenuFunctionForFunction(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:updateMenuFunctionForFunction", null);
		this.menuUpdate(request, response, 
				RightSetEnum.DAO_DELETE_MENU_FUNCTION_BY_FUNCTION_ID.toString(), 
				RightSetEnum.METHOD_UPDATE_MENU_FUNCTION_FOR_FUNCTION.toString(), 
				FunctionEnum.URL_FUNCTIONS.toString(),false,false);
	}

	/**
	 * Init RightSetView object
	 * @param request
	 * @return
	 * @throws CoreException 
	 * @throws IOException 
	 */
	protected RightSetView initRightSetView(HttpServletRequest request)
			throws CoreException, IOException {
		String[] idArray = this.getArrayParameter(
				RightSetEnum.ATTR_ID.toString(), request);
		RightSetView rsv = (RightSetView)request.getAttribute(RightSetEnum.ATTR_RIGHT_SET_VIEW.toString());
		if ( rsv == null ) {
			throw new CoreException("RightSetView object is null.");
		}
		
		rsv.setFromObjectsId(array2str(idArray));
		rsv.setFromObjectsName(request.getParameter(RightSetEnum.PARAM_NAME.toString()));
		
		logger.debug("id : " + rsv.getFromObjectsId(), null);
		logger.debug("name : " + rsv.getFromObjectsName(), null);
		
		return rsv;
	}
	
	/**
	 * Find target object by id
	 * @param fromObjectsId
	 * @param actionName
	 * @return
	 * @throws ExecuteSQLException 
	 */
	protected List<Integer> findTargetObjectsId(String fromObjectsId, String actionName) 
			throws ExecuteSQLException{
		IListValueAction sAction = PersistenceFactory.getInstance().getListValueAction();
		Map<String, Object> c = new HashMap<String, Object>();
		String[] idArray = fromObjectsId.split(",");
		String[] idKeys = new String[idArray.length];
		StringBuilder condtionId = new StringBuilder();
		for ( int i = 0 ; i < idArray.length ; i++ ) {
			idKeys[i] = "#"+idArray[i]+"#";
			condtionId.append(idKeys[i]);
			if ( i < idArray.length - 1 ) {
				condtionId.append(",");
			}
			c.put(idKeys[i], idArray[i]);
		}
		c.put(RightSetEnum.PARAM_CONDITION_ID.toString(), condtionId.toString());
		logger.debug("findTargetObject:actionName:"+actionName, null);
		return sAction.intListAction(actionName, c);
	}
	
	@Override
	public void findUserRoleByUserId(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:findUserRoleByUserId", null);
		try {
			RightSetView rsv = this.initRightSetView(request);
			rsv.setActionName(RightSetEnum.FUNC_UPDATE_USER_ROLE_FOR_USER.toString());		
			List<Integer> selectedObject = this.findTargetObjectsId(
					rsv.getFromObjectsId(), RightSetEnum.DAO_FIND_ROLES_BY_USER_ID.toString());
			
			@SuppressWarnings("unchecked")
			IPojoAction<Role> pAction = (IPojoAction<Role>) PersistenceFactory.getInstance().getPojoAction();
			List<Role> roles = pAction.pojoListAction(RightSetEnum.DAO_FIND_ALL_ROLES.toString(),null);
			if ( roles == null || roles.isEmpty() ) {
				rsv.setTreeNodes("[]");
			} else {
				rsv.setTreeNodes(this.getRolesJSON(roles, selectedObject));
			}
			
		} catch (IOException | ExecuteSQLException e) {
			logger.error("Finding roles by user's id has wrong.", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_7"));
		}
	}

	@Override
	public void findUserRoleByRoleId(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:findUserRoleByRoleId", null);
		try {
			RightSetView rsv = this.initRightSetView(request);
			rsv.setActionName(RightSetEnum.FUNC_UPDATE_USER_ROLE_FOR_ROLE.toString());		
			List<Integer> selectedObject = this.findTargetObjectsId(
					rsv.getFromObjectsId(), RightSetEnum.DAO_FIND_USERS_BY_ROLE_ID.toString());
			
			@SuppressWarnings("unchecked")
			IPojoAction<User> pAction = (IPojoAction<User>) PersistenceFactory.getInstance().getPojoAction();
			List<User> users = pAction.pojoListAction(RightSetEnum.DAO_FIND_ALL_USERS.toString(),null);
			if ( users == null || users.isEmpty() ) {
				rsv.setTreeNodes("[]");
			} else {
				rsv.setTreeNodes(this.getUsersJSON(users, selectedObject));
			}
			
		} catch (IOException | ExecuteSQLException e) {
			logger.error("Finding users by role's id has wrong.", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_7"));
		}
	}

	@Override
	public void findRoleMenuByRoleId(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:findRoleMenuByRoleId", null);
		try {
			RightSetView rsv = this.initRightSetView(request);
			rsv.setActionName(RightSetEnum.FUNC_UPDATE_ROLE_MENU_FOR_ROLE.toString());		
			List<Integer> selectedObject = this.findTargetObjectsId(
					rsv.getFromObjectsId(), RightSetEnum.DAO_FIND_MENUS_BY_ROLE_ID.toString());
			
			@SuppressWarnings("unchecked")
			IPojoAction<Menu> pAction = (IPojoAction<Menu>) PersistenceFactory.getInstance().getPojoAction();
			List<Menu> objs = pAction.pojoListAction(RightSetEnum.DAO_FIND_ALL_MENUS.toString(),null);
			if ( objs == null || objs.isEmpty() ) {
				rsv.setTreeNodes("[]");
			} else {
				MenuTree mt = new MenuTree(objs);
				rsv.setTreeNodes(mt.getSelectedJSON(request.getLocale(), selectedObject));
			}
			
		} catch (IOException | ExecuteSQLException e) {
			logger.error("Finding menu by role's id has wrong.", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_7"));
		}
	}

	@Override
	public void findRoleMenuByMenuId(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:findRoleMenuByMenuId", null);
		try {
			RightSetView rsv = this.initRightSetView(request);
			rsv.setActionName(RightSetEnum.FUNC_UPDATE_ROLE_MENU_FOR_MENU.toString());		
			List<Integer> selectedObject = this.findTargetObjectsId(
					rsv.getFromObjectsId(), RightSetEnum.DAO_FIND_ROLES_BY_MENU_ID.toString());
			
			@SuppressWarnings("unchecked")
			IPojoAction<Role> pAction = (IPojoAction<Role>) PersistenceFactory.getInstance().getPojoAction();
			List<Role> objs = pAction.pojoListAction(RightSetEnum.DAO_FIND_ALL_ROLES.toString(),null);
			if ( objs == null || objs.isEmpty() ) {
				rsv.setTreeNodes("[]");
			} else {
				rsv.setTreeNodes(this.getRolesJSON(objs, selectedObject));
			}
			
		} catch (IOException | ExecuteSQLException e) {
			logger.error("Finding roles by menu's id has wrong.", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_7"));
		}
	}

	@Override
	public void findMenuFunctionByMenuId(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:findMenuFunctionByMenuId", null);
		try {
			RightSetView rsv = this.initRightSetView(request);
			rsv.setActionName(RightSetEnum.FUNC_UPDATE_MENU_FUNCTION_FOR_MENU.toString());		
			List<Integer> selectedObject = this.findTargetObjectsId(
					rsv.getFromObjectsId(), RightSetEnum.DAO_FIND_FUNCTIONS_BY_MENU_ID.toString());
			
			@SuppressWarnings("unchecked")
			IPojoAction<Function> pAction = (IPojoAction<Function>) PersistenceFactory.getInstance().getPojoAction();
			List<Function> objs = pAction.pojoListAction(RightSetEnum.DAO_FIND_ALL_FUNCTIONS.toString(),null);
			if ( objs == null || objs.isEmpty() ) {
				rsv.setTreeNodes("[]");
			} else {
				rsv.setTreeNodes(this.getFunctionJSON(objs, selectedObject));
			}
			
		} catch (IOException | ExecuteSQLException e) {
			logger.error("Finding functions by menu's id has wrong.", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_7"));
		}
	}

	@Override
	public void findMenuFunctionByFunctionId(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		logger.debug("RightSetAction:findMenuFunctionByFunctionId", null);
		try {
			RightSetView rsv = this.initRightSetView(request);
			rsv.setActionName(RightSetEnum.FUNC_UPDATE_MENU_FUNCTION_FOR_FUNCTION.toString());		
			List<Integer> selectedObject = this.findTargetObjectsId(
					rsv.getFromObjectsId(), RightSetEnum.DAO_FIND_MENUS_BY_FUNCTION_ID.toString());
			
			@SuppressWarnings("unchecked")
			IPojoAction<Menu> pAction = (IPojoAction<Menu>) PersistenceFactory.getInstance().getPojoAction();
			List<Menu> objs = pAction.pojoListAction(RightSetEnum.DAO_FIND_ALL_MENUS.toString(),null);
			if ( objs == null || objs.isEmpty() ) {
				rsv.setTreeNodes("[]");
			} else {
				MenuTree mt = new MenuTree(objs);
				rsv.setTreeNodes(mt.getSelectedJSON(request.getLocale(),selectedObject));
			}
			
		} catch (IOException | ExecuteSQLException e) {
			logger.error("Finding functions by menu's id has wrong.", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetAction_info_7"));
		}
	}

	/**
	 * Get RightSetView object
	 * @param request
	 * @return
	 * @throws CoreException
	 */
	public static final RightSetView getRightSetView(HttpServletRequest request) 
			throws CoreException{
		RightSetView rsv = new RightSetView();
		String actionName = request.getParameter(ParameterEnum.ACTION.toString());
		String usr = LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetEnum_info_1");
		String role = LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetEnum_info_2");
		String menu = LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetEnum_info_3");
		String func = LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.rightset.RightSetEnum_info_4");
		
		if ( RightSetEnum.FUNC_FIND_MENU_FUNCTION_BY_FUNCTION_ID.toString().equals(actionName) ) {
			rsv.setSourceUrl(getSourceUrl("functions.jsp","page_sys_functions_title",request));
			rsv.setFromObjectTypeName(func);
			rsv.setTargetObjectTypeName(menu);
		} else if ( RightSetEnum.FUNC_FIND_MENU_FUNCTION_BY_MENU_ID.toString().equals(actionName) ) {
			rsv.setSourceUrl(getSourceUrl("menus.jsp","page_sys_menus_title",request));
			rsv.setFromObjectTypeName(menu);
			rsv.setTargetObjectTypeName(func);
		} else if ( RightSetEnum.FUNC_FIND_ROLE_MENU_BY_MENU_ID.toString().equals(actionName) ) {
			rsv.setSourceUrl(getSourceUrl("menus.jsp","page_sys_menus_title",request));
			rsv.setFromObjectTypeName(menu);
			rsv.setTargetObjectTypeName(role);
		} else if ( RightSetEnum.FUNC_FIND_ROLE_MENU_BY_ROLE_ID.toString().equals(actionName) ) {
			rsv.setSourceUrl(getSourceUrl("roles.jsp","page_sys_roles_label_1",request));
			rsv.setFromObjectTypeName(role);
			rsv.setTargetObjectTypeName(menu);
		} else if ( RightSetEnum.FUNC_FIND_USER_ROLE_BY_ROLE_ID.toString().equals(actionName) ) {
			rsv.setSourceUrl(getSourceUrl("roles.jsp","page_sys_roles_label_1",request));
			rsv.setFromObjectTypeName(role);
			rsv.setTargetObjectTypeName(usr);
		} else if ( RightSetEnum.FUNC_FIND_USER_ROLE_BY_USER_ID.toString().equals(actionName) ) {
			rsv.setSourceUrl(getSourceUrl("users.jsp","page_sys_users_label_1",request));
			rsv.setFromObjectTypeName(usr);
			rsv.setTargetObjectTypeName(role);
		} else {
			throw new CoreException("The action " + actionName + " is not exist.");
		} 
		request.setAttribute(ParameterEnum.ACTION.toString(), actionName);
		request.setAttribute(RightSetEnum.ATTR_RIGHT_SET_VIEW.toString(), rsv);//通过request的attribute属性与后台进行数据交换
		return rsv;
	}
	
	/**
	 * Get url of source
	 * @param url
	 * @param name
	 * @param request
	 * @return
	 */
	public static String getSourceUrl(String url, String name, HttpServletRequest request) {
		StringBuilder sb = new StringBuilder();
		sb.append("<a href=\"").append(url).append("\">")
		  .append(LocaleToolkit.getLocaleString(request, name))
		  .append("</a>&nbsp;&nbsp;>&nbsp;&nbsp;");
		return sb.toString();
	}
	
	/**
	 * Get json of Function objects
	 * @return
	 * @throws CoreException
	 */
	protected String getFunctionJSON(List<Function> l, List<Integer> selectedObject) throws CoreException{
		if ( l != null && !l.isEmpty() ) {
			StringBuilder sb = new StringBuilder();
			sb.append("[");
			int lastIndex = l.size()-1;
			Function obj = null;
			for ( int i = 0 ; i < l.size() ; i++ ) {
				obj = l.get(i);
				sb.append("{")
				  .append("id:").append(obj.getId()).append(",")
				  .append("pId:").append(0).append(",")
				  .append("name:\"").append(obj.getName());
				if ( !PlatformTools.isNull(obj.getMethod_name()) ) {
				  sb.append(" -> ").append(obj.getMethod_name());
				}
				if ( !PlatformTools.isNull(obj.getPage_url()) ) {
				  sb.append(" -> ").append(obj.getPage_url());
				}
				sb.append("\"");
				if ( selectedObject.contains(obj.getId()) ) {
					sb.append(",checked:true");
				}
				sb.append("}");
				if ( i < lastIndex ) {
					sb.append(",");
				}
			}
			sb.append("]");
			return sb.toString();
		}
		return "[]";
	}
	
	/**
	 * Get json of Role objects
	 * @return
	 * @throws CoreException
	 */
	protected String getRolesJSON(List<Role> l, List<Integer> selectedObject) throws CoreException{
		if ( l != null && !l.isEmpty() ) {
			StringBuilder sb = new StringBuilder();
			sb.append("[");
			int lastIndex = l.size()-1;
			Role obj = null;
			for ( int i = 0 ; i < l.size() ; i++ ) {
				obj = l.get(i);
				sb.append("{")
				  .append("id:").append(obj.getId()).append(",")
				  .append("pId:").append(0).append(",")
				  .append("name:\"").append(obj.getName()).append("\"");
				if ( selectedObject.contains(obj.getId()) ) {
					sb.append(",checked:true");
				}
				sb.append("}");
				if ( i < lastIndex ) {
					sb.append(",");
				}
			}
			sb.append("]");
			return sb.toString();
		}
		return "[]";
	}
	
	/**
	 * Get json of User objects
	 * @return
	 * @throws CoreException
	 */
	protected String getUsersJSON(List<User> l, List<Integer> selectedObject) throws CoreException{
		if ( l != null && !l.isEmpty() ) {
			StringBuilder sb = new StringBuilder();
			sb.append("[");
			int lastIndex = l.size()-1;
			User obj = null;
			for ( int i = 0 ; i < l.size() ; i++ ) {
				obj = l.get(i);
				sb.append("{")
				  .append("id:").append(obj.getId()).append(",")
				  .append("pId:").append(0).append(",")
				  .append("name:\"").append(obj.getLogin_name()).append("\"");
				if ( selectedObject.contains(obj.getId()) ) {
					sb.append(",checked:true");
				}
				sb.append("}");
				if ( i < lastIndex ) {
					sb.append(",");
				}
			}
			sb.append("]");
			return sb.toString();
		}
		return "[]";
	}

	@Override
	public Map<String,Integer> getUrl2RoleIdRelation() throws CoreException {
		logger.debug("RightSetAction:getUrl2RoleIdRelation", null);
		try {
			IMapAction mAction = (IMapAction) PersistenceFactory.getInstance().getMapAction();
			List<Map<String,Object>> list = mAction.mapListAction(RightSetEnum.DAO_GET_URL_2_ROLE_ID_RELATION.toString(),null);
			
			Map<String,Integer> iMap = new HashMap<String,Integer>();
			if ( list == null ) {
				return null;
			}
			Object url = null;
			String urlStr = null;
			Object roleId = null;
			for ( Map<String,Object> map : list ) {
				if ( CommonTools.isNull(map) ){
					continue;
				}
				url = map.get(RightSetEnum.PARAM_URL.toString());
				roleId = map.get(RightSetEnum.PARAM_ROLE_ID.toString());
				if ( CommonTools.isNull(url) || CommonTools.isNull(roleId) || 
						!CommonTools.isIntegerNumber(roleId) ) {
					continue;
				}
				int index = url.toString().indexOf("/");
				if ( index > -1 ) {
					urlStr = url.toString().substring(index);
					logger.debug("Url2RoleId:url >> "+urlStr, null);
					iMap.put(urlStr + "_" + roleId, (Integer)roleId);
				}
			}
			
			logger.debug("Url2RoleId:"+iMap.size(), null);
			return iMap;
		} catch (ExecuteSQLException e) {
			logger.error("To get relation of url2RoleId has error.", e);
		}
		return null;
	}

	@Override
	public Map<String,Integer> getActionParameter2RoleIdRelation()
			throws CoreException {
		logger.debug("RightSetAction:getActionParameter2RoleIdRelation", null);
		try {
			IMapAction mAction = (IMapAction) PersistenceFactory.getInstance().getMapAction();
			Map<String,Object> map = mAction.mapAction(RightSetEnum.DAO_GET_ACTION_PARAMETER_2_ROLE_ID_RELATION.toString(),null);
			Map<String,Integer> iMap = new HashMap<String,Integer>();
			if ( map == null ) {
				return null;
			}
			Set<Entry<String,Object>> set = map.entrySet();
			for ( Entry<String,Object> en : set ) {
				if ( !CommonTools.isNull(en.getKey()) &&
						!CommonTools.isNull(en.getValue()) ) {
					if ( CommonTools.isIntegerNumber(en.getValue()) ) {
						iMap.put(en.getKey(), (Integer)en.getValue());
					}
				}
			}
			logger.debug("ActionParameter2RoleId:"+iMap.size(), null);
			return iMap;
		} catch (ExecuteSQLException e) {
			logger.error("To get relation of actionParameter2RoleId has error.", e);
		}
		return null;
	}

	@Override
	public void getFunctionPageUrlForMenuItem(
			HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		logger.debug("RightSetAction:getFunctionPageUrlForMenuItem", null);
		try {
			Object obj = request.getAttribute(RightSetEnum.ATTR_MENU_ID.toString());
			if ( PlatformTools.isNull(obj) ) {
				request.setAttribute(RightSetEnum.ATTR_FUNCTION_PAGE_URL.toString(), null);
				return;
			}
			Map<String,Object> c = new HashMap<String,Object>();
			c.put(RightSetEnum.ATTR_ID.toString(), PlatformTools.toInt(obj));
			
			IListValueAction action = (IListValueAction) PersistenceFactory.getInstance().getListValueAction();
			List<String> l = action.stringListAction(RightSetEnum.DAO_GET_FUNCTION_PAGE_URL_FOR_MENU_ITEM.toString(), c);
			if ( PlatformTools.isNull(l) && l.isEmpty() ) {
				request.setAttribute(RightSetEnum.ATTR_FUNCTION_PAGE_URL.toString(), null);
				return;
			}
			request.setAttribute(RightSetEnum.ATTR_FUNCTION_PAGE_URL.toString(), l);
			
		} catch (ExecuteSQLException e) {
			logger.error("The method getFunctionPageUrlForMenuItem is error.", e);
		}
	}
	
}
