/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.user;

import javax.servlet.http.HttpServletRequest;

import com.nonesole.web.platform.basic.IPojo;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * User POJO
 * @author jack lee
 * @version 1.0
 * @date 2016-09-25
 */
public class User implements IPojo{

	private int id;
	private String login_name;
	private String login_email;
	private String login_cell_phone_number;
	private String login_key;
	private int status;
	
	/**
	 * @return the login_name
	 */
	public String getLogin_name() {
		return login_name;
	}
	/**
	 * @param login_name the login_name to set
	 */
	public void setLogin_name(String login_name) {
		this.login_name = login_name;
	}
	/**
	 * @return the login_email
	 */
	public String getLogin_email() {
		return login_email;
	}
	/**
	 * @param login_email the login_email to set
	 */
	public void setLogin_email(String login_email) {
		this.login_email = login_email;
	}
	/**
	 * @return the login_cell_phone_number
	 */
	public String getLogin_cell_phone_number() {
		return login_cell_phone_number;
	}
	/**
	 * @param login_cell_phone_number the login_cell_phone_number to set
	 */
	public void setLogin_cell_phone_number(String login_cell_phone_number) {
		this.login_cell_phone_number = login_cell_phone_number;
	}
	/**
	 * @return the login_key
	 */
	public String getLogin_key() {
		return login_key;
	}
	/**
	 * @param login_key the login_key to set
	 */
	public void setLogin_key(String login_key) {
		this.login_key = login_key;
	}
	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}
	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}
	/**
	 * @return the status
	 */
	public int getStatus() {
		return status;
	}
	/**
	 * @param status the status to set
	 */
	public void setStatus(int status) {
		this.status = status;
	}
	
	/**
	 * Get user status
	 * @return String
	 */
	public String getUserStatus(HttpServletRequest request){
		String s = LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.User_info_1");
		switch(this.status){
			case 1 : s = LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.User_info_2");break;
			case 2 : s = LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.User_info_3");break;
		}
		return s;
	}
	
	/**
	 * @return String
	 */
	public String toString(){
		return String.valueOf(id);
	}
	
	/**
	 * @return
	 */
	public boolean validate(int action){
		if ( action == INSERT ) {
			//不对ID做检查
		} else if ( action == UPDATE || action == CHECK) {
			//对ID做检查
			if ( this.id < 1 ) {
				return false;
			}
		}
		
		if ( !PlatformTools.validateUserName(this.login_name) ) {
			if ( !PlatformTools.isEmail(this.login_email) ) {
				if ( !PlatformTools.isCellPhoneNumber(this.login_cell_phone_number) ) {
					return false;
				}
			} 
		} 
		
		return true;
	}
	
	public static final int INSERT = 1;
	public static final int UPDATE = 2;
	public static final int CHECK = 3;

	@Override
	public int getObjectId() {
		return id;
	}
}
