/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.user;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IOrmAction;
import com.nonesole.persistence2.sql.define.IOrmPlusAction;
import com.nonesole.persistence2.sql.define.IPojoAction;
import com.nonesole.persistence2.sql.define.ISingleValueAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.ActionControler;
import com.nonesole.web.platform.basic.BasicAction;
import com.nonesole.web.platform.basic.ICommon;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.pages.Page2Condtion;
import com.nonesole.web.platform.pages.Page2Parameter;
import com.nonesole.web.platform.pages.Pages2;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;
import com.nonesole.web.platform.sys.rightset.RightSetEnum;
import com.nonesole.web.platform.sys.userdetail.UserDetail;

/**
 * User action
 * @author jack lee
 * @version 1.0
 * @date 2016-09-13
 */
public class UserAction extends BasicAction implements IUser {

	public UserAction(){
		super();
	}
	
	@Override
	public void add(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		try {
			Map<String,Object> c = this.parameter2map(UserEnum.DAO_FIND_SAME_USER.toString(), request);
			ISingleValueAction sAction = PersistenceFactory.getInstance().getSingleValueAction();
			Integer row = sAction.intAction(UserEnum.DAO_FIND_SAME_USER.toString(), c);
			if ( row != null && row > 0 ) {
				PlatformTools.setCookieMessage(response, 
						LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_1"));
				return;
			}
			
			User u = new User();
			this.parameter2orm(u, request);
			if ( !u.validate(User.INSERT) ) {
				PlatformTools.setCookieMessage(response, "用户添加失败！");
				throw new CoreException("User object has wrong fields.Please check it.");
			}
			u.setLogin_key(UserEnum.PARAM_DEFAULT_LOGIN_KEY.toString());
			u.setLogin_key(PlatformTools.md5(u.getLogin_key()));
			IOrmAction action = PersistenceFactory.getInstance().getOrmAction();
			action.insert(u);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_3"));
		} catch (ExecuteSQLException | ClassNotFoundException | OperationsException e) {
			logger.error("UserAction:add", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_2"));
		} finally {
			this.redirect(request, response, UserEnum.URL_USERS.toString());			
		}
		
	}

	@Override
	public void update(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		try {
			Map<String,Object> c = this.parameter2map(UserEnum.DAO_FIND_SAME_USER_FOR_UPDATE.toString(), request);
			ISingleValueAction sAction = PersistenceFactory.getInstance().getSingleValueAction();
			Integer row = sAction.intAction(UserEnum.DAO_FIND_SAME_USER_FOR_UPDATE.toString(), c);
			if ( row != null && row > 0 ) {
				PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_1"));
				return;
			}
			User u = new User();
			this.parameter2orm(u, request);
			IOrmPlusAction ormPlus = PersistenceFactory.getInstance().getOrmPlusAction();
			ormPlus.doAction(UserEnum.UPDATE_USER_METHOD.toString(), u);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_4"));
		} catch (ClassNotFoundException | OperationsException | ExecuteSQLException e) {
			logger.error("[UserAction:update]", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_5"));
		} finally {
			this.redirect(request, response, UserEnum.URL_USERS.toString());
		}
		
	}
	
	/* (non-Javadoc)
	 * @see com.nonesole.web.platform.IBaseAction#query(javax.servlet.http.HttpSession, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	public void query(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Page2Parameter loginName = this.createKeyWordParam(request, DAO_PARAM_LOGIN_NAME);
		Page2Parameter email = this.createKeyWordParam(request, DAO_PARAM_EMAIL);
		Page2Parameter cellphone = this.createKeyWordParam(request, DAO_PARAM_CELL_PHONE);
		Page2Parameter status = this.createStatusParam(request);
		Page2Condtion condition = new UserPage2Condtion();
		condition.setDaoConditionKey(ICommon.DAO_SQL_CONDITION)
				.add(loginName)
				.add(email)
				.add(cellphone)
				.add(status);
		Pages2<User> page = new Pages2<User>()
				.setpCondition(condition)
				.setRequest(request)
				.setUrl(PAGE_QUERY)
				.setTotalRowsDaoName(DAO_METHOD_GET_TOTAL_ROWS)
				.setResultsDaoName(DAO_METHOD_GET_RESULTS)
				.initAll();
		request.setAttribute(ICommon.ATTR_PAGES, page);
	}

	@Override
	public void delete(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		String parameter = UserEnum.PARAM_USER_ID.toString();
		String sqlPart = RightSetEnum.PARAM_CONDITION_ID.toString();
		String url =  UserEnum.URL_USERS.toString();
		this.deleteByIdArray(parameter, User.class, request, response);
		this.deleteByIdArray(parameter, UserDetail.class, request, response);
		//删除User与Role关联表中的数据
		this.deleteByIdArray(parameter, 
				RightSetEnum.DAO_DELETE_USER_ROLE_BY_USER_ID.toString(), 
				RightSetEnum.PARAM_USER_ID.toString(), 
				sqlPart, request, response);
		ActionControler.getInstance().init();
		PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_8"));
		this.redirect(request, response, url);
		
	}

	@Override
	public void lock(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		this.setStatus(session, request, response, UserEnum.LOCK_METHOD.toString());
	}

	@Override
	public void unlock(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		this.setStatus(session, request, response, UserEnum.UNLOCK_METHOD.toString());
	}
	
	/**
	 * Set status of user
	 * @param userIdArray
	 * @param status
	 * @param request
	 * @param response
	 */
	private void setGlobalUserStatus(String parameter, String status, 
			HttpServletRequest request, HttpServletResponse response){
		String[] userIdArray = request.getParameterValues(UserEnum.PARAM_USER_ID.toString());
		if ( userIdArray == null || userIdArray.length < 1 ) {
			return;
		}
		User u = this.getLoginUser(request);
		if ( u == null ) {
			return;
		}
		String info = null;
		boolean lock = UserEnum.LOCK_METHOD.toString().equals(status);
		ActionControler ctrl = ActionControler.getInstance();
		for ( String idStr : userIdArray ) {
			int id = Integer.valueOf(idStr);
			if ( lock ) {
				if ( u.getId() != id ) {
					ctrl.addLockedUser(id);
				}
			} else {
				ctrl.removeLockedUser(id);
			}
		}
		if ( lock ) {
			info = "com.nonesole.web.platform.sys.user.UserAction_info_10";
		} else {
			info = "com.nonesole.web.platform.sys.user.UserAction_info_12";
		}
		PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, info));
	}
	
	/**
	 * Check user array
	 * @param request
	 * @return
	 */
	private String checkUserArray(HttpServletRequest request) {
		String[] idArray = request.getParameterValues(UserEnum.PARAM_USER_ID.toString());
		if ( idArray == null ) return null;
		User u = this.getLoginUser(request);
		if ( u == null ) return null;
		
		List<String> l = new ArrayList<String>();
		for ( String id : idArray ) {
			if ( u.getId() == Integer.valueOf(id) ) {
				continue;
			}
			l.add(id);
		}
		
		String[] result = new String[l.size()];
		l.toArray(result);
		return this.array2string(result);
	}
	
	/**
	 * Set user's status
	 * @param session
	 * @param request
	 * @param response
	 * @param status
	 * @throws CoreException
	 */
	private void setStatus(HttpSession session, HttpServletRequest request,
			HttpServletResponse response, String status) throws CoreException {
		try {
			String idString = this.checkUserArray(request);
			if ( idString != null )  {
				Map<String, Object> c = new HashMap<String, Object>();
				c.put(UserEnum.PARAM_USER_ID.toString(), idString);
				Connection conn =this.getDefaultConnection(request, response, UserEnum.URL_USERS.toString());
				PersistenceFactory.getInstance().getVoidAction().voidAction(status, c,  conn, false);
				this.setGlobalUserStatus(UserEnum.PARAM_USER_ID.toString(), status, request, response);
			}
		} catch (ExecuteSQLException e) {
			if ( UserEnum.LOCK_METHOD.toString().equals(status) ) {
				PlatformTools.setCookieMessage(response, 
						LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_11"));
			} else {
				PlatformTools.setCookieMessage(response, 
						LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_13"));
			}
			logger.error("[UserAction:setStatus]", e);
		} finally {
			this.redirect(request, response, UserEnum.URL_USERS.toString());
		}
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public void getUserDetail(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		IPojoAction<User> acts = (IPojoAction<User>) PersistenceFactory.getInstance().getPojoAction();
		Map<String,Object> c = new HashMap<String,Object>();
		c.put(UserEnum.PARAM_USER_ID.toString(), 
				PlatformTools.toInt(request.getParameter(UserEnum.PARAM_USER_ID.toString())));
		User u = null;
		try {
			u = acts.pojoAction(UserEnum.GET_USER_DETAILS.toString(), c);
		} catch (ExecuteSQLException e) {
			logger.error("UserAction:getUserDetail", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_14"));
		} 
		request.setAttribute(UserEnum.ATTR_USER_DETAIL.toString(), u);
	}

	/**
	 * Get Page2Condition
	 * @author jack lee
	 * @version 1.0
	 */
	class UserPage2Condtion extends Page2Condtion {
		public UserPage2Condtion(){
			super();
		}
		@Override
		public Page2Condtion createPreparedSqlCondition() {
			if (!PlatformTools.isNull(this.getUrlCondition())
					&& this.isPrepared) {
				StringBuilder sb = new StringBuilder();
				sb.append(" where ");
				String keyword = this.getKeyWordCondition();
				String status = this.getStatusCondition();
				
				if ( !PlatformTools.isNull(keyword) ) {
					sb.append(" (").append(keyword).append(") ");
					if ( !PlatformTools.isNull(status.toString()) ) {
						sb.append(" and ");
					}
				} 
				sb.append(status).append(" ");
				this.daoConditionValue = sb.toString();
			}
			return this;
		}
		private String getKeyWordCondition(){
			int index = 1;
			StringBuilder keyword = new StringBuilder();
			String tmp = null;
			for (Page2Parameter pp : this.list) {
				if ( ICommon.PARAM_QUERY_KEY_WORD.equals(pp.getKey()) ) {
					tmp = pp.getComplexPreparedSqlCondition();
					if ( !PlatformTools.isNull(tmp) ) {
						if ( index > 1 ) {
							keyword.append(" or ");
						}
						keyword.append(pp.getComplexPreparedSqlCondition());
						index++;
					}
				}
			}
			return keyword.toString();
		}
		private String getStatusCondition(){
			int index = 1;
			String tmp = null;
			StringBuilder status = new StringBuilder();
			for (Page2Parameter pp : this.list) {
				if ( !ICommon.PARAM_QUERY_KEY_WORD.equals(pp.getKey()) ) {
					tmp = pp.getComplexPreparedSqlCondition();
					if ( !PlatformTools.isNull(tmp) ) {
						if ( index > 1 ) {
							status.append(" and ");
						}
						status.append(pp.getComplexPreparedSqlCondition());
						index++;
					}
				}
			}
			return status.toString();
		}
	}

	@Override
	public void updatePwd(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		try {
			User u = new User();
			this.parameter2orm(u, request);
			u.setLogin_key(PlatformTools.md5(u.getLogin_key()));
			IOrmPlusAction ormPlus = PersistenceFactory.getInstance().getOrmPlusAction();
			ormPlus.doAction(UserEnum.UPDATE_PWD_METHOD.toString(), u);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_4"));
		} catch (ClassNotFoundException | OperationsException | ExecuteSQLException e) {
			logger.error("[UserAction:update]", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.user.UserAction_info_5"));
		} finally {
			this.redirect(request, response, UserEnum.URL_USERS.toString());
		}
	}
}
