/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.userdetail;

import java.text.ParseException;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IVoidAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.basic.BasicAction;
import com.nonesole.web.platform.basic.ICommon;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.pages.Page2Condtion;
import com.nonesole.web.platform.pages.Page2Parameter;
import com.nonesole.web.platform.pages.Pages2;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;

/**
 * UserDetail action
 * @author jack lee
 * @version 1.0
 * @date 2016-09-26
 */
public class UserDetailAction extends BasicAction implements IUserDetail{

	public UserDetailAction(){
		super();
	}
	
	@Override
	public void add(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
	}
	
	@Override
	public void update(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		try {
			Map<String,Object> c = this.parameter2map(UserDetailEnum.DAO_UPDATE_USER.toString(), request);
			//对时间进行转换
			Object obj = request.getParameter(UserDetailEnum.PARAM_STRING_BIRTHDAY.toString());
			long time = PlatformTools.string2time(obj==null?"":obj.toString(), UserDetailEnum.TIME_FORMAT.toString());
			c.put(UserDetailEnum.PARAM_BIRTHDAY.toString(), time);
			IVoidAction acts = PersistenceFactory.getInstance().getVoidAction();
			acts.voidAction(UserDetailEnum.DAO_UPDATE_USER.toString(), c);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.userdetail.UserDetailAction_info_1"));
		} catch (ExecuteSQLException | ParseException e) {
			logger.fatal("[UserDetailAction:update]", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.sys.userdetail.UserDetailAction_info_2"));
		} finally {
			this.redirect(request, response, UserDetailEnum.URL_USERS.toString());
		}
	}

	@Override
	public void query(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Page2Parameter loginName = this.createKeyWordParam(request, DAO_PARAM_LOGIN_NAME);
		Page2Parameter email = this.createKeyWordParam(request, DAO_PARAM_EMAIL);
		Page2Parameter nickName = this.createKeyWordParam(request, DAO_PARAM_NICK_NAME);
		Page2Parameter qq = this.createKeyWordParam(request, DAO_PARAM_QQ);
		Page2Parameter address = this.createKeyWordParam(request, DAO_PARAM_ADDRESS);
		Page2Parameter wechat = this.createKeyWordParam(request, DAO_PARAM_WECHAT);
		Page2Parameter cellphone = this.createKeyWordParam(request, DAO_PARAM_CELL_PHONE);
		Page2Parameter status = this.createStatusParam(request);
		
		Page2Condtion condition = new UserDetailPage2Condtion() ;
		condition.setDaoConditionKey(ICommon.DAO_SQL_CONDITION)
				.add(loginName)
				.add(email)
				.add(nickName)
				.add(qq)
				.add(address)
				.add(wechat)
				.add(cellphone)
				.add(status);

		Pages2<UserDetail> page = new Pages2<UserDetail>()
				.setpCondition(condition)
				.setRequest(request)
				.setUrl(PAGE_QUERY)
				.setTotalRowsDaoName(DAO_METHOD_GET_TOTAL_ROWS)
				.setResultsDaoName(DAO_METHOD_GET_RESULTS)
				.initAll();

		request.setAttribute(ICommon.ATTR_PAGES, page);
	}
	
	@Override
	public void delete(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
	}

	@Override
	public void findUserDetail(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		Object object = super.findObjectById(DAO_METHOD_FIND_OBJECT_BY_ID, request, response);
		request.setAttribute(ICommon.ATTR_OBJECT, object);
	}

	/**
	 * Get Page2Condition
	 * @author jack lee
	 * @version 1.0
	 */
	class UserDetailPage2Condtion extends Page2Condtion {
		public UserDetailPage2Condtion(){
			super();
		}
		@Override
		public Page2Condtion createPreparedSqlCondition() {
			if (!PlatformTools.isNull(this.getUrlCondition())
					&& this.isPrepared) {
				StringBuilder sb = new StringBuilder();
				sb.append(" where ");
				String keyword = this.getKeyWordCondition();
				String status = this.getStatusCondition();
				
				if ( !PlatformTools.isNull(keyword) ) {
					sb.append(" (").append(keyword).append(") ");
					if ( !PlatformTools.isNull(status.toString()) ) {
						sb.append(" and ");
					}
				} 
				sb.append(status).append(" ");
				this.daoConditionValue = sb.toString();
			}
			return this;
		}
		private String getKeyWordCondition(){
			int index = 1;
			StringBuilder keyword = new StringBuilder();
			String tmp = null;
			for (Page2Parameter pp : this.list) {
				if ( ICommon.PARAM_QUERY_KEY_WORD.equals(pp.getKey()) ) {
					tmp = pp.getComplexPreparedSqlCondition();
					if ( !PlatformTools.isNull(tmp) ) {
						if ( index > 1 ) {
							keyword.append(" or ");
						}
						keyword.append(pp.getComplexPreparedSqlCondition());
						index++;
					}
				}
			}
			return keyword.toString();
		}
		private String getStatusCondition(){
			int index = 1;
			String tmp = null;
			StringBuilder status = new StringBuilder();
			for (Page2Parameter pp : this.list) {
				if ( !ICommon.PARAM_QUERY_KEY_WORD.equals(pp.getKey()) ) {
					tmp = pp.getComplexPreparedSqlCondition();
					if ( !PlatformTools.isNull(tmp) ) {
						if ( index > 1 ) {
							status.append(" and ");
						}
						status.append(pp.getComplexPreparedSqlCondition());
						index++;
					}
				}
			}
			return status.toString();
		}
	}
}
